@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -12,6 +12,7 @@ import abc
 import ast
 import logging
 import os
+import string
 import typing
 
 from abc import ABC
@@ -27,6 +28,12 @@ from pynguin.utils.orderedset import OrderedSet
 from pynguin.utils.typetracing import unwrap
 
 
+if typing.TYPE_CHECKING:
+    from collections.abc import Callable
+    from typing import Any
+    from typing import ClassVar
+
+
 ConstantTypes = float | int | str | bytes | complex
 
 # Used for generic type hinting
@@ -39,7 +46,7 @@ logger = logging.getLogger(__name__)
 class ConstantPool:
     """A pool of constants for various types."""
 
-    def __init__(self):
+    def __init__(self):  # noqa: D107
         self._constants: dict[type[ConstantTypes], OrderedSet[ConstantTypes]] = {
             tp_: OrderedSet() for tp_ in typing.get_args(ConstantTypes)
         }
@@ -82,7 +89,7 @@ class ConstantPool:
         Returns:
             A random element of the given type
         """
-        return typing.cast(T, randomness.choice(tuple(self._constants[tp_])))
+        return typing.cast("T", randomness.choice(tuple(self._constants[tp_])))
 
     def get_all_constants_for(self, tp_: type[T]) -> OrderedSet[T]:
         """Get all values from the constant pool.
@@ -93,7 +100,7 @@ class ConstantPool:
         Returns:
             All constants of the given type
         """
-        return typing.cast(OrderedSet[T], self._constants[tp_])
+        return typing.cast("OrderedSet[T]", self._constants[tp_])
 
     def __len__(self):
         return sum(len(value) for value in self._constants.values())
@@ -101,28 +108,28 @@ class ConstantPool:
 
 class RestrictedConstantPool(ConstantPool):
     """A constant pool that is restricted in its size.
+
     If the size limit is reached, the oldest values are purged.
     """
 
     def __init__(self, max_size: int = 50):
-        """Create a new restricted constant pool with the given max number of constants
-        per collected type.
+        """Create a new restricted constant pool.
 
         Args:
-            max_size: The maximum number of collected values.
+            max_size: The maximum number of collected values per type.
         """
         super().__init__()
         assert max_size > 0, "Size limit for constant pool must be positive."
         self._max_size = max_size
 
-    def add_constant(self, constant: ConstantTypes) -> None:
+    def add_constant(self, constant: ConstantTypes) -> None:  # noqa: D102
         values = self._constants[type(constant)]
         values.add(constant)
         if len(values) > self._max_size:
             values.remove(values[0])
 
 
-class ConstantProvider(abc.ABC):  # pylint:disable=too-few-public-methods
+class ConstantProvider(abc.ABC):
     """Provides constants."""
 
     @abc.abstractmethod
@@ -137,21 +144,17 @@ class ConstantProvider(abc.ABC):  # pylint:disable=too-few-public-methods
         """
 
 
-class EmptyConstantProvider(ConstantProvider):  # pylint:disable=too-few-public-methods
+class EmptyConstantProvider(ConstantProvider):
     """Empty provider."""
 
-    def get_constant_for(self, tp_: type[T]) -> T | None:
+    def get_constant_for(self, tp_: type[T]) -> T | None:  # noqa: D102
         return None
 
 
-class DelegatingConstantProvider(
-    ConstantProvider, ABC
-):  # pylint:disable=too-few-public-methods
+class DelegatingConstantProvider(ConstantProvider, ABC):
     """Either provides values from its own pool or delegates to another provider."""
 
-    def __init__(
-        self, pool: ConstantPool, delegate: ConstantProvider, probability: float
-    ):
+    def __init__(self, pool: ConstantPool, delegate: ConstantProvider, probability: float):
         """Create a new provider.
 
         Args:
@@ -164,11 +167,8 @@ class DelegatingConstantProvider(
         self._delegate = delegate
         self._probability = probability
 
-    def get_constant_for(self, tp_: type[T]) -> T | None:
-        if (
-            self._pool.has_constant_for(tp_)
-            and randomness.next_float() < self._probability
-        ):
+    def get_constant_for(self, tp_: type[T]) -> T | None:  # noqa: D102
+        if self._pool.has_constant_for(tp_) and randomness.next_float() < self._probability:
             return self._pool.get_constant_for(tp_)
         return self._delegate.get_constant_for(tp_)
 
@@ -193,28 +193,24 @@ class DynamicConstantProvider(DelegatingConstantProvider):
             max_constant_length: The maximum length of strings to store.
         """
         super().__init__(pool, delegate, probability)
-        assert (
-            max_constant_length > 0
-        ), "Length limit for constant pool elements must be positive."
+        assert max_constant_length > 0, "Length limit for constant pool elements must be positive."
         self._max_constant_length = max_constant_length
 
     # A map containing the names of all string functions which are instrumented.
     # Maps those names to a function producing a value that negates the functions
     # result.
-    STRING_FUNCTION_LOOKUP = {
+    STRING_FUNCTION_LOOKUP: ClassVar[dict[str, Callable[[Any], str]]] = {
         "isalnum": lambda value: f"{value}!" if value.isalnum() else "isalnum",
         "islower": lambda value: value.upper() if value.islower() else value.lower(),
         "isupper": lambda value: value.lower() if value.isupper() else value.upper(),
-        "isdecimal": lambda value: "non_decimal" if value.isdecimal() else "0123456789",
+        "isdecimal": lambda value: ("non_decimal" if value.isdecimal() else string.digits),
         "isalpha": lambda value: f"{value}1" if value.isalpha() else "isalpha",
         "isdigit": lambda value: f"{value}_" if value.isdigit() else "0",
-        "isidentifier": lambda value: f"{value}!"
-        if value.isidentifier()
-        else "is_Identifier",
+        "isidentifier": lambda value: (f"{value}!" if value.isidentifier() else "is_Identifier"),
         "isnumeric": lambda value: f"{value}A" if value.isnumeric() else "012345",
-        "isprintable": lambda value: f"{value}{os.linesep}"
-        if value.isprintable()
-        else "is_printable",
+        "isprintable": lambda value: (
+            f"{value}{os.linesep}" if value.isprintable() else "is_printable"
+        ),
         "isspace": lambda value: f"{value}a" if value.isspace() else "   ",
         "istitle": lambda value: f"{value} AAA" if value.istitle() else "Is Title",
     }
@@ -228,10 +224,7 @@ class DynamicConstantProvider(DelegatingConstantProvider):
         # Might be a proxy.
         value = unwrap(value)
         if type(value) in typing.get_args(ConstantTypes):
-            if (
-                isinstance(value, (str, bytes))
-                and len(value) > self._max_constant_length
-            ):
+            if isinstance(value, str | bytes) and len(value) > self._max_constant_length:
                 return
             self._pool.add_constant(value)
 
@@ -288,17 +281,22 @@ def collect_static_constants(project_path: str | os.PathLike) -> ConstantPool:
         A dict of type to set of constants
     """
     collector = _ConstantCollector()
+    path = Path(project_path).resolve()
     for module in _find_modules_with_constants(project_path):
-        with open(os.path.join(project_path, module), encoding="utf-8") as module_file:
+        module_path = path / module
+        with module_path.open(mode="r", encoding="utf-8") as module_file:
             try:
                 tree = ast.parse(module_file.read())
                 collector.visit(tree)
-            except BaseException as exception:  # pylint: disable=broad-except
-                logger.exception("Cannot collect constants: %s", exception)
+            except BaseException as exception:  # noqa: BLE001
+                logger.warning(
+                    "Could not collect constants from %s. Skipping constant collection (%s).",
+                    module,
+                    exception,
+                )
     return collector.constants
 
 
-# pylint: disable=invalid-name, missing-function-docstring
 class _ConstantCollector(ast.NodeVisitor):
     """AST visitor that collects constants."""
 
@@ -306,24 +304,26 @@ class _ConstantCollector(ast.NodeVisitor):
         self._pool = ConstantPool()
         self._string_expressions: OrderedSet[str] = OrderedSet()
 
-    def visit_Constant(self, node: ast.Constant):
+    def visit_Constant(self, node: ast.Constant):  # noqa: N802
         if type(node.value) in typing.get_args(ConstantTypes):
             self._pool.add_constant(node.value)
         return self.generic_visit(node)
 
-    def visit_Module(self, node: ast.Module):
+    def visit_Module(self, node: ast.Module):  # noqa: N802
         return self._visit_doc_string(node)
 
-    def visit_FunctionDef(self, node: ast.FunctionDef):
+    def visit_FunctionDef(self, node: ast.FunctionDef):  # noqa: N802
         return self._visit_doc_string(node)
 
-    def visit_ClassDef(self, node: ast.ClassDef):
+    def visit_ClassDef(self, node: ast.ClassDef):  # noqa: N802
         return self._visit_doc_string(node)
 
-    def visit_AsyncFunctionDef(self, node: ast.AsyncFunctionDef):
+    def visit_AsyncFunctionDef(self, node: ast.AsyncFunctionDef):  # noqa: N802
         return self._visit_doc_string(node)
 
-    def _visit_doc_string(self, node: ast.AST):
+    def _visit_doc_string(
+        self, node: ast.AsyncFunctionDef | ast.FunctionDef | ast.ClassDef | ast.Module
+    ):
         if docstring := ast.get_docstring(node):
             self._string_expressions.add(docstring)
         return self.generic_visit(node)
@@ -338,3 +338,22 @@ class _ConstantCollector(ast.NodeVisitor):
         for doc in self._string_expressions:
             self._pool.remove_constant(doc)
         return self._pool
+
+
+class MLConstantPool:
+    """Manages a dictionary of constants for ML-specific testing."""
+
+    def __init__(self):  # noqa: D107
+        self._pool: dict[str, int] = {}
+
+    def add(self, key: str, value: int) -> None:
+        """Adds a key-value pair to the constant pool."""
+        self._pool[key] = value
+
+    def reset(self) -> None:
+        """Clears the constant pool."""
+        self._pool.clear()
+
+    def get_value(self, key: str, default: int | None = None) -> int | None:
+        """Retrieves the value for a given key, or returns the default if not found."""
+        return self._pool.get(key, default)
