@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -37,18 +37,23 @@ ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
+
 from __future__ import annotations
 
 import ast
 
-from abc import ABCMeta
+from abc import ABC
 from ast import iter_child_nodes
 from collections import defaultdict
-from collections.abc import Sequence
 from dataclasses import dataclass
+from typing import TYPE_CHECKING
 
 
-class _ASTVisitor(metaclass=ABCMeta):
+if TYPE_CHECKING:
+    from collections.abc import Sequence
+
+
+class _ASTVisitor(ABC):  # noqa: B024
     """Performs a depth-first walk of the AST."""
 
     def __init__(self):
@@ -155,8 +160,9 @@ class _PathGraphingAstVisitor(_ASTVisitor):
         for node in node_list:
             self.dispatch(node)
 
-    # pylint: disable=invalid-name
-    def visitFunctionDef(self, node: ast.FunctionDef | ast.AsyncFunctionDef) -> None:
+    def visitFunctionDef(  # noqa: N802
+        self, node: ast.FunctionDef | ast.AsyncFunctionDef
+    ) -> None:
         """Visits a function-definition node.
 
         Args:
@@ -182,7 +188,7 @@ class _PathGraphingAstVisitor(_ASTVisitor):
             self.graphs[f"{self.class_name}{node.name}"] = self.graph
             self.reset()
 
-    visitAsyncFunctionDef = visitFunctionDef
+    visitAsyncFunctionDef = visitFunctionDef  # noqa: N815
 
     def __append_path_node(self, name: str) -> _PathNode | None:
         if not self.tail:
@@ -193,8 +199,7 @@ class _PathGraphingAstVisitor(_ASTVisitor):
         self.tail = path_node
         return path_node
 
-    # pylint: disable=invalid-name
-    def visitSimpleStatement(self, node: ast.stmt) -> None:
+    def visitSimpleStatement(self, node: ast.stmt) -> None:  # noqa: N802
         """Visits a simple statement node of the AST.
 
         Args:
@@ -215,8 +220,7 @@ class _PathGraphingAstVisitor(_ASTVisitor):
         else:
             super().default(node, *args)
 
-    # pylint: disable=invalid-name
-    def visitLoop(self, node: ast.AsyncFor | ast.For | ast.While) -> None:
+    def visitLoop(self, node: ast.AsyncFor | ast.For | ast.While) -> None:  # noqa: N802
         """Visits a loop node.
 
         Args:
@@ -225,10 +229,9 @@ class _PathGraphingAstVisitor(_ASTVisitor):
         name = f"Loop {node.lineno}"
         self.__subgraph(node, name)
 
-    visitAsyncFor = visitFor = visitWhile = visitLoop
+    visitAsyncFor = visitFor = visitWhile = visitLoop  # noqa: N815
 
-    # pylint: disable=invalid-name
-    def visitIf(self, node: ast.If) -> None:
+    def visitIf(self, node: ast.If) -> None:  # noqa: N802
         """Visits an if expression node.
 
         Args:
@@ -285,8 +288,7 @@ class _PathGraphingAstVisitor(_ASTVisitor):
                 self.graph.connect(loose_end, bottom)
             self.tail = bottom
 
-    # pylint: disable=invalid-name
-    def visitTryExcept(self, node: ast.Try) -> None:
+    def visitTryExcept(self, node: ast.Try) -> None:  # noqa: N802
         """Visits a try-except AST node.
 
         Args:
@@ -295,10 +297,9 @@ class _PathGraphingAstVisitor(_ASTVisitor):
         name = f"TryExcept {node.lineno}"
         self.__subgraph(node, name, extra_blocks=node.handlers)
 
-    visitTry = visitTryExcept
+    visitTry = visitTryExcept  # noqa: N815
 
-    # pylint: disable=invalid-name
-    def visitWith(self, node: ast.With | ast.AsyncWith) -> None:
+    def visitWith(self, node: ast.With | ast.AsyncWith) -> None:  # noqa: N802
         """Visits a with-block AST node.
 
         Args:
@@ -308,7 +309,7 @@ class _PathGraphingAstVisitor(_ASTVisitor):
         self.__append_path_node(name)
         self.dispatch_list(node.body)
 
-    visitAsyncWith = visitWith
+    visitAsyncWith = visitWith  # noqa: N815
 
 
 def mccabe_complexity(tree: ast.AST) -> int:
