@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Implements simple constant seeding strategies."""
+
 from __future__ import annotations
 
 import ast
@@ -12,6 +13,7 @@ import inspect
 import logging
 import os
 
+from pathlib import Path
 from typing import TYPE_CHECKING
 from typing import Any
 from typing import AnyStr
@@ -22,9 +24,8 @@ import pynguin.configuration as config
 import pynguin.ga.testcasechromosome as tcc
 import pynguin.testcase.defaulttestcase as dtc
 import pynguin.testcase.statement as stmt
-import pynguin.utils.statistics.statistics as stat
+import pynguin.utils.statistics.stats as stat
 
-from pynguin.analyses.constants import ConstantProvider
 from pynguin.analyses.typesystem import ANY
 from pynguin.analyses.typesystem import Instance
 from pynguin.analyses.typesystem import ProperType
@@ -45,6 +46,7 @@ if TYPE_CHECKING:
     import pynguin.testcase.testfactory as tf
     import pynguin.testcase.variablereference as vr
 
+    from pynguin.analyses.constants import ConstantProvider
     from pynguin.analyses.module import ModuleTestCluster
 
 
@@ -60,7 +62,7 @@ class InitialPopulationProvider:
         test_factory: tf.TestFactory,
         constant_provider: ConstantProvider,
     ):
-        """Create new population provider
+        """Create new population provider.
 
         Args:
             test_cluster: Test cluster used to construct test cases
@@ -74,8 +76,7 @@ class InitialPopulationProvider:
 
     @staticmethod
     def _get_ast_tree(module_path: AnyStr | os.PathLike[AnyStr]) -> ast.Module | None:
-
-        """Returns the ast tree from a module
+        """Returns the ast tree from a module.
 
         Args:
             module_path: The path to the project's root
@@ -85,26 +86,33 @@ class InitialPopulationProvider:
         """
         module_name = config.configuration.module_name.rsplit(".", maxsplit=1)[-1]
         logger.debug("Module name: %s", module_name)
-        result: list[AnyStr] = []
+        result: list[Path] = []
         for root, _, files in os.walk(module_path):
+            root_path = Path(root).resolve()  # type: ignore[arg-type]
             for name in files:
                 assert isinstance(name, str)
                 if module_name in name and "test_" in name:
-                    result.append(os.path.join(root, name))
+                    result.append(root_path / name)
                     break
         try:
             if len(result) > 0:
                 logger.debug("Module name found: %s", result[0])
-                stat.track_output_variable(RuntimeVariable.SuitableTestModule, True)
-                with open(result[0], encoding="utf-8") as module_file:
+                stat.track_output_variable(RuntimeVariable.SuitableTestModule, value=True)
+                with result[0].open(mode="r", encoding="utf-8") as module_file:
                     return ast.parse(module_file.read())
             else:
                 logger.debug("No suitable test module found.")
-                stat.track_output_variable(RuntimeVariable.SuitableTestModule, False)
+                stat.track_output_variable(RuntimeVariable.SuitableTestModule, value=False)
                 return None
-        except BaseException as exception:  # pylint: disable=broad-except
-            logger.exception("Cannot read module: %s", exception)
-            stat.track_output_variable(RuntimeVariable.SuitableTestModule, False)
+        except BaseException as exception:  # noqa: BLE001
+            try:
+                logger.exception("Cannot read module: %s", exception)
+            except BaseException:  # noqa: BLE001
+                # Starting with Python 3.11, the exception method tries to parse the file
+                # using the ast.parse function which can fail.
+                logger.error("Cannot read module: %s", exception)
+
+            stat.track_output_variable(RuntimeVariable.SuitableTestModule, value=False)
             return None
 
     def collect_testcases(self, module_path: AnyStr | os.PathLike[AnyStr]) -> None:
@@ -126,19 +134,17 @@ class InitialPopulationProvider:
         transformer.visit(tree)
         self._testcases = transformer.testcases
         stat.track_output_variable(RuntimeVariable.FoundTestCases, len(self._testcases))
-        stat.track_output_variable(
-            RuntimeVariable.CollectedTestCases, len(self._testcases)
-        )
+        stat.track_output_variable(RuntimeVariable.CollectedTestCases, len(self._testcases))
         self._mutate_testcases_initially()
 
     def _mutate_testcases_initially(self):
         """Mutates the initial population."""
-        for _ in range(0, config.configuration.seeding.initial_population_mutations):
+        for _ in range(config.configuration.seeding.initial_population_mutations):
             for testcase in self._testcases:
                 testcase_wrapper = tcc.TestCaseChromosome(testcase, self._test_factory)
                 testcase_wrapper.mutate()
                 if not testcase_wrapper.test_case.statements:
-                    self._testcases.remove(testcase)
+                    self._testcases.remove(testcase)  # noqa: B909
 
     def random_testcase(self) -> tc.TestCase:
         """Provides a random seeded test case.
@@ -184,13 +190,9 @@ def create_assign_stmt(
         o for o in objs_under_test if isinstance(o, GenericCallableAccessibleObject)
     }
     if isinstance(value, ast.Constant):
-        new_stmt = create_stmt_from_constant(
-            value, testcase, constant_provider=constant_provider
-        )
+        new_stmt = create_stmt_from_constant(value, testcase, constant_provider=constant_provider)
     elif isinstance(value, ast.UnaryOp):
-        new_stmt = create_stmt_from_unaryop(
-            value, testcase, constant_provider=constant_provider
-        )
+        new_stmt = create_stmt_from_unaryop(value, testcase, constant_provider=constant_provider)
     elif isinstance(value, ast.Call):
         new_stmt = create_stmt_from_call(
             value,
@@ -199,7 +201,7 @@ def create_assign_stmt(
             ref_dict,
             constant_provider=constant_provider,
         )
-    elif isinstance(value, (ast.List, ast.Set, ast.Dict, ast.Tuple)):
+    elif isinstance(value, ast.List | ast.Set | ast.Dict | ast.Tuple):
         new_stmt = create_stmt_from_collection(
             value,
             testcase,
@@ -236,7 +238,7 @@ def create_assert_stmt(
         operator = assert_node.test.ops[0]  # type: ignore[attr-defined]
     except (KeyError, AttributeError):
         return None
-    if isinstance(operator, (ast.Is, ast.Eq)):
+    if isinstance(operator, ast.Is | ast.Eq):
         assertion = create_assertion(source, val_elem)
     if assertion is not None:
         return assertion, source
@@ -270,7 +272,9 @@ def create_variable_references_from_call_args(
     gen_callable: GenericCallableAccessibleObject,
     ref_dict: dict[str, vr.VariableReference],
 ) -> dict[str, vr.VariableReference] | None:
-    """Takes the arguments of an ast.Call node and returns the variable references of
+    """Creates variable references from call arguments.
+
+    Takes the arguments of an ast.Call node and returns the variable references of
     the corresponding statements.
 
     Args:
@@ -281,7 +285,6 @@ def create_variable_references_from_call_args(
 
     Returns:
         The dict with the variable references of the call_args.
-
     """
     var_refs: dict[str, vr.VariableReference] = {}
     # We have to ignore the first parameter (usually 'self') for regular methods and
@@ -292,21 +295,22 @@ def create_variable_references_from_call_args(
     shift_by = 1 if gen_callable.is_method() or gen_callable.is_constructor() else 0
 
     # Handle positional arguments.
-    for (name, param), call_arg in zip(
+    # TODO(sl) check for the zip, it sometimes gets lists of different lengths, where it
+    #  silently swallows the rest of the longer list—the strict parameter would prevent
+    #  this but causes failures in our test suite; needs investigation...
+    for (name, param), call_arg in zip(  # noqa: B905
         list(gen_callable.inferred_signature.signature.parameters.items())[shift_by:],
         call_args,
     ):
         if (
             param.kind
-            in (
+            in {
                 inspect.Parameter.POSITIONAL_ONLY,
                 inspect.Parameter.POSITIONAL_OR_KEYWORD,
-            )
+            }
         ) and isinstance(call_arg, ast.Name):
             reference = ref_dict.get(call_arg.id)
-        elif param.kind == inspect.Parameter.VAR_POSITIONAL and isinstance(
-            call_arg, ast.Starred
-        ):
+        elif param.kind == inspect.Parameter.VAR_POSITIONAL and isinstance(call_arg, ast.Starred):
             reference = ref_dict.get(call_arg.value.id)  # type: ignore[attr-defined]
         else:
             return None
@@ -320,9 +324,7 @@ def create_variable_references_from_call_args(
         keyword = call_keyword.arg
         if keyword is None:
             # **kwargs has to be the last parameter?
-            keyword = list(gen_callable.inferred_signature.signature.parameters.keys())[
-                -1
-            ]
+            keyword = list(gen_callable.inferred_signature.signature.parameters.keys())[-1]
             if (
                 gen_callable.inferred_signature.signature.parameters[keyword].kind
                 != inspect.Parameter.VAR_KEYWORD
@@ -338,7 +340,6 @@ def create_variable_references_from_call_args(
     return var_refs
 
 
-# pylint: disable=too-many-return-statements
 def create_stmt_from_constant(
     constant: ast.Constant, testcase: tc.TestCase, constant_provider: ConstantProvider
 ) -> stmt.VariableCreatingStatement | None:
@@ -359,21 +360,13 @@ def create_stmt_from_constant(
     if isinstance(val, bool):
         return stmt.BooleanPrimitiveStatement(testcase, val)
     if isinstance(val, int):
-        return stmt.IntPrimitiveStatement(
-            testcase, val, constant_provider=constant_provider
-        )
+        return stmt.IntPrimitiveStatement(testcase, val, constant_provider=constant_provider)
     if isinstance(val, float):
-        return stmt.FloatPrimitiveStatement(
-            testcase, val, constant_provider=constant_provider
-        )
+        return stmt.FloatPrimitiveStatement(testcase, val, constant_provider=constant_provider)
     if isinstance(val, str):
-        return stmt.StringPrimitiveStatement(
-            testcase, val, constant_provider=constant_provider
-        )
+        return stmt.StringPrimitiveStatement(testcase, val, constant_provider=constant_provider)
     if isinstance(val, bytes):
-        return stmt.BytesPrimitiveStatement(
-            testcase, val, constant_provider=constant_provider
-        )
+        return stmt.BytesPrimitiveStatement(testcase, val, constant_provider=constant_provider)
     logger.info("Could not find case for constant while handling assign statement.")
     return None
 
@@ -399,12 +392,8 @@ def create_stmt_from_unaryop(
             testcase, (-1) * val, constant_provider=constant_provider
         )
     if isinstance(val, int):
-        return stmt.IntPrimitiveStatement(
-            testcase, (-1) * val, constant_provider=constant_provider
-        )
-    logger.info(
-        "Could not find case for unary operator while handling assign statement."
-    )
+        return stmt.IntPrimitiveStatement(testcase, (-1) * val, constant_provider=constant_provider)
+    logger.info("Could not find case for unary operator while handling assign statement.")
     return None
 
 
@@ -415,8 +404,10 @@ def create_stmt_from_call(
     ref_dict: dict[str, vr.VariableReference],
     constant_provider: ConstantProvider,
 ) -> stmt.VariableCreatingStatement | None:
-    """Creates the corresponding statement from an ast.call node. Depending on the call,
-    this can be a GenericConstructor, GenericMethod or GenericFunction statement.
+    """Creates the corresponding statement from an ast.call node.
+
+    Depending on the call, this can be a GenericConstructor, GenericMethod, or
+    GenericFunction statement.
 
     Args:
         call: the ast.Call node
@@ -430,7 +421,7 @@ def create_stmt_from_call(
         The corresponding statement.
     """
     try:
-        call.func.attr  # type: ignore[attr-defined]
+        call.func.attr  # type: ignore[attr-defined]  # noqa: B018
     except AttributeError:
         return try_generating_specific_function(
             call,
@@ -451,7 +442,9 @@ def find_gen_callable(
     objs_under_test: set,
     ref_dict: dict[str, vr.VariableReference],
 ) -> GenericConstructor | GenericMethod | GenericFunction | None:
-    """Traverses the accessible objects under test and returns the one matching with the
+    """Find a call object.
+
+    Traverses the accessible objects under test and returns the one matching with the
     ast.call object. Unfortunately, there is no possibility to clearly determine if the
     ast.call object is a constructor, method or function. Hence, the looping over all
     accessible objects is unavoidable. Then, by the name of the ast.call and by the
@@ -495,9 +488,8 @@ def find_gen_callable(
                 var_type = ref_dict[obj_from_ast].type
                 if isinstance(var_type, Instance) and var_type.type == obj.owner:
                     return obj
-        elif isinstance(obj, GenericFunction):
-            if call_name == obj.function_name:
-                return obj
+        elif isinstance(obj, GenericFunction) and call_name == obj.function_name:
+            return obj
     return None
 
 
@@ -507,8 +499,7 @@ def assemble_stmt_from_gen_callable(
     call: ast.Call,
     ref_dict: dict[str, vr.VariableReference],
 ) -> stmt.ParametrizedStatement | None:
-    """Takes a generic callable and assembles the corresponding parametrized statement
-    from it.
+    """Takes a generic callable and assembles the corresponding parametrized statement.
 
     Args:
         testcase: the testcase of the statement
@@ -521,19 +512,22 @@ def assemble_stmt_from_gen_callable(
         The corresponding statement.
     """
     for arg in call.args:
-        if not isinstance(arg, (ast.Name, ast.Starred)):
+        if not isinstance(arg, ast.Name | ast.Starred):
             return None
     for keyword in call.keywords:
         if not isinstance(keyword, ast.keyword):
             return None
     var_refs = create_variable_references_from_call_args(
-        call.args, call.keywords, gen_callable, ref_dict  # type: ignore[arg-type]
+        call.args,  # type: ignore[arg-type]
+        call.keywords,
+        gen_callable,
+        ref_dict,
     )
     if var_refs is None:
         return None
     if isinstance(gen_callable, GenericFunction):
         return stmt.FunctionStatement(
-            testcase, cast(GenericCallableAccessibleObject, gen_callable), var_refs
+            testcase, cast("GenericCallableAccessibleObject", gen_callable), var_refs
         )
     if isinstance(gen_callable, GenericMethod):
         return stmt.MethodStatement(
@@ -544,7 +538,7 @@ def assemble_stmt_from_gen_callable(
         )
     if isinstance(gen_callable, GenericConstructor):
         return stmt.ConstructorStatement(
-            testcase, cast(GenericCallableAccessibleObject, gen_callable), var_refs
+            testcase, cast("GenericCallableAccessibleObject", gen_callable), var_refs
         )
     return None
 
@@ -556,8 +550,9 @@ def create_stmt_from_collection(
     ref_dict: dict[str, vr.VariableReference],
     constant_provider: ConstantProvider,
 ) -> stmt.VariableCreatingStatement | None:
-    """Creates the corresponding statement from an ast.List node. Lists contain other
-    statements.
+    """Creates the corresponding statement from an ast.List node.
+
+    Lists contain other statements.
 
     Args:
         coll_node: the ast node. It has the type of one of the collection types.
@@ -573,9 +568,8 @@ def create_stmt_from_collection(
     Returns:
         The corresponding list statement.
     """
-    coll_elems: None | (
-        list[vr.VariableReference]
-        | list[tuple[vr.VariableReference, vr.VariableReference]]
+    coll_elems: None | (  # noqa: RUF036
+        list[vr.VariableReference] | list[tuple[vr.VariableReference, vr.VariableReference]]
     )
     if isinstance(coll_node, ast.Dict):
         keys = create_elements(
@@ -598,7 +592,7 @@ def create_stmt_from_collection(
             testcase.test_cluster.type_system.to_type_info(dict),
             (get_collection_type(keys), get_collection_type(values)),
         )
-        coll_elems = list(zip(keys, values))
+        coll_elems = list(zip(keys, values, strict=True))
     else:
         elements = coll_node.elts
         coll_elems = create_elements(
@@ -622,20 +616,19 @@ def create_stmt_from_collection(
                 testcase.test_cluster.type_system.to_type_info(set),
                 (get_collection_type(coll_elems),),
             )
-    return create_specific_collection_stmt(
-        testcase, coll_node, coll_elems_type, coll_elems
-    )
+    return create_specific_collection_stmt(testcase, coll_node, coll_elems_type, coll_elems)
 
 
-def create_elements(
+def create_elements(  # noqa: C901
     elements: Any,
     testcase: tc.TestCase,
     objs_under_test: set[GenericCallableAccessibleObject],
     ref_dict: dict[str, vr.VariableReference],
     constant_provider: ConstantProvider,
 ) -> list[vr.VariableReference] | None:
-    """Creates the elements of a collection by calling the corresponding methods for
-    creation. This can be recursive.
+    """Creates the elements of a collection.
+
+    Calls the corresponding methods for creation. This can be recursive.
 
     Args:
         elements: The elements of the collection
@@ -677,7 +670,7 @@ def create_elements(
             if not statement:
                 return None
             coll_elems.append(testcase.add_variable_creating_statement(statement))
-        elif isinstance(elem, (ast.List, ast.Tuple, ast.Set, ast.Dict)):
+        elif isinstance(elem, ast.List | ast.Tuple | ast.Set | ast.Dict):
             statement = create_stmt_from_collection(
                 elem,
                 testcase,
@@ -699,8 +692,9 @@ def create_elements(
 
 
 def get_collection_type(coll_elems: list[vr.VariableReference]) -> ProperType:
-    """Returns the type of a collection. If objects of multiple types are in the
-    collection, this function returns None.
+    """Returns the type of a collection.
+
+    If objects of multiple types are in the collection, this function returns None.
 
     Args:
         coll_elems: a list of variable references
@@ -712,7 +706,7 @@ def get_collection_type(coll_elems: list[vr.VariableReference]) -> ProperType:
         return ANY
     coll_type = coll_elems[0].type
     for elem in coll_elems:
-        if not elem.type == coll_type:
+        if elem.type != coll_type:
             coll_type = ANY
             break
     return coll_type
@@ -723,7 +717,7 @@ def create_specific_collection_stmt(
     coll_node: ast.List | ast.Set | ast.Dict | ast.Tuple,
     coll_elems_type: ProperType,
     coll_elems: list[Any],
-) -> None | (
+) -> None | (  # noqa: RUF036
     stmt.ListStatement | stmt.SetStatement | stmt.DictStatement | stmt.TupleStatement
 ):
     """Creates the corresponding collection statement from an ast node.
@@ -756,7 +750,9 @@ def try_generating_specific_function(
     ref_dict: dict[str, vr.VariableReference],
     constant_provider: ConstantProvider,
 ) -> stmt.VariableCreatingStatement | None:
-    """Calls to creating a collection (list, set, tuple, dict) via their keywords and
+    """Aims to generate specific functions.
+
+    Calls to creating a collection (list, set, tuple, dict) via their keywords and
     not via literal syntax are considered as ast.Call statements. But for these calls,
     no accessible object under test is in the test_cluster. To parse them anyway, these
     method transforms them to the corresponding ast statement, for example a call of a
@@ -772,7 +768,6 @@ def try_generating_specific_function(
 
     Returns:
         The corresponding statement.
-
     """
     try:
         func_id = str(call.func.id)  # type: ignore[attr-defined]
@@ -782,7 +777,7 @@ def try_generating_specific_function(
         try:
             set_node = ast.Set(
                 elts=call.args,
-                ctx=ast.Load(),
+                ctx=ast.Load(),  # type: ignore[call-arg]
             )
         except AttributeError:
             return None
@@ -825,7 +820,7 @@ def try_generating_specific_function(
         )
     if func_id == "dict":
         try:
-            dict_node = ast.Dict(
+            dict_node = ast.Dict(  # type: ignore[call-arg]
                 keys=(
                     call.args[0].keys if call.args else []  # type: ignore[attr-defined]
                 ),
@@ -848,15 +843,13 @@ def try_generating_specific_function(
     return None
 
 
-# pylint: disable=invalid-name, missing-function-docstring, too-many-instance-attributes
 class AstToTestCaseTransformer(ast.NodeVisitor):
-    """An AST NodeVisitor that tries to convert an AST into our internal
-    test case representation."""
+    """Transforms a Python AST into our internal test-case representation."""
 
-    def __init__(
+    def __init__(  # noqa: D107
         self,
         test_cluster: ModuleTestCluster,
-        create_assertions: bool,
+        create_assertions: bool,  # noqa: FBT001
         constant_provider: ConstantProvider,
     ):
         self._current_testcase: dtc.DefaultTestCase = dtc.DefaultTestCase(test_cluster)
@@ -868,7 +861,7 @@ class AstToTestCaseTransformer(ast.NodeVisitor):
         self._create_assertions = create_assertions
         self._constant_provider = constant_provider
 
-    def visit_FunctionDef(self, node: ast.FunctionDef) -> Any:
+    def visit_FunctionDef(self, node: ast.FunctionDef) -> Any:  # noqa: D102, N802
         self._number_found_testcases += 1
         self._current_testcase = dtc.DefaultTestCase(self._test_cluster)
         self._current_parsable = True
@@ -877,7 +870,7 @@ class AstToTestCaseTransformer(ast.NodeVisitor):
         if self._current_parsable:
             self._testcases.append(self._current_testcase)
 
-    def visit_Assign(self, node: ast.Assign) -> Any:
+    def visit_Assign(self, node: ast.Assign) -> Any:  # noqa: D102, N802
         if self._current_parsable:
             if (
                 result := create_assign_stmt(
@@ -894,8 +887,8 @@ class AstToTestCaseTransformer(ast.NodeVisitor):
                 var_ref = self._current_testcase.add_variable_creating_statement(stm)
                 self._var_refs[ref_id] = var_ref
 
-    def visit_Assert(self, node: ast.Assert) -> Any:
-        if self._current_parsable and self._create_assertions:
+    def visit_Assert(self, node: ast.Assert) -> Any:  # noqa: D102, N802
+        if self._current_parsable and self._create_assertions:  # noqa: SIM102
             if (result := create_assert_stmt(self._var_refs, node)) is not None:
                 assertion, var_ref = result
                 self._current_testcase.get_statement(
@@ -905,6 +898,7 @@ class AstToTestCaseTransformer(ast.NodeVisitor):
     @property
     def testcases(self) -> list[dtc.DefaultTestCase]:
         """Provides the testcases that could be generated from the given AST.
+
         It is possible that not every aspect of the AST could be transformed
         to our internal representation.
 
