@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -10,6 +10,7 @@ The implementation of this module contains some code adopted from the ``darglint
 library (https://github.com/terrencepreilly/darglint), which was released by Terrence
 Reilly under MIT license.
 """
+
 from __future__ import annotations
 
 import ast
@@ -17,7 +18,7 @@ import dataclasses
 import logging
 
 from collections import deque
-from collections.abc import Iterable
+from typing import TYPE_CHECKING
 from typing import Any
 from typing import TypeAlias
 
@@ -26,6 +27,10 @@ import astroid
 from astroid.nodes.as_string import to_code
 
 
+if TYPE_CHECKING:
+    from collections.abc import Iterable
+
+
 _LOGGER = logging.getLogger(__name__)
 AstroidFunctionDef: TypeAlias = astroid.AsyncFunctionDef | astroid.FunctionDef
 ASTFunctionDef: TypeAlias = ast.AsyncFunctionDef | ast.FunctionDef
@@ -58,24 +63,21 @@ class _FunctionScopedVisitorMixin(ast.NodeVisitor):
         super().__init__()
         self.in_function: bool = False
 
-    # pylint: disable=invalid-name, missing-docstring
-    def visit_AsyncFunctionDef(self, node: ast.AsyncFunctionDef) -> ast.AST:
+    def visit_AsyncFunctionDef(  # noqa: N802
+        self, node: ast.AsyncFunctionDef
+    ) -> ast.AST:
         if not self.in_function:
             self.in_function = True
-            return getattr(super(), "visit_AsyncFunctionDef", super().generic_visit)(
-                node
-            )
+            return getattr(super(), "visit_AsyncFunctionDef", super().generic_visit)(node)
         return ast.Pass()
 
-    # pylint: disable=invalid-name, missing-docstring
-    def visit_FunctionDef(self, node: ast.FunctionDef) -> ast.AST:
+    def visit_FunctionDef(self, node: ast.FunctionDef) -> ast.AST:  # noqa: N802
         if not self.in_function:
             self.in_function = True
             return getattr(super(), "visit_FunctionDef", super().generic_visit)(node)
         return ast.Pass()
 
-    # pylint: disable=invalid-name, missing-docstring
-    def visit_Lambda(self, node: ast.Lambda) -> ast.AST:
+    def visit_Lambda(self, node: ast.Lambda) -> ast.AST:  # noqa: N802
         if not self.in_function:
             self.in_function = True
             return getattr(super(), "visit_Lambda", super().generic_visit)(node)
@@ -122,7 +124,7 @@ class _Context:
             elif isinstance(curr, ast.Tuple):
                 names: list[str] = []
                 for node in curr.elts:
-                    if isinstance(node, (ast.Attribute, ast.Name)):
+                    if isinstance(node, ast.Attribute | ast.Name):
                         names.extend(self.__get_attr_name(node))
                     else:
                         _LOGGER.error(
@@ -134,8 +136,7 @@ class _Context:
                 return names
             else:
                 _LOGGER.error(
-                    "While getting ast.Attribute representation a node had an "
-                    "unexpected type %s",
+                    "While getting ast.Attribute representation a node had an unexpected type %s",
                     curr.__class__.__name__,
                 )
                 curr = None
@@ -145,13 +146,12 @@ class _Context:
 
     @staticmethod
     def __get_name_name(name: ast.Name | ast.Tuple) -> str | list[str]:
-        assert isinstance(name, (ast.Name, ast.Tuple))
+        assert isinstance(name, ast.Name | ast.Tuple)
         if isinstance(name, ast.Name):
             return name.id
         return [node.id for node in name.elts if isinstance(node, ast.Name)]
 
-    # pylint: disable=too-many-branches, too-many-return-statements
-    def __get_exception_name(self, raises: ast.Raise) -> str | list[str]:
+    def __get_exception_name(self, raises: ast.Raise) -> str | list[str]:  # noqa: C901
         if isinstance(raises, str):
             return raises
 
@@ -162,9 +162,9 @@ class _Context:
             return name
         if isinstance(raises.exc, ast.Call):
             if hasattr(raises.exc.func, "id"):
-                return getattr(raises.exc.func, "id")
+                return raises.exc.func.id
             if hasattr(raises.exc.func, "attr"):
-                return getattr(raises.exc.func, "attr")
+                return raises.exc.func.attr
             _LOGGER.debug(
                 "Raises function call has neither id nor attr, has only %s",
                 str(dir(raises.exc.func)),
@@ -174,10 +174,10 @@ class _Context:
         elif isinstance(raises.exc, ast.Subscript):
             id_repr = ""
             if hasattr(raises.exc.value, "id"):
-                id_repr = getattr(raises.exc.value, "id")
+                id_repr = raises.exc.value.id
             n_repr = ""
             if hasattr(raises.exc.slice, "value"):
-                value = getattr(raises.exc.slice, "value")
+                value = raises.exc.slice.value
                 if hasattr(value, "n"):
                     n_repr = value.n
             return f"{id_repr}[{n_repr}]"
@@ -205,7 +205,7 @@ class _Context:
             A list of exceptions to be passed up to the parent context
         """
         name = self.__get_exception_name(node)
-        if name == "":
+        if not name:  # string `name` is empty
             if self.bare_handler_exceptions is not None:
                 return self.bare_handler_exceptions | self.exceptions
             if self.exceptions:
@@ -314,11 +314,10 @@ class _RaiseVisitor(ast.NodeVisitor):
         """
         return self.contexts[-1]
 
-    # pylint: disable=invalid-name, missing-docstring
-    def visit_Raise(self, node: ast.Raise) -> ast.AST:
+    def visit_Raise(self, node: ast.Raise) -> ast.AST:  # noqa: N802
         bubbles = self.context.add_exception(node)
         if bubbles:
-            assert len(self.contexts) > 1
+            assert len(self.contexts) >= 1
             if len(self.contexts) < 2:
                 return self.generic_visit(node)
             parent_context = self.contexts[-2]
@@ -326,16 +325,15 @@ class _RaiseVisitor(ast.NodeVisitor):
 
         return self.generic_visit(node)
 
-    # pylint: disable=invalid-name, missing-docstring
-    def visit_Try(self, node: ast.Try) -> None:
+    def visit_Try(self, node: ast.Try) -> None:  # noqa: N802
         self.contexts.append(_Context())
         for child in node.body:
             self.visit(child)
         for handler in node.handlers:
             if handler.type:
-                if handler.name and (isinstance(handler.type, (ast.Name, ast.Tuple))):
+                if handler.name and (isinstance(handler.type, ast.Name | ast.Tuple)):
                     self.context.add_variable(handler.name, handler.type)
-                elif isinstance(handler.type, (ast.Attribute, ast.Name, ast.Tuple)):
+                elif isinstance(handler.type, ast.Attribute | ast.Name | ast.Tuple):
                     self.context.set_handling(handler.type)
                 else:
                     _LOGGER.error(
@@ -371,15 +369,18 @@ class _RaiseVisitor(ast.NodeVisitor):
         context = self.contexts.pop()
         self.context.extend(context)
 
-    # pylint: disable=invalid-name, missing-docstring
-    def visit_Assert(self, node: ast.Assert) -> ast.AST:
+    def visit_Assert(self, node: ast.Assert) -> ast.AST:  # noqa: N802
         # If we see an assert statement in the subject under test we expect that the
         # assertion can also be failing, thus it is legitimate to raise an
         # AssertionError.  Hence, we add the AssertionError to the set of raised
         # exceptions.
         self.visit_Raise(
             ast.Raise(
-                exc=ast.Call(func=ast.Name(id="AssertionError", ctx=ast.Load())),
+                exc=ast.Call(
+                    func=ast.Name(id="AssertionError", ctx=ast.Load()),
+                    args=[],
+                    keywords=[],
+                ),
             )
         )
         # Make sure that we also execute a visit_Assert method in another analysis
@@ -394,13 +395,11 @@ class _YieldVisitor(ast.NodeVisitor):
         super().__init__()
         self.yields: list[ast.Yield | ast.YieldFrom] = []
 
-    # pylint: disable=invalid-name, missing-docstring
-    def visit_Yield(self, node: ast.Yield) -> ast.AST:
+    def visit_Yield(self, node: ast.Yield) -> ast.AST:  # noqa: N802
         self.yields.append(node)
         return self.generic_visit(node)
 
-    # pylint: disable=invalid-name, missing-docstring
-    def visit_YieldFrom(self, node: ast.YieldFrom) -> ast.AST:
+    def visit_YieldFrom(self, node: ast.YieldFrom) -> ast.AST:  # noqa: N802
         self.yields.append(node)
         return self.generic_visit(node)
 
@@ -412,8 +411,7 @@ class _ReturnVisitor(ast.NodeVisitor):
         super().__init__()
         self.returns: list[ast.Return | None] = []
 
-    # pylint: disable=invalid-name, missing-docstring
-    def visit_Return(self, node: ast.Return) -> ast.AST:
+    def visit_Return(self, node: ast.Return) -> ast.AST:  # noqa: N802
         self.returns.append(node)
         return self.generic_visit(node)
 
@@ -425,15 +423,13 @@ class _AssertVisitor(ast.NodeVisitor):
         super().__init__()
         self.asserts: list[ast.Assert] = []
 
-    # pylint: disable=invalid-name, missing-docstring
-    def visit_Assert(self, node: ast.Assert) -> ast.AST:
+    def visit_Assert(self, node: ast.Assert) -> ast.AST:  # noqa: N802
         self.asserts.append(node)
         # Make sure that we also execute a visit_Assert method in another analysis
         # visitor class.
         return getattr(super(), "visit_Assert", super().generic_visit)(node)
 
 
-# pylint: disable=too-many-ancestors
 class FunctionAnalysisVisitor(
     _FunctionScopedVisitorMixin,  # needs to be first in order!
     _RaiseVisitor,
@@ -449,7 +445,7 @@ class FunctionAnalysisVisitor(
 
 
 @dataclasses.dataclass
-class FunctionDescription:  # pylint: disable=too-many-instance-attributes
+class FunctionDescription:
     """Describes a function or method in the subject under test.
 
     Attributes:
@@ -477,6 +473,7 @@ class FunctionDescription:  # pylint: disable=too-many-instance-attributes
 
 def astroid_to_ast(astroid_in: AstroidFunctionDef) -> ASTFunctionDef:
     """Some part of the analysis only works with Pythons AST (for now).
+
     So it is necessary to convert astroid to AST.
 
     Args:
@@ -506,14 +503,12 @@ def get_function_node_from_ast(
     if name not in tree.locals:
         return None
     maybe_function = tree.locals[name][0]
-    if isinstance(maybe_function, (astroid.FunctionDef, astroid.AsyncFunctionDef)):
+    if isinstance(maybe_function, astroid.FunctionDef | astroid.AsyncFunctionDef):
         return maybe_function
     return None
 
 
-def get_class_node_from_ast(
-    tree: astroid.Module | None, name: str
-) -> astroid.ClassDef | None:
+def get_class_node_from_ast(tree: astroid.Module | None, name: str) -> astroid.ClassDef | None:
     """Get the AST Node that represents the class with the given name.
 
     Args:
