@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides a base class for assertions."""
+
 from __future__ import annotations
 
 from abc import ABC
@@ -14,6 +15,8 @@ from typing import Any
 
 
 if TYPE_CHECKING:
+    import ast
+
     import pynguin.testcase.variablereference as vr
 
     from pynguin.slicer.executionflowbuilder import UniqueInstruction
@@ -22,7 +25,7 @@ if TYPE_CHECKING:
 class Assertion:
     """Base class for assertions."""
 
-    def __init__(self):
+    def __init__(self):  # noqa: D107
         # this makes an assertion stateful, but is required for caching in slicing
         self._checked_instructions: list[UniqueInstruction] = []
 
@@ -44,9 +47,7 @@ class Assertion:
         """
 
     @abstractmethod
-    def clone(
-        self, memo: dict[vr.VariableReference, vr.VariableReference]
-    ) -> Assertion:
+    def clone(self, memo: dict[vr.VariableReference, vr.VariableReference]) -> Assertion:
         """Clone this assertion.
 
         Args:
@@ -56,7 +57,7 @@ class Assertion:
         """
 
     @abstractmethod
-    def __eq__(self, other: Any) -> bool:
+    def __eq__(self, other: object) -> bool:
         pass  # pragma: no cover
 
     @abstractmethod
@@ -67,7 +68,7 @@ class Assertion:
 class ReferenceAssertion(Assertion, ABC):
     """An assertion on a single reference."""
 
-    def __init__(self, source: vr.Reference):
+    def __init__(self, source: vr.Reference):  # noqa: D107
         super().__init__()
         self._source = source
 
@@ -80,9 +81,19 @@ class ReferenceAssertion(Assertion, ABC):
         """
         return self._source
 
+    @source.setter
+    def source(self, value: vr.Reference) -> None:
+        """Set the reference to be used for assertions.
+
+        Args:
+            value (vr.Reference): The reference to set for assertions.
+        """
+        self._source = value
+
 
 class TypeNameAssertion(ReferenceAssertion):
     """An assertion that a reference has a type with a certain name.
+
     We compare the string representation of the fully qualified name.
     Using an isinstance check would also be possible, but classes might not always
     be accessible, for example when nested inside a function.
@@ -92,7 +103,7 @@ class TypeNameAssertion(ReferenceAssertion):
         assert f"{type(int_0).__module__}.{type(int_0).__qualname__}" == "builtins.int"
     """
 
-    def __init__(self, source: vr.Reference, module: str, qualname: str):
+    def __init__(self, source: vr.Reference, module: str, qualname: str):  # noqa: D107
         super().__init__(source)
         self._module = module
         self._qualname = qualname
@@ -115,15 +126,15 @@ class TypeNameAssertion(ReferenceAssertion):
         """
         return self._qualname
 
-    def accept(self, visitor: AssertionVisitor) -> None:
+    def accept(self, visitor: AssertionVisitor) -> None:  # noqa: D102
         visitor.visit_type_name_assertion(self)
 
-    def clone(
+    def clone(  # noqa: D102
         self, memo: dict[vr.VariableReference, vr.VariableReference]
     ) -> TypeNameAssertion:
         return TypeNameAssertion(self._source.clone(memo), self._module, self._qualname)
 
-    def __eq__(self, other: Any) -> bool:
+    def __eq__(self, other: object) -> bool:
         return (
             isinstance(other, TypeNameAssertion)
             and self._source == other._source
@@ -145,7 +156,7 @@ class FloatAssertion(ReferenceAssertion):
         assert float_0 == pytest.approx(42, rel=0.01, abs=0.01)
     """
 
-    def __init__(self, source: vr.Reference, value: float):
+    def __init__(self, source: vr.Reference, value: float):  # noqa: D107
         super().__init__(source)
         self._value = value
 
@@ -158,15 +169,15 @@ class FloatAssertion(ReferenceAssertion):
         """
         return self._value
 
-    def accept(self, visitor: AssertionVisitor) -> None:
+    def accept(self, visitor: AssertionVisitor) -> None:  # noqa: D102
         visitor.visit_float_assertion(self)
 
-    def clone(
+    def clone(  # noqa: D102
         self, memo: dict[vr.VariableReference, vr.VariableReference]
     ) -> FloatAssertion:
         return FloatAssertion(self.source.clone(memo), self._value)
 
-    def __eq__(self, other: Any) -> bool:
+    def __eq__(self, other: object) -> bool:
         return (
             isinstance(other, FloatAssertion)
             and self._source == other._source
@@ -192,7 +203,7 @@ class ObjectAssertion(ReferenceAssertion):
         assert var_2 == "Foobar"
     """
 
-    def __init__(self, source: vr.Reference, value: Any):
+    def __init__(self, source: vr.Reference, value: Any):  # noqa: D107
         super().__init__(source)
         self._object = value
 
@@ -205,15 +216,15 @@ class ObjectAssertion(ReferenceAssertion):
         """
         return self._object
 
-    def accept(self, visitor: AssertionVisitor) -> None:
+    def accept(self, visitor: AssertionVisitor) -> None:  # noqa: D102
         visitor.visit_object_assertion(self)
 
-    def clone(
+    def clone(  # noqa: D102
         self, memo: dict[vr.VariableReference, vr.VariableReference]
     ) -> ObjectAssertion:
         return ObjectAssertion(self.source.clone(memo), self._object)
 
-    def __eq__(self, other: Any) -> bool:
+    def __eq__(self, other: Any) -> bool:  # noqa: PYI032
         return (
             isinstance(other, ObjectAssertion)
             and self._source == other._source
@@ -229,6 +240,46 @@ class ObjectAssertion(ReferenceAssertion):
         return f"ObjectAssertion({self._source!r}, {self._object!r})"
 
 
+class IsInstanceAssertion(ReferenceAssertion):
+    """An assertion that checks if a reference is an instance of a given type."""
+
+    def __init__(  # noqa: D107
+        self, source: vr.Reference, expected_type: ast.Attribute | ast.Name
+    ):
+        super().__init__(source)
+        self._expected_type = expected_type
+
+    @property
+    def expected_type(self) -> ast.Attribute | ast.Name:
+        """Provides the expected type.
+
+        Returns:
+            The expected type
+        """
+        return self._expected_type
+
+    def accept(self, visitor: AssertionVisitor) -> None:  # noqa: D102
+        visitor.visit_isinstance_assertion(self)
+
+    def clone(  # noqa: D102
+        self, memo: dict[vr.VariableReference, vr.VariableReference]
+    ) -> IsInstanceAssertion:
+        return IsInstanceAssertion(self.source.clone(memo), self._expected_type)
+
+    def __eq__(self, other: object) -> bool:
+        return (
+            isinstance(other, IsInstanceAssertion)
+            and self._source == other._source
+            and self._expected_type == other._expected_type
+        )
+
+    def __hash__(self) -> int:
+        return hash((self._source, self._expected_type))
+
+    def __repr__(self):
+        return f"IsInstanceAssertion({self._source!r}, {self._expected_type!r})"
+
+
 class CollectionLengthAssertion(ReferenceAssertion):
     """An assertion on the length of a reference.
 
@@ -240,7 +291,7 @@ class CollectionLengthAssertion(ReferenceAssertion):
         assert len(var_0) == 42
     """
 
-    def __init__(self, source: vr.Reference, length: int):
+    def __init__(self, source: vr.Reference, length: int):  # noqa: D107
         super().__init__(source)
         self._length = length
 
@@ -253,15 +304,15 @@ class CollectionLengthAssertion(ReferenceAssertion):
         """
         return self._length
 
-    def accept(self, visitor: AssertionVisitor) -> None:
+    def accept(self, visitor: AssertionVisitor) -> None:  # noqa: D102
         visitor.visit_collection_length_assertion(self)
 
-    def clone(
+    def clone(  # noqa: D102
         self, memo: dict[vr.VariableReference, vr.VariableReference]
     ) -> CollectionLengthAssertion:
         return CollectionLengthAssertion(self.source.clone(memo), self._length)
 
-    def __eq__(self, other: Any) -> bool:
+    def __eq__(self, other: object) -> bool:
         return (
             isinstance(other, CollectionLengthAssertion)
             and self._source == other._source
@@ -292,10 +343,10 @@ class ExceptionAssertion(Assertion):
         # hold
         self._exception_type_name: str = exception_type_name
 
-    def accept(self, visitor: AssertionVisitor) -> None:
+    def accept(self, visitor: AssertionVisitor) -> None:  # noqa: D102
         visitor.visit_exception_assertion(self)
 
-    def clone(
+    def clone(  # noqa: D102
         self, memo: dict[vr.VariableReference, vr.VariableReference]
     ) -> Assertion:
         return ExceptionAssertion(self._module, self._exception_type_name)
@@ -318,7 +369,7 @@ class ExceptionAssertion(Assertion):
         """
         return self._module
 
-    def __eq__(self, other: Any) -> bool:
+    def __eq__(self, other: object) -> bool:
         return (
             isinstance(other, ExceptionAssertion)
             and self._exception_type_name == other._exception_type_name
@@ -363,9 +414,16 @@ class AssertionVisitor:
         """
 
     @abstractmethod
-    def visit_collection_length_assertion(
-        self, assertion: CollectionLengthAssertion
-    ) -> None:
+    def visit_isinstance_assertion(self, assertion: IsInstanceAssertion) -> None:
+        """Visit an isInstance assertion.
+
+        Args:
+            assertion: the visited assertion
+
+        """
+
+    @abstractmethod
+    def visit_collection_length_assertion(self, assertion: CollectionLengthAssertion) -> None:
         """Visit a collection length assertion.
 
         Args:
