@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides an output trace."""
+
 from __future__ import annotations
 
 import dataclasses
@@ -26,7 +27,7 @@ class AssertionTrace:
     def __init__(self) -> None:
         """Create new assertion trace."""
         # One entry per statement, i.e., the assertions after executing that statement.
-        self._trace: dict[int, OrderedSet[ass.Assertion]] = defaultdict(OrderedSet)
+        self.trace: dict[int, OrderedSet[ass.Assertion]] = defaultdict(OrderedSet)
 
     def add_entry(self, position: int, assertion: ass.Assertion) -> None:
         """Add an entry to this trace.
@@ -36,7 +37,7 @@ class AssertionTrace:
                 state is observed.
             assertion: the made assertion.
         """
-        self._trace[position].add(assertion)
+        self.trace[position].add(assertion)
 
     def get_assertions(self, statement: stmt.Statement) -> OrderedSet[ass.Assertion]:
         """Get all assertions contained within this trace for the given statement.
@@ -49,8 +50,8 @@ class AssertionTrace:
             All assertions in this trace for the given statement.
         """
         position = statement.get_position()
-        if position in self._trace:
-            return OrderedSet(self._trace[position])
+        if position in self.trace:
+            return OrderedSet(self.trace[position])
         return OrderedSet()
 
     def get_all_assertions(self) -> dict[int, OrderedSet[ass.Assertion]]:
@@ -60,11 +61,11 @@ class AssertionTrace:
             A dict that maps every statement position to all recorded assertions.
 
         """
-        return dict(self._trace)
+        return dict(self.trace)
 
     def clear(self) -> None:
         """Clear this trace."""
-        self._trace.clear()
+        self.trace.clear()
 
     def clone(self) -> AssertionTrace:
         """Clone this trace.
@@ -73,12 +74,18 @@ class AssertionTrace:
             a clone of this trace.
         """
         copy: AssertionTrace = AssertionTrace()
-        for stmt_key, stmt_value in self._trace.items():
-            copy._trace[stmt_key] = OrderedSet()
+        for stmt_key, stmt_value in self.trace.items():
+            copy.trace[stmt_key] = OrderedSet()
             for entry in stmt_value:
-                copy._trace[stmt_key].add(entry)
+                copy.trace[stmt_key].add(entry)
         return copy
 
+    def __eq__(self, other: object) -> bool:
+        return isinstance(other, AssertionTrace) and self.trace == other.trace
+
+    def __hash__(self) -> int:
+        return hash(self.trace)
+
 
 @dataclasses.dataclass
 class AssertionVerificationTrace:
@@ -107,6 +114,7 @@ class AssertionVerificationTrace:
 
     def was_violated(self, stmt_idx: int, assertion_idx: int) -> bool:
         """Was the assertion at the given position violated?
+
         This may happen because the assertion failed or another error occurred.
 
         Args:
@@ -118,6 +126,4 @@ class AssertionVerificationTrace:
         """
         if stmt_idx in self.failed and assertion_idx in self.failed[stmt_idx]:
             return True
-        if stmt_idx in self.error and assertion_idx in self.error[stmt_idx]:
-            return True
-        return False
+        return bool(stmt_idx in self.error and assertion_idx in self.error[stmt_idx])
