@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -9,14 +9,15 @@
 This module provides the main entry location for the program execution from the command
 line.
 """
+
 from __future__ import annotations
 
-import argparse
 import logging
 import os
 import sys
 
 from pathlib import Path
+from typing import TYPE_CHECKING
 
 import simple_parsing
 
@@ -29,6 +30,11 @@ import pynguin.configuration as config
 from pynguin.__version__ import __version__
 from pynguin.generator import run_pynguin
 from pynguin.generator import set_configuration
+from pynguin.utils.configuration_writer import write_configuration
+
+
+if TYPE_CHECKING:
+    import argparse
 
 
 def _create_argument_parser() -> argparse.ArgumentParser:
@@ -37,9 +43,7 @@ def _create_argument_parser() -> argparse.ArgumentParser:
         description="Pynguin is an automatic unit test generation framework for Python",
         fromfile_prefix_chars="@",
     )
-    parser.add_argument(
-        "--version", action="version", version="%(prog)s " + __version__
-    )
+    parser.add_argument("--version", action="version", version="%(prog)s " + __version__)
     parser.add_argument(
         "-v",
         "--verbose",
@@ -57,6 +61,12 @@ def _create_argument_parser() -> argparse.ArgumentParser:
         default=False,
         help="Don't use rich for nicer consoler output.",
     )
+    parser.add_argument(
+        "--log-file",
+        "--log_file",
+        help="Path to an optional log file.",
+        type=Path,
+    )
     parser.add_arguments(config.Configuration, dest="config")
 
     return parser
@@ -104,8 +114,7 @@ def _parse_comma_separated_option(arguments: list[str], option: str) -> list[str
     if "," not in arguments[index + 1]:
         return arguments
     variables = arguments[index + 1].split(",")
-    output = arguments[: index + 1] + variables + arguments[index + 2 :]
-    return output
+    return arguments[: index + 1] + variables + arguments[index + 2 :]
 
 
 def _setup_output_path(output_path: str) -> None:
@@ -116,29 +125,33 @@ def _setup_output_path(output_path: str) -> None:
 
 def _setup_logging(
     verbosity: int,
-    no_rich: bool,
+    no_rich: bool,  # noqa: FBT001
+    log_file: Path | None,
 ) -> Console | None:
     level = logging.WARNING
+    if log_file is not None:
+        level = logging.INFO
     if verbosity == 1:
         level = logging.INFO
     if verbosity >= 2:
         level = logging.DEBUG
 
     console = None
+    handler: logging.Handler
     if no_rich:
-        handler: logging.Handler = logging.StreamHandler()
+        handler = logging.StreamHandler()
     else:
         install()
         console = Console(tab_size=4)
-        handler = RichHandler(
-            rich_tracebacks=True, log_time_format="[%X]", console=console
-        )
+        handler = RichHandler(rich_tracebacks=True, log_time_format="[%X]", console=console)
         handler.setFormatter(logging.Formatter("%(message)s"))
 
+    if log_file is not None:
+        handler = logging.FileHandler(log_file)
+
     logging.basicConfig(
         level=level,
-        format="%(asctime)s [%(levelname)s]"
-        "(%(name)s:%(funcName)s:%(lineno)d): %(message)s",
+        format="%(asctime)s [%(levelname)s](%(name)s:%(funcName)s:%(lineno)d): %(message)s",
         datefmt="[%X]",
         handlers=[handler],
     )
@@ -182,12 +195,17 @@ to see why this happens and what you must do to prevent it."""
 
     argument_parser = _create_argument_parser()
     parsed = argument_parser.parse_args(argv)
-    # pylint: disable=no-member
+
     _setup_output_path(parsed.config.test_case_output.output_path)
-    # pylint: disable=no-member
-    console = _setup_logging(parsed.verbosity, parsed.no_rich)
+
+    console = _setup_logging(
+        verbosity=parsed.verbosity,
+        no_rich=parsed.no_rich,
+        log_file=parsed.log_file,
+    )
 
     set_configuration(parsed.config)
+    write_configuration()
     if console is not None:
         with console.status("Running Pynguin..."):
             return run_pynguin().value
@@ -196,4 +214,8 @@ to see why this happens and what you must do to prevent it."""
 
 
 if __name__ == "__main__":
+    import multiprocess as mp
+
+    mp.set_start_method("spawn")
+
     sys.exit(main(sys.argv))
