@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides a configuration interface for the test generator."""
+
 import dataclasses
 import enum
 import time
@@ -36,6 +37,11 @@ class Algorithm(str, enum.Enum):
     test case generation as a many-objective optimisation problem with dynamic selection
     of the targets.  TSE vol. 44 issue 2)."""
 
+    LLM = "LLM"
+    """Query a large language model for tests.  NOTE: This does not execute the usual test and
+    assertion generation stages of Pynguin but queries an LLM and writes the resulting test cases to
+    the output file."""
+
     MIO = "MIO"
     """The MIO test suite generation algorithm (cf. Andrea Arcuri. Many Independent
     Objective (MIO) Algorithm for Test Suite Generation.  Proc. SBSE 2017)."""
@@ -44,6 +50,9 @@ class Algorithm(str, enum.Enum):
     """The many-objective sorting algorithm (cf. Panichella et al. Reformulating Branch
     Coverage as a Many-Objective Optimization Problem.  Proc. ICST 2015)."""
 
+    LLMOSA = "LLMOSA"
+    """The many-objective sorting algorithm with LLM."""
+
     RANDOM = "RANDOM"
     """A feedback-direct random test generation approach similar to the algorithm
     proposed by Randoop (cf. Pacheco et al. Feedback-directed random test generation.
@@ -71,6 +80,9 @@ class Algorithm(str, enum.Enum):
 class AssertionGenerator(str, enum.Enum):
     """Different approaches for assertion generation supported by Pynguin."""
 
+    LLM = "LLM"
+    """Use Large Language Model(LLM) for assertion generation."""
+
     MUTATION_ANALYSIS = "MUTATION_ANALYSIS"
     """Use the mutation analysis approach for assertion generation."""
 
@@ -85,8 +97,9 @@ class AssertionGenerator(str, enum.Enum):
 
 
 class MutationStrategy(str, enum.Enum):
-    """Different strategies for creating mutants when using the MUTATION_ANALYSIS
-    approach for assertion generation.
+    """Different strategies for creating mutants.
+
+    Only respected when using the MUTATION_ANALYSIS approach for assertion generation.
     """
 
     FIRST_ORDER_MUTANTS = "FIRST_ORDER_MUTANTS"
@@ -113,6 +126,38 @@ class MutationStrategy(str, enum.Enum):
     IEEE Transactions on SE 39.4 2013)"""
 
 
+class MinimizationDirection(str, enum.Enum):
+    """Directions for test case minimization.
+
+    Either start with the first statement and try removing it and all forward dependent
+    statements, or start with the last statement and try removing it and all backward
+    dependent statements.
+    """
+
+    FORWARD = "FORWARD"
+    """Apply forward minimization, which removes statements from the beginning to the end."""
+
+    BACKWARD = "BACKWARD"
+    """Apply backward minimization, which removes statements from the end to the beginning."""
+
+
+class MinimizationStrategy(str, enum.Enum):
+    """Different strategies for minimizing test cases."""
+
+    NONE = "NONE"
+    """Do not apply any minimization."""
+
+    CASE = "CASE"
+    """Apply minimization at the test case level."""
+
+    SUITE = "SUITE"
+    """Apply minimization at the test suite level, i.e., remove entire test cases that
+    do not contribute to the coverage of the module under test."""
+
+    COMBINED = "COMBINED"
+    """Apply minimization at the test case level but check against all test cases in the suite."""
+
+
 class TypeInferenceStrategy(str, enum.Enum):
     """The different available type-inference strategies."""
 
@@ -160,7 +205,6 @@ class Selection(str, enum.Enum):
     """Tournament selection.  Use `tournament_size` to set size."""
 
 
-# pylint:disable=too-many-instance-attributes
 @dataclasses.dataclass
 class StatisticsOutputConfiguration:
     """Configuration related to output."""
@@ -181,9 +225,7 @@ class StatisticsOutputConfiguration:
     """Interpolate timeline values"""
 
     coverage_metrics: list[CoverageMetric] = dataclasses.field(
-        default_factory=lambda: [
-            CoverageMetric.BRANCH,
-        ]
+        default_factory=lambda: [CoverageMetric.BRANCH]
     )
     """List of coverage metrics that are optimised during the search"""
 
@@ -218,6 +260,18 @@ class StatisticsOutputConfiguration:
     should be exported? Expects positive integers."""
 
 
+@dataclasses.dataclass
+class Minimization:
+    """Configuration for test case minimization."""
+
+    test_case_minimization_strategy: MinimizationStrategy = MinimizationStrategy.CASE
+    """Strategy to apply for minimizing test cases to remove redundant statements
+    while preserving fitness."""
+
+    test_case_minimization_direction: MinimizationDirection = MinimizationDirection.BACKWARD
+    """Direction to apply for minimizing test cases."""
+
+
 @dataclasses.dataclass
 class TestCaseOutputConfiguration:
     """Configuration related to test case output."""
@@ -225,6 +279,11 @@ class TestCaseOutputConfiguration:
     output_path: str
     """Path to an output folder for the generated test cases."""
 
+    crash_path: str = ""
+    """Path to an output folder for the generated test cases that caused a crash.
+    Only works when running in a subprocess. If set to "", the are stored in the
+    output_path."""
+
     export_strategy: ExportStrategy = ExportStrategy.PY_TEST
     """The export strategy determines for which test-runner system the
     generated tests should fit."""
@@ -251,6 +310,14 @@ class TestCaseOutputConfiguration:
     """Should the results be post processed? For example, truncate test cases after
     statements that raise an exception."""
 
+    minimization: Minimization = dataclasses.field(
+        default_factory=lambda: Minimization(
+            test_case_minimization_strategy=MinimizationStrategy.CASE,
+            test_case_minimization_direction=MinimizationDirection.BACKWARD,
+        )
+    )
+    """Strategy to apply for minimizing test cases."""
+
     float_precision: float = 0.01
     """Precision to use in float comparisons and assertions"""
 
@@ -258,7 +325,6 @@ class TestCaseOutputConfiguration:
     """Format the generated test cases using black."""
 
 
-# pylint:disable=too-many-instance-attributes
 @dataclasses.dataclass
 class SeedingConfiguration:
     """Configuration related to seeding."""
@@ -377,20 +443,68 @@ class TypeInferenceConfiguration:
     type_inference_strategy: TypeInferenceStrategy = TypeInferenceStrategy.TYPE_HINTS
     """The strategy for type-inference that shall be used"""
 
-    type_tracing: bool = False
-    """Trace usage of parameters with unknown types to improve type guesses."""
+    type_tracing: bool | float = 0.0
+    """Probability to trace usage of parameters with unknown types to improve type
+    guesses during test execution. Type tracing requires a separate second.
+    The value should be a float in [0,1]. Boolean is kept for backwards compatibility
+    as Python internally converts True to 1.0 and False to 0.0 anyways."""
 
-    type4py: bool = False
-    """Get type information from Type4Py."""
 
-    type4py_uri: str = "https://type4py.com/"
-    """URI of the Type4Py server.
-    Currently only for the module under test.
-    For example: http://localhost:5001/
-    See https://github.com/saltudelft/type4py/wiki"""
+@dataclasses.dataclass
+class PynguinMLConfiguration:
+    """Configurations for PynguinML, specifically for machine learning API testing."""
 
-    type4py_timeout: int = 10
-    """Read timeout when requesting data from the Type4Py API."""
+    ml_testing_enabled: bool = False
+    """
+    Enables ML-based test generation using API constraints. When set to True, PynguinML
+    will use constraints and other ML-specific behaviours during input generation.
+
+    Note: 'constraints_path' must also be set to locate the constraints files.
+    """
+
+    constraints_path: str = ""
+    """
+    Directory path to YAML/JSON constraints files.
+    """
+
+    dtype_mapping_path: str = ""
+    """
+    Path to a YAML/JSON file that maps library-specific datatypes to NumPy datatypes.
+    Used by PynguinML for input generation. Specify only valid NumPy dtypes without
+    the "np." prefix. Example: "torch.int32" -> "int32".
+    """
+
+    constructor_function: str = ""
+    """
+    Optional constructor function to build tensors (e.g., torch.tensor or
+    tensorflow.convert_to_tensor). Must accept a np.ndarray as input.
+    Note: Also set 'constructor_function_parameter' to specify which parameter
+    receives the np.ndarray.
+    """
+
+    constructor_function_parameter: str = ""
+    """
+    Name of the parameter in the constructor function that receives the np.ndarray.
+    Must be set when 'constructor_function' is specified.
+    """
+
+    max_ndim: int = 4
+    """
+    Maximum number of dimensions of the tensors.
+    If set too big, it can cause memory errors due to large tensor sizes.
+    """
+
+    max_shape_dim: int = 4
+    """
+    Maximum size of a dimension of a shape.
+    If set too big, it can cause memory errors due to large tensor sizes.
+    """
+
+    ignore_constraints_probability: float = 0.25
+    """
+    Probability of ignoring constraints and falling back to Pynguin's default
+    behavior when constructing statements.
+    """
 
 
 @dataclasses.dataclass
@@ -448,7 +562,7 @@ class TestCreationConfiguration:
     is used."""
 
     wrap_var_param_type_probability: float = 0.7
-    """Probability to wrap the type required for a *arg or **kwargs parameter
+    """Probability to wrap the type required for a ``*arg`` or ``**kwargs`` parameter
     in a list or dict, respectively. Expects values in [0,1]"""
 
     negate_type: float = 0.1
@@ -539,10 +653,14 @@ class SearchAlgorithmConfiguration:
     """Focus search by filtering out elements from the test cluster when
      they are fully covered."""
 
+    number_of_mutations: int = 1
+    """Number of mutations that should be applied in one breeding step."""
+
 
 @dataclasses.dataclass
 class StoppingConfiguration:
     """Configuration related to when Pynguin should stop.
+
     Note that these are mostly soft-limits rather than hard limits, because
     the search algorithms only check the condition at the start of each algorithm
     iteration.
@@ -566,12 +684,72 @@ class StoppingConfiguration:
     maximum_test_execution_timeout: int = 5
     """The maximum time (in seconds) after which a test case times out."""
 
+    maximum_coverage: int = 100
+    """The maximum percentage of coverage after which the generation shall stop."""
+
+    maximum_coverage_plateau: int = -1
+    """Maximum number of algorithm iterations without coverage change before the
+    algorithms stops."""
+
+    minimum_coverage: int = 100
+    """Minimum coverage for the plateau-based stopping condition.  Expects values larger
+    than 0 but less than 100 to activate the stopping condition; also requires the
+    setting of minimum_plateau_iterations."""
+
+    minimum_plateau_iterations: int = -1
+    """Minimum iterations without a coverage change to stop early.  Expects values
+    larger than 0; also requires the setting of minimum_coverage."""
+
+    maximum_memory: int = 3000
+    """Maximum memory usage in MB after which the generation shall stop.
+    Expects values in MB greater than 0 of -1 to disable the check."""
+
     test_execution_time_per_statement: int = 1
     """The time (in seconds) per statement that a test is allowed to run
     (up to maximum_test_execution_timeout)."""
 
 
-# pylint: disable=too-many-instance-attributes, pointless-string-statement
+@dataclasses.dataclass
+class LLMConfiguration:
+    """Configuration for the LLM."""
+
+    api_key: str = ""
+    """The api key to call OpenAI LLM with."""
+
+    model_name: str = "gpt-4o-mini"
+    """The OpenAI Model to use for completions."""
+
+    temperature: float = 0.8
+    """The temperature to use when querying the model.
+    The value must be from [0.0, 1.0]."""
+
+    hybrid_initial_population: bool = False
+    """Whether to include the LLM test cases in the initial population."""
+
+    llm_test_case_percentage: float = 0.5
+    """The percentage of LLM test cases in on the initial population. The value must
+     be from [0.0, 1.0]."""
+
+    enable_response_caching: bool = False
+    """Whether to enable caching for responses of the model."""
+
+    call_llm_for_uncovered_targets: bool = False
+    """Whether to call the LLM for the uncovered targets initially."""
+
+    coverage_threshold: float = 1
+    """The coverage threshold when to call the LLM for low-coverage targets.
+    The value must be from [0.0, 1.0]."""
+
+    call_llm_on_stall_detection: bool = False
+    """Whether to call the LLM for the uncovered targets in coverage stalls."""
+
+    max_plateau_len: int = 25
+    """The number of iterations to allow before soliciting the LLM."""
+
+    max_llm_interventions: int = 1
+    """The maximum number of allowed LLM interventions."""
+
+
 @dataclasses.dataclass
 class Configuration:
     """General configuration for the test generator."""
@@ -593,14 +771,13 @@ class Configuration:
     )
     """Statistic Output configuration."""
 
-    stopping: StoppingConfiguration = dataclasses.field(
-        default_factory=StoppingConfiguration
-    )
+    stopping: StoppingConfiguration = dataclasses.field(default_factory=StoppingConfiguration)
     """Stopping configuration."""
 
-    seeding: SeedingConfiguration = dataclasses.field(
-        default_factory=SeedingConfiguration
-    )
+    large_language_model: LLMConfiguration = dataclasses.field(default_factory=LLMConfiguration)
+    """Large Language Model(LLM) configuration."""
+
+    seeding: SeedingConfiguration = dataclasses.field(default_factory=SeedingConfiguration)
     """Seeding configuration."""
 
     type_inference: TypeInferenceConfiguration = dataclasses.field(
@@ -608,6 +785,9 @@ class Configuration:
     )
     """Type inference configuration."""
 
+    pynguinml: PynguinMLConfiguration = dataclasses.field(default_factory=PynguinMLConfiguration)
+    """PynguinML configuration."""
+
     test_creation: TestCreationConfiguration = dataclasses.field(
         default_factory=TestCreationConfiguration
     )
@@ -624,6 +804,19 @@ class Configuration:
     random: RandomConfiguration = dataclasses.field(default_factory=RandomConfiguration)
     """Configuration used for the RANDOM algorithm."""
 
+    ignore_modules: list[str] = dataclasses.field(default_factory=list)
+    """Ignore the modules specified here from the module analysis."""
+
+    ignore_methods: list[str] = dataclasses.field(default_factory=list)
+    """Ignore the methods specified here from the module analysis."""
+
+    subprocess: bool = False
+    """Run the test generation in a subprocess."""
+
+    subprocess_if_recommended: bool = True
+    """Use the subprocess mode if the SUT analysis recommends it based on used
+    C extension modules."""
+
 
 # Singleton instance of the configuration.
 configuration = Configuration(
