@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Factory for chromosome used by the genetic algorithm."""
+
 from abc import abstractmethod
 from typing import Generic
 from typing import TypeVar
@@ -12,10 +13,9 @@ from typing import TypeVar
 import pynguin.ga.chromosome as chrom
 
 
-T = TypeVar("T", bound=chrom.Chromosome)  # pylint: disable=invalid-name
+T = TypeVar("T", bound=chrom.Chromosome)
 
 
-# pylint: disable=too-few-public-methods
 class ChromosomeFactory(Generic[T]):
     """A factory that provides new chromosomes."""
 
