@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides classes for computations on chromosomes, e.g., fitness and coverage."""
+
 from __future__ import annotations
 
 import abc
@@ -13,42 +14,44 @@ import math
 import statistics
 
 from abc import abstractmethod
-from collections.abc import Callable
 from typing import TYPE_CHECKING
 from typing import Any
 from typing import TypeVar
 
-import pynguin.utils.opcodes as op
-
+from pynguin.instrumentation import version
+from pynguin.instrumentation.tracer import ExecutionTrace
 from pynguin.slicer.dynamicslicer import AssertionSlicer
 from pynguin.slicer.dynamicslicer import DynamicSlicer
-from pynguin.testcase.execution import ExecutionTrace
-from pynguin.testcase.statement import Statement
 
 
 if TYPE_CHECKING:
+    from collections.abc import Callable
+
+    from pynguin.ga.testcasechromosome import TestCaseChromosome
+    from pynguin.ga.testsuitechromosome import TestSuiteChromosome
+    from pynguin.instrumentation.tracer import SubjectProperties
     from pynguin.slicer.dynamicslicer import SlicingCriterion
+    from pynguin.slicer.dynamicslicer import UniqueInstruction
     from pynguin.testcase.execution import AbstractTestCaseExecutor
     from pynguin.testcase.execution import ExecutionResult
-    from pynguin.testcase.execution import SubjectProperties
+    from pynguin.testcase.statement import Statement
 
 
 @dataclasses.dataclass(eq=False)
-class ChromosomeComputation(abc.ABC):  # pylint:disable=too-few-public-methods
+class ChromosomeComputation(abc.ABC):
     """An abstract computation on chromosomes."""
 
     _executor: AbstractTestCaseExecutor
     """Executor that will be used by the computation to execute chromosomes."""
 
 
-class TestCaseChromosomeComputation(
-    ChromosomeComputation, metaclass=abc.ABCMeta
-):  # pylint:disable=too-few-public-methods
+class TestCaseChromosomeComputation(ChromosomeComputation, abc.ABC):
     """A function that computes something on a test case chromosome."""
 
-    def _run_test_case_chromosome(self, individual) -> ExecutionResult:
-        """Runs a test suite and updates the execution results for
-        all test cases that were changed.
+    def _run_test_case_chromosome(self, individual: TestCaseChromosome) -> ExecutionResult:
+        """Runs a test suite and updates the execution results.
+
+        Updates all test cases that were changed.
 
         Args:
             individual: The individual to run
@@ -57,23 +60,20 @@ class TestCaseChromosomeComputation(
             A list of execution results
         """
         if individual.changed or individual.get_last_execution_result() is None:
-            individual.set_last_execution_result(
-                self._executor.execute(individual.test_case)
-            )
+            individual.set_last_execution_result(self._executor.execute(individual.test_case))
             individual.changed = False
         result = individual.get_last_execution_result()
         assert result is not None
         return result
 
 
-class TestSuiteChromosomeComputation(
-    ChromosomeComputation, metaclass=abc.ABCMeta
-):  # pylint:disable=too-few-public-methods
+class TestSuiteChromosomeComputation(ChromosomeComputation, abc.ABC):
     """A function that computes something on a test suite chromosome."""
 
-    def _run_test_suite_chromosome(self, individual) -> list[ExecutionResult]:
-        """Runs a test suite and updates the execution results for
-        all test cases that were changed.
+    def _run_test_suite_chromosome(self, individual: TestSuiteChromosome) -> list[ExecutionResult]:
+        """Runs a test suite and updates the execution results.
+
+        Updates all test cases that were changed.
 
         Args:
             individual: The individual to run
@@ -81,23 +81,42 @@ class TestSuiteChromosomeComputation(
         Returns:
             A list of execution results
         """
-        results: list[ExecutionResult] = []
-        for test_case_chromosome in individual.test_case_chromosomes:
-            if (
+        test_case_chromosomes = tuple(
+            (
+                test_case_chromosome,
                 test_case_chromosome.changed
-                or test_case_chromosome.get_last_execution_result() is None
-            ):
-                test_case_chromosome.set_last_execution_result(
-                    self._executor.execute(test_case_chromosome.test_case)
-                )
+                or test_case_chromosome.get_last_execution_result() is None,
+            )
+            for test_case_chromosome in individual.test_case_chromosomes
+        )
+
+        changed_results_iterator = iter(
+            self._executor.execute_multiple(
+                test_case_chromosome.test_case
+                for test_case_chromosome, changed in test_case_chromosomes
+                if changed
+            )
+        )
+
+        results: list[ExecutionResult] = []
+
+        for test_case_chromosome, changed in test_case_chromosomes:
+            result: ExecutionResult | None
+            if changed:
+                result = next(changed_results_iterator)
+                test_case_chromosome.set_last_execution_result(result)
                 test_case_chromosome.changed = False
                 # If we execute a suite which in turn executes it's test cases,
                 # then we have to invalidate the values of the test cases, because
                 # the test case is no longer aware that it was changed.
                 test_case_chromosome.invalidate_cache()
-            result = test_case_chromosome.get_last_execution_result()
+            else:
+                result = test_case_chromosome.get_last_execution_result()
+
             assert result is not None
+
             results.append(result)
+
         return results
 
 
@@ -118,6 +137,7 @@ class FitnessFunction:
     @abstractmethod
     def compute_is_covered(self, individual) -> bool:
         """Compute if the goal of this fitness function is covered.
+
         This computation is usually cheaper than computing the fitness, because
         we are not interested in the distance, but only a boolean result.
 
@@ -137,12 +157,10 @@ class FitnessFunction:
         """
 
 
-class TestCaseFitnessFunction(
-    TestCaseChromosomeComputation, FitnessFunction, metaclass=abc.ABCMeta
-):
+class TestCaseFitnessFunction(TestCaseChromosomeComputation, FitnessFunction, abc.ABC):
     """Base class for test case fitness functions."""
 
-    def __init__(self, executor, code_object_id: int):
+    def __init__(self, executor, code_object_id: int):  # noqa: D107
         super().__init__(executor)
         self._code_object_id = code_object_id
 
@@ -159,38 +177,32 @@ class TestCaseFitnessFunction(
 class BranchDistanceTestCaseFitnessFunction(TestCaseFitnessFunction):
     """A fitness function based on branch distances and entered code objects."""
 
-    def compute_fitness(self, individual) -> float:
+    def compute_fitness(self, individual) -> float:  # noqa: D102
         result = self._run_test_case_chromosome(individual)
         merged_trace = analyze_results([result])
-        tracer = self._executor.tracer
 
-        return compute_branch_distance_fitness(
-            merged_trace, tracer.get_subject_properties()
-        )
+        return compute_branch_distance_fitness(merged_trace, self._executor.subject_properties)
 
-    def compute_is_covered(self, individual) -> bool:
+    def compute_is_covered(self, individual) -> bool:  # noqa: D102
         result = self._run_test_case_chromosome(individual)
         merged_trace = analyze_results([result])
-        tracer = self._executor.tracer
 
         return compute_branch_distance_fitness_is_covered(
-            merged_trace, tracer.get_subject_properties()
+            merged_trace, self._executor.subject_properties
         )
 
-    def is_maximisation_function(self) -> bool:
+    def is_maximisation_function(self) -> bool:  # noqa: D102
         return False
 
 
-class TestSuiteFitnessFunction(
-    TestSuiteChromosomeComputation, FitnessFunction, metaclass=abc.ABCMeta
-):
+class TestSuiteFitnessFunction(TestSuiteChromosomeComputation, FitnessFunction, abc.ABC):
     """Base class for test suite fitness functions."""
 
 
 class BranchDistanceTestSuiteFitnessFunction(TestSuiteFitnessFunction):
     """A fitness function based on branch distances and entered code objects."""
 
-    def __init__(self, executor):
+    def __init__(self, executor):  # noqa: D107
         super().__init__(executor)
         self._excluded_code_objects: set[int] = set()
         self._excluded_true_predicates: set[int] = set()
@@ -199,7 +211,9 @@ class BranchDistanceTestSuiteFitnessFunction(TestSuiteFitnessFunction):
     def restrict(
         self, exclude_code: set[int], exclude_true: set[int], exclude_false: set[int]
     ) -> None:
-        """Restrict this fitness function, i.e., which branches/code objects it
+        """Restrict this fitness function.
+
+        Restricts the fitness function with respect to the branches/code objects it
         considers.
 
         Args:
@@ -212,89 +226,86 @@ class BranchDistanceTestSuiteFitnessFunction(TestSuiteFitnessFunction):
         self._excluded_true_predicates.update(exclude_true)
         self._excluded_false_predicates.update(exclude_false)
 
-    def compute_fitness(self, individual) -> float:
+    def compute_fitness(self, individual) -> float:  # noqa: D102
         results = self._run_test_suite_chromosome(individual)
         merged_trace = analyze_results(results)
-        tracer = self._executor.tracer
 
         return compute_branch_distance_fitness(
             merged_trace,
-            tracer.get_subject_properties(),
+            self._executor.subject_properties,
             self._excluded_code_objects,
             self._excluded_true_predicates,
             self._excluded_false_predicates,
         )
 
-    def compute_is_covered(self, individual) -> bool:
+    def compute_is_covered(self, individual) -> bool:  # noqa: D102
         results = self._run_test_suite_chromosome(individual)
         merged_trace = analyze_results(results)
-        tracer = self._executor.tracer
 
         return compute_branch_distance_fitness_is_covered(
             merged_trace,
-            tracer.get_subject_properties(),
+            self._executor.subject_properties,
             self._excluded_code_objects,
             self._excluded_true_predicates,
             self._excluded_false_predicates,
         )
 
-    def is_maximisation_function(self) -> bool:
+    def is_maximisation_function(self) -> bool:  # noqa: D102
         return False
 
 
 class LineTestSuiteFitnessFunction(TestSuiteFitnessFunction):
     """A fitness function based on lines covered and entered code objects."""
 
-    def compute_fitness(self, individual) -> float:
+    def compute_fitness(self, individual) -> float:  # noqa: D102
         results = self._run_test_suite_chromosome(individual)
         merged_trace = analyze_results(results)
-        tracer = self._executor.tracer
-        existing_lines = tracer.get_subject_properties().existing_lines
+
+        existing_lines = self._executor.subject_properties.existing_lines
         return len(existing_lines) - len(merged_trace.covered_line_ids)
 
-    def compute_is_covered(self, individual) -> bool:
+    def compute_is_covered(self, individual) -> bool:  # noqa: D102
         results = self._run_test_suite_chromosome(individual)
         merged_trace = analyze_results(results)
-        tracer = self._executor.tracer
 
         return compute_line_coverage_fitness_is_covered(
             merged_trace,
-            tracer.get_subject_properties(),
+            self._executor.subject_properties,
         )
 
-    def is_maximisation_function(self) -> bool:
+    def is_maximisation_function(self) -> bool:  # noqa: D102
         return False
 
 
 class StatementCheckedTestSuiteFitnessFunction(TestSuiteFitnessFunction):
-    """A fitness function based on lines included in the backward
-    slice of each statement of a test suite.
+    """A fitness function for the checked statement coverage of test suites.
+
+    A fitness function based on lines included in the backward slice of each statement
+    of a test suite.
     """
 
-    def compute_fitness(self, individual) -> float:
+    def compute_fitness(self, individual) -> float:  # noqa: D102
         results = self._run_test_suite_chromosome(individual)
         merged_trace = analyze_results(results)
-        tracer = self._executor.tracer
 
-        return len(tracer.get_subject_properties().existing_lines) - len(
+        return len(self._executor.subject_properties.existing_lines) - len(
             merged_trace.checked_lines
         )
 
-    def compute_is_covered(self, individual) -> bool:
+    def compute_is_covered(self, individual) -> bool:  # noqa: D102
         results = self._run_test_suite_chromosome(individual)
         merged_trace = analyze_results(results)
-        tracer = self._executor.tracer
 
         return compute_checked_coverage_statement_fitness_is_covered(
             merged_trace,
-            tracer.get_subject_properties(),
+            self._executor.subject_properties,
         )
 
-    def is_maximisation_function(self) -> bool:
+    def is_maximisation_function(self) -> bool:  # noqa: D102
         return False
 
 
-class CoverageFunction:  # pylint:disable=too-few-public-methods
+class CoverageFunction:
     """Interface for a coverage function."""
 
     @abstractmethod
@@ -309,68 +320,62 @@ class CoverageFunction:  # pylint:disable=too-few-public-methods
         """
 
 
-# pylint: disable=too-few-public-methods
-class TestSuiteCoverageFunction(
-    TestSuiteChromosomeComputation, CoverageFunction, metaclass=abc.ABCMeta
-):
+class TestSuiteCoverageFunction(TestSuiteChromosomeComputation, CoverageFunction, abc.ABC):
     """Base class for all coverage functions that act on test suite level."""
 
 
-class TestCaseCoverageFunction(
-    TestCaseChromosomeComputation, CoverageFunction, metaclass=abc.ABCMeta
-):  # pylint: disable=too-few-public-methods
+class TestCaseCoverageFunction(TestCaseChromosomeComputation, CoverageFunction, abc.ABC):
     """Base class for all coverage functions that act on test case level."""
 
 
 class TestSuiteBranchCoverageFunction(TestSuiteCoverageFunction):
     """Computes branch coverage on test suites."""
 
-    def compute_coverage(self, individual) -> float:
+    def compute_coverage(self, individual) -> float:  # noqa: D102
         results = self._run_test_suite_chromosome(individual)
         merged_trace = analyze_results(results)
-        tracer = self._executor.tracer
-        return compute_branch_coverage(merged_trace, tracer.get_subject_properties())
+
+        return compute_branch_coverage(merged_trace, self._executor.subject_properties)
 
 
 class TestCaseBranchCoverageFunction(TestCaseCoverageFunction):
     """Computes branch coverage on test cases."""
 
-    def compute_coverage(self, individual) -> float:
+    def compute_coverage(self, individual) -> float:  # noqa: D102
         result = self._run_test_case_chromosome(individual)
         merged_trace = analyze_results([result])
-        tracer = self._executor.tracer
-        return compute_branch_coverage(merged_trace, tracer.get_subject_properties())
+
+        return compute_branch_coverage(merged_trace, self._executor.subject_properties)
 
 
 class TestSuiteLineCoverageFunction(TestSuiteCoverageFunction):
     """Computes line coverage on test suites."""
 
-    def compute_coverage(self, individual) -> float:
+    def compute_coverage(self, individual) -> float:  # noqa: D102
         results = self._run_test_suite_chromosome(individual)
         merged_trace = analyze_results(results)
-        tracer = self._executor.tracer
-        return compute_line_coverage(merged_trace, tracer.get_subject_properties())
+
+        return compute_line_coverage(merged_trace, self._executor.subject_properties)
 
 
 class TestCaseLineCoverageFunction(TestCaseCoverageFunction):
     """Computes line coverage on test cases."""
 
-    def compute_coverage(self, individual) -> float:
+    def compute_coverage(self, individual) -> float:  # noqa: D102
         result = self._run_test_case_chromosome(individual)
         merged_trace = analyze_results([result])
-        tracer = self._executor.tracer
-        return compute_line_coverage(merged_trace, tracer.get_subject_properties())
+
+        return compute_line_coverage(merged_trace, self._executor.subject_properties)
 
 
 class TestSuiteStatementCheckedCoverageFunction(TestSuiteCoverageFunction):
     """Computes checked coverage on the statements of test suites."""
 
-    def compute_coverage(self, individual) -> float:
+    def compute_coverage(self, individual) -> float:  # noqa: D102
         results = self._run_test_suite_chromosome(individual)
         merged_trace = analyze_results(results)
-        tracer = self._executor.tracer
 
-        existing = len(tracer.get_subject_properties().existing_lines)
+        existing = len(self._executor.subject_properties.existing_lines)
 
         if existing == 0:
             # Nothing to cover => everything is covered.
@@ -385,11 +390,11 @@ class TestSuiteStatementCheckedCoverageFunction(TestSuiteCoverageFunction):
 class TestCaseStatementCheckedCoverageFunction(TestCaseCoverageFunction):
     """Computes checked coverage on the statements of test cases."""
 
-    def compute_coverage(self, individual) -> float:
+    def compute_coverage(self, individual) -> float:  # noqa: D102
         result = self._run_test_case_chromosome(individual)
         merged_trace = analyze_results([result])
-        tracer = self._executor.tracer
-        existing = len(tracer.get_subject_properties().existing_lines)
+
+        existing = len(self._executor.subject_properties.existing_lines)
 
         if existing == 0:
             # Nothing to cover => everything is covered.
@@ -404,51 +409,42 @@ class TestCaseStatementCheckedCoverageFunction(TestCaseCoverageFunction):
 class TestSuiteAssertionCheckedCoverageFunction(TestSuiteCoverageFunction):
     """Computes checked coverage on test suites with assertions."""
 
-    def compute_coverage(self, individual) -> float:
+    def compute_coverage(self, individual) -> float:  # noqa: D102
         results = self._run_test_suite_chromosome(individual)
         merged_trace = analyze_results(results)
-        tracer = self._executor.tracer
-        return compute_assertion_checked_coverage(
-            merged_trace, tracer.get_subject_properties()
-        )
+
+        return compute_assertion_checked_coverage(merged_trace, self._executor.subject_properties)
 
 
 class TestCaseAssertionCheckedCoverageFunction(TestCaseCoverageFunction):
     """Computes checked coverage on test cases with assertions."""
 
-    def compute_coverage(self, individual) -> float:
+    def compute_coverage(self, individual) -> float:  # noqa: D102
         result = self._run_test_case_chromosome(individual)
         merged_trace = analyze_results([result])
-        tracer = self._executor.tracer
-        return compute_assertion_checked_coverage(
-            merged_trace, tracer.get_subject_properties()
-        )
+
+        return compute_assertion_checked_coverage(merged_trace, self._executor.subject_properties)
 
 
 class ComputationCache:
     """Caches computation results and computes values on demand."""
 
-    def __init__(
+    def __init__(  # noqa: D107
         self,
         chromosome,
+        *,
         fitness_functions: list[FitnessFunction] | None = None,
         coverage_functions: list[CoverageFunction] | None = None,
         fitness_cache: dict[FitnessFunction, float] | None = None,
         is_covered_cache: dict[FitnessFunction, bool] | None = None,
         coverage_cache: dict[CoverageFunction, float] | None = None,
-    ):  # pylint:disable=too-many-arguments
+    ):
         self._chromosome = chromosome
-        self._fitness_functions = fitness_functions if fitness_functions else []
-        self._coverage_functions = coverage_functions if coverage_functions else []
-        self._fitness_cache: dict[FitnessFunction, float] = (
-            fitness_cache if fitness_cache else {}
-        )
-        self._is_covered_cache: dict[FitnessFunction, bool] = (
-            is_covered_cache if is_covered_cache else {}
-        )
-        self._coverage_cache: dict[CoverageFunction, float] = (
-            coverage_cache if coverage_cache else {}
-        )
+        self._fitness_functions = fitness_functions or []
+        self._coverage_functions = coverage_functions or []
+        self._fitness_cache: dict[FitnessFunction, float] = fitness_cache or {}
+        self._is_covered_cache: dict[FitnessFunction, bool] = is_covered_cache or {}
+        self._coverage_cache: dict[CoverageFunction, float] = coverage_cache or {}
 
     def clone(self, new_chromosome) -> ComputationCache:
         """Create a deep copy of this cache.
@@ -461,11 +457,11 @@ class ComputationCache:
         """
         return ComputationCache(
             new_chromosome,
-            list(self._fitness_functions),
-            list(self._coverage_functions),
-            dict(self._fitness_cache),
-            dict(self._is_covered_cache),
-            dict(self._coverage_cache),
+            fitness_functions=list(self._fitness_functions),
+            coverage_functions=list(self._coverage_functions),
+            fitness_cache=dict(self._fitness_cache),
+            is_covered_cache=dict(self._is_covered_cache),
+            coverage_cache=dict(self._coverage_cache),
         )
 
     def get_fitness_functions(self) -> list[FitnessFunction]:
@@ -485,9 +481,9 @@ class ComputationCache:
         Args:
             fitness_function: A fitness function
         """
-        assert (
-            not fitness_function.is_maximisation_function()
-        ), "Currently only minimization is supported"
+        assert not fitness_function.is_maximisation_function(), (
+            "Currently only minimization is supported"
+        )
         self._fitness_functions.append(fitness_function)
 
     def get_coverage_functions(self) -> list[CoverageFunction]:
@@ -542,10 +538,8 @@ class ComputationCache:
         for fitness_func in self._fitness_functions if only is None else (only,):
             if fitness_func not in self._fitness_cache:
                 new_value = fitness_func.compute_fitness(self._chromosome)
-                assert (
-                    not math.isnan(new_value)
-                    and not math.isinf(new_value)
-                    and new_value >= 0
+                assert (  # noqa: PT018
+                    not math.isnan(new_value) and not math.isinf(new_value) and new_value >= 0
                 ), f"Invalid fitness value {new_value}"
                 self._fitness_cache[fitness_func] = new_value
                 # When computing a minimising fitness value, we can also determine
@@ -562,7 +556,7 @@ class ComputationCache:
         for coverage_func in self._coverage_functions if only is None else (only,):
             if coverage_func not in self._coverage_cache:
                 new_value = coverage_func.compute_coverage(self._chromosome)
-                assert (
+                assert (  # noqa: PT018
                     not math.isnan(new_value)
                     and not math.isinf(new_value)
                     and (0 <= new_value <= 1)
@@ -713,14 +707,12 @@ def compute_branch_distance_fitness(
     exclude_code = set() if exclude_code is None else exclude_code
 
     # Check if all branch-less code objects were executed.
-    code_objects_missing: float = len(
-        subject_properties.branch_less_code_objects.difference(
-            trace.executed_code_objects, exclude_code
-        )
+    code_objects_missing: float = sum(
+        1.0
+        for code_object_id in subject_properties.branch_less_code_objects
+        if code_object_id not in trace.executed_code_objects and code_object_id not in exclude_code
     )
-    assert (
-        code_objects_missing >= 0.0
-    ), "Amount of non covered code objects cannot be negative"
+    assert code_objects_missing >= 0.0, "Amount of non covered code objects cannot be negative"
 
     # Handle None for branches.
     exclude_true = set() if exclude_true is None else exclude_true
@@ -730,16 +722,12 @@ def compute_branch_distance_fitness(
     predicate_fitness: float = 0.0
     for predicate in subject_properties.existing_predicates:
         if predicate not in exclude_true:
-            predicate_fitness += _predicate_fitness(
-                predicate, trace.true_distances, trace
-            )
+            predicate_fitness += _predicate_fitness(predicate, trace.true_distances, trace)
         if predicate not in exclude_false:
-            predicate_fitness += _predicate_fitness(
-                predicate, trace.false_distances, trace
-            )
+            predicate_fitness += _predicate_fitness(predicate, trace.false_distances, trace)
+
     assert predicate_fitness >= 0.0, "Predicate fitness cannot be negative."
-    total_fitness = code_objects_missing + predicate_fitness
-    return total_fitness
+    return code_objects_missing + predicate_fitness
 
 
 def _predicate_fitness(
@@ -747,10 +735,7 @@ def _predicate_fitness(
 ) -> float:
     if predicate in branch_distances and branch_distances[predicate] == 0.0:
         return 0.0
-    if (
-        predicate in trace.executed_predicates
-        and trace.executed_predicates[predicate] >= 2
-    ):
+    if predicate in trace.executed_predicates and trace.executed_predicates[predicate] >= 2:
         return normalise(branch_distances[predicate])
     return 1.0
 
@@ -778,13 +763,9 @@ def compute_branch_distance_fitness_is_covered(
     exclude_code = set() if exclude_code is None else exclude_code
 
     # Check if all branch-less code objects were executed.
-    if (
-        len(
-            subject_properties.branch_less_code_objects.difference(
-                trace.executed_code_objects, exclude_code
-            )
-        )
-        > 0
+    if any(
+        code_object_id not in trace.executed_code_objects and code_object_id not in exclude_code
+        for code_object_id in subject_properties.branch_less_code_objects
     ):
         return False
 
@@ -794,15 +775,9 @@ def compute_branch_distance_fitness_is_covered(
 
     # Check if all predicates are covered
     for predicate in subject_properties.existing_predicates:
-        if (
-            predicate not in exclude_true
-            and (predicate, 0.0) not in trace.true_distances
-        ):
+        if predicate not in exclude_true and (predicate, 0.0) not in trace.true_distances:
             return False
-        if (
-            predicate not in exclude_false
-            and (predicate, 0.0) not in trace.false_distances
-        ):
+        if predicate not in exclude_false and (predicate, 0.0) not in trace.false_distances:
             return False
     return True
 
@@ -837,11 +812,10 @@ def compute_checked_coverage_statement_fitness_is_covered(
     return len(trace.checked_lines) == len(subject_properties.existing_lines)
 
 
-def compute_branch_coverage(
-    trace: ExecutionTrace, subject_properties: SubjectProperties
-) -> float:
-    """Computes branch coverage on bytecode instructions which should equal
-    decision coverage on source.
+def compute_branch_coverage(trace: ExecutionTrace, subject_properties: SubjectProperties) -> float:
+    """Computes branch coverage on bytecode instructions.
+
+    The resulting coverage should be equal to decision coverage on source code.
 
     Args:
         trace: The execution trace
@@ -851,11 +825,9 @@ def compute_branch_coverage(
         The computed coverage value
     """
     covered = len(
-        trace.executed_code_objects.intersection(
-            subject_properties.branch_less_code_objects
-        )
+        trace.executed_code_objects.intersection(subject_properties.branch_less_code_objects)
     )
-    existing = len(subject_properties.branch_less_code_objects)
+    existing = sum(1 for _ in subject_properties.branch_less_code_objects)
 
     # Every predicate creates two branches
     existing += len(subject_properties.existing_predicates) * 2
@@ -865,18 +837,12 @@ def compute_branch_coverage(
     covered += len([v for v in trace.true_distances.values() if v == 0.0])
     covered += len([v for v in trace.false_distances.values() if v == 0.0])
 
-    if existing == 0:
-        # Nothing to cover => everything is covered.
-        coverage = 1.0
-    else:
-        coverage = covered / existing
+    coverage = 1.0 if existing == 0 else covered / existing
     assert 0.0 <= coverage <= 1.0, "Coverage must be in [0,1]"
     return coverage
 
 
-def compute_line_coverage(
-    trace: ExecutionTrace, subject_properties: SubjectProperties
-) -> float:
+def compute_line_coverage(trace: ExecutionTrace, subject_properties: SubjectProperties) -> float:
     """Computes line coverage on bytecode instructions.
 
     Args:
@@ -899,26 +865,19 @@ def compute_line_coverage(
 
 
 def _cleanse_included_implicit_return_none(
-    subject_properties, statement_checked_lines, statement_slice
+    subject_properties: SubjectProperties,
+    statement_checked_lines: set[int],
+    statement_slice: list[UniqueInstruction],
 ):
-    if (
-        # check if the last included instructions before the store
-        # are a "return None"
-        len(statement_slice) >= 3
-        and statement_slice[-3].opcode == op.LOAD_CONST
-        and statement_slice[-3].arg is None
-        and statement_slice[-2].opcode == op.RETURN_VALUE
-    ):
-        if (
-            # check if the "return None" is implicit or explicit
-            len(statement_slice) != 3
-            and statement_slice[-4].lineno != statement_slice[-3].lineno
-        ):
-            statement_checked_lines.remove(
-                DynamicSlicer.get_line_id_by_instruction(
-                    statement_slice[-3], subject_properties
-                )
+    # check if the last included instructions before the store
+    # are a explicit "return None"
+    if version.end_with_explicit_return_none(statement_slice[:-1]):
+        statement_checked_lines.remove(
+            DynamicSlicer.get_line_id_by_instruction(
+                statement_slice[-version.RETURN_NONE_SIZE - 1],
+                subject_properties,
             )
+        )
 
 
 def compute_statement_checked_lines(
@@ -928,6 +887,7 @@ def compute_statement_checked_lines(
     statement_slicing_criteria: dict[int, SlicingCriterion],
 ) -> set[int]:
     """Computes checked coverage on bytecode instructions.
+
     Each statement can be sliced, returning a list of instructions
     that are checked by the return value of the statement.
     If we combine all lists of instructions returned by slicing all statements,
@@ -964,7 +924,9 @@ def compute_statement_checked_lines(
         )
 
         _cleanse_included_implicit_return_none(
-            subject_properties, statement_checked_lines, statement_slice
+            subject_properties,
+            statement_checked_lines,
+            statement_slice,
         )
 
         checked_lines_ids.update(statement_checked_lines)
@@ -975,6 +937,7 @@ def compute_assertion_checked_coverage(
     trace: ExecutionTrace, subject_properties: SubjectProperties
 ) -> float:
     """Computes checked coverage on bytecode instructions.
+
     Each assertion can be sliced, returning a list of instructions
     that are checked by an assertion.
     If we combine all lists of instructions returned by slicing all assertions,
@@ -1004,9 +967,7 @@ def compute_assertion_checked_coverage(
             assertion_checked_instructions = assertion_slicer.slice_assertion(
                 executed_assertion, trace
             )
-            executed_assertion.assertion.checked_instructions.extend(
-                assertion_checked_instructions
-            )
+            executed_assertion.assertion.checked_instructions.extend(assertion_checked_instructions)
             # checked at any point by the assertion of a statement
             checked_instructions.extend(assertion_checked_instructions)
 
