@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides a chromosome for a single test case."""
+
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
@@ -33,7 +34,8 @@ class TestCaseChromosome(chrom.Chromosome):
         test_factory: tf.TestFactory | None = None,
         orig: TestCaseChromosome | None = None,
     ) -> None:
-        """
+        """Initializes a new test-case chromosome.
+
         Must supply either a TestCaseChromosome to copy from or the remaining arguments.
 
         Args:
@@ -43,20 +45,18 @@ class TestCaseChromosome(chrom.Chromosome):
         """
         super().__init__(orig=orig)
         if orig is None:
-            assert (
-                test_case is not None
-            ), "Cannot create test case chromosome without test case"
+            assert test_case is not None, "Cannot create test case chromosome without test case"
             self._test_case: tc.TestCase = test_case
             self._test_factory: tf.TestFactory | None = test_factory
             self.changed = True
             self._last_execution_result: ExecutionResult | None = None
             self._num_mutations = 0
         else:
-            self._test_case = orig._test_case.clone()
-            self._test_factory = orig._test_factory
+            self._test_case = orig._test_case.clone()  # noqa: SLF001
+            self._test_factory = orig._test_factory  # noqa: SLF001
             self.changed = orig.changed
-            self._last_execution_result = orig._last_execution_result
-            self._num_mutations = orig._num_mutations
+            self._last_execution_result = orig._last_execution_result  # noqa: SLF001
+            self._num_mutations = orig._num_mutations  # noqa: SLF001
 
     @property
     def test_case(self) -> tc.TestCase:
@@ -76,18 +76,18 @@ class TestCaseChromosome(chrom.Chromosome):
         # TODO(fk) what to do with this when crossover is used?
         return self._num_mutations
 
-    def size(self) -> int:
+    def size(self) -> int:  # noqa: D102
         return self._test_case.size()
 
-    def length(self) -> int:
+    def length(self) -> int:  # noqa: D102
         return self.size()
 
-    def cross_over(
+    def cross_over(  # noqa: D102
         self, other: chrom.Chromosome, position1: int, position2: int
     ) -> None:
-        assert isinstance(
-            other, TestCaseChromosome
-        ), "Cannot perform crossover with " + str(type(other))
+        assert isinstance(other, TestCaseChromosome), "Cannot perform crossover with " + str(
+            type(other)
+        )
         assert self._test_factory is not None, "Crossover requires a test factory."
 
         offspring_test_case = self.test_case.clone(position1)
@@ -97,14 +97,11 @@ class TestCaseChromosome(chrom.Chromosome):
                 offspring_test_case, other.test_case.get_statement(j)
             )
 
-        if (
-            offspring_test_case.size()
-            < config.configuration.search_algorithm.chromosome_length
-        ):
+        if offspring_test_case.size() < config.configuration.search_algorithm.chromosome_length:
             self._test_case = offspring_test_case
             self.changed = True
 
-    def mutate(self) -> None:
+    def mutate(self) -> None:  # noqa: D102
         changed = False
 
         if (
@@ -120,25 +117,22 @@ class TestCaseChromosome(chrom.Chromosome):
         backup = self.test_case.clone()
 
         if (
-            randomness.next_float()
-            <= config.configuration.search_algorithm.test_delete_probability
+            randomness.next_float() <= config.configuration.search_algorithm.test_delete_probability
+            and self._mutation_delete()
         ):
-            if self._mutation_delete():
-                changed = True
+            changed = True
 
         if (
-            randomness.next_float()
-            <= config.configuration.search_algorithm.test_change_probability
+            randomness.next_float() <= config.configuration.search_algorithm.test_change_probability
+            and self._mutation_change()
         ):
-            if self._mutation_change():
-                changed = True
+            changed = True
 
         if (
-            randomness.next_float()
-            <= config.configuration.search_algorithm.test_insert_probability
+            randomness.next_float() <= config.configuration.search_algorithm.test_insert_probability
+            and self._mutation_insert()
         ):
-            if self._mutation_insert():
-                changed = True
+            changed = True
 
         assert self._test_factory, "Required for mutation"
         if not self._test_factory.has_call_on_sut(self._test_case):
@@ -165,8 +159,7 @@ class TestCaseChromosome(chrom.Chromosome):
 
     def _delete_statement(self, idx: int) -> bool:
         assert self._test_factory, "Mutation requires a test factory."
-        modified = self._test_factory.delete_statement_gracefully(self._test_case, idx)
-        return modified
+        return self._test_factory.delete_statement_gracefully(self._test_case, idx)
 
     def _mutation_change(self) -> bool:
         last_mutatable_statement = self.get_last_mutatable_statement()
@@ -199,11 +192,13 @@ class TestCaseChromosome(chrom.Chromosome):
         return changed
 
     def _mutation_insert(self) -> bool:
-        """With exponentially decreasing probability, insert statements at a
-        random position.
+        """Insertion mutation operation.
+
+        With exponentially decreasing probability, insert statements at a random
+        position.
 
         Returns:
-            Whether or not the test case was changed
+            Whether the test case was changed
         """
         changed = False
         alpha = config.configuration.search_algorithm.statement_insertion_probability
@@ -221,9 +216,7 @@ class TestCaseChromosome(chrom.Chromosome):
                 # Also include the position after the last mutatable statement.
                 max_position += 1
 
-            position = self._test_factory.insert_random_statement(
-                self._test_case, max_position
-            )
+            position = self._test_factory.insert_random_statement(self._test_case, max_position)
             exponent += 1
             if 0 <= position < self.size():
                 changed = True
@@ -285,10 +278,10 @@ class TestCaseChromosome(chrom.Chromosome):
             return False
         return self._last_execution_result.has_test_exceptions()
 
-    def accept(self, visitor: cv.ChromosomeVisitor) -> None:
+    def accept(self, visitor: cv.ChromosomeVisitor) -> None:  # noqa: D102
         visitor.visit_test_case_chromosome(self)
 
-    def clone(self) -> TestCaseChromosome:
+    def clone(self) -> TestCaseChromosome:  # noqa: D102
         return TestCaseChromosome(orig=self)
 
     def __eq__(self, other):
@@ -299,7 +292,7 @@ class TestCaseChromosome(chrom.Chromosome):
         # This condition is playing with fire, but it is required to not lose coverage
         # information on flaky tests. For more information on this see #169.
         # Be careful when comparing TestCaseChromosomes!
-        if (left := self._last_execution_result) is not None and (
+        if (left := self._last_execution_result) is not None and (  # noqa: SIM102
             right := other._last_execution_result
         ) is not None:
             if left.execution_trace != right.execution_trace:
