@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides a factory to create test case chromosomes."""
+
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
@@ -14,20 +15,20 @@ import pynguin.ga.chromosomefactory as cf
 import pynguin.ga.testcasechromosome as tcc
 
 from pynguin.utils import randomness
-from pynguin.utils.orderedset import OrderedSet
 
 
 if TYPE_CHECKING:
+    import pynguin.ga.algorithms.archive as arch
     import pynguin.ga.computations as ff
     import pynguin.ga.testcasefactory as tcf
-    import pynguin.generation.algorithms.archive as arch
     import pynguin.testcase.testfactory as tf
 
+    from pynguin.utils.orderedset import OrderedSet
 
-class TestCaseChromosomeFactory(
-    cf.ChromosomeFactory[tcc.TestCaseChromosome]
-):  # pylint:disable=too-few-public-methods.
+
+class TestCaseChromosomeFactory(cf.ChromosomeFactory[tcc.TestCaseChromosome]):
     """A factory that creates test case chromosomes using the given test case factory.
+
     Also add the given fitness functions to the newly created test case chromosome.
     """
 
@@ -50,24 +51,21 @@ class TestCaseChromosomeFactory(
         self._test_case_factory = test_case_factory
         self._fitness_functions = fitness_functions
 
-    def get_chromosome(self) -> tcc.TestCaseChromosome:
+    def get_chromosome(self) -> tcc.TestCaseChromosome:  # noqa: D102
         test_case = self._test_case_factory.get_test_case()
-        chrom = tcc.TestCaseChromosome(
-            test_case=test_case, test_factory=self._test_factory
-        )
+        chrom = tcc.TestCaseChromosome(test_case=test_case, test_factory=self._test_factory)
         for func in self._fitness_functions:
             chrom.add_fitness_function(func)
         return chrom
 
 
-class ArchiveReuseTestCaseChromosomeFactory(
-    cf.ChromosomeFactory[tcc.TestCaseChromosome]
-):  # pylint:disable=too-few-public-methods.
-    """Provides test case chromosomes from an archive with some probability,
-    otherwise delegates to wrapped chromosome factory.
+class ArchiveReuseTestCaseChromosomeFactory(cf.ChromosomeFactory[tcc.TestCaseChromosome]):
+    """Provides test case chromosomes from an archive with some probability.
+
+    Otherwise, delegates to wrapped chromosome factory.
     """
 
-    def __init__(
+    def __init__(  # noqa: D107
         self,
         delegate: cf.ChromosomeFactory[tcc.TestCaseChromosome],
         archive: arch.Archive,
@@ -75,7 +73,7 @@ class ArchiveReuseTestCaseChromosomeFactory(
         self._delegate = delegate
         self._archive = archive
 
-    def get_chromosome(self) -> tcc.TestCaseChromosome:
+    def get_chromosome(self) -> tcc.TestCaseChromosome:  # noqa: D102
         pick_from = self._archive.solutions
         if (
             len(pick_from) > 0
