@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides a factories for generating different kind of test cases."""
+
 from __future__ import annotations
 
 from abc import abstractmethod
@@ -13,8 +14,6 @@ from typing import TYPE_CHECKING
 import pynguin.configuration as config
 import pynguin.testcase.defaulttestcase as dtc
 
-from pynguin.analyses.module import TestCluster
-from pynguin.analyses.seeding import InitialPopulationProvider
 from pynguin.utils import randomness
 
 
@@ -22,8 +21,10 @@ if TYPE_CHECKING:
     import pynguin.testcase.testcase as tc
     import pynguin.testcase.testfactory as tf
 
+    from pynguin.analyses.module import TestCluster
+    from pynguin.analyses.seeding import InitialPopulationProvider
+
 
-# pylint:disable=too-few-public-methods
 class TestCaseFactory:
     """Abstract class for test case factories."""
 
@@ -33,7 +34,7 @@ class TestCaseFactory:
         Args:
             test_factory: The used test factory
         """
-        self._test_factory = test_factory
+        self.test_factory = test_factory
 
     @abstractmethod
     def get_test_case(self) -> tc.TestCase:
@@ -47,22 +48,21 @@ class TestCaseFactory:
 class RandomLengthTestCaseFactory(TestCaseFactory):
     """Create random test cases with random length."""
 
-    def __init__(self, test_factory: tf.TestFactory, test_cluster: TestCluster):
+    def __init__(  # noqa: D107
+        self, test_factory: tf.TestFactory, test_cluster: TestCluster
+    ):
         super().__init__(test_factory)
         self._test_cluster = test_cluster
 
-    def get_test_case(self) -> tc.TestCase:
+    def get_test_case(self) -> tc.TestCase:  # noqa: D102
         test_case = dtc.DefaultTestCase(self._test_cluster)
         attempts = 0
-        size = randomness.next_int(
-            1, config.configuration.search_algorithm.chromosome_length + 1
-        )
+        size = randomness.next_int(1, config.configuration.search_algorithm.chromosome_length + 1)
 
         while (
-            test_case.size() < size
-            and attempts < config.configuration.test_creation.max_attempts
+            test_case.size() < size and attempts < config.configuration.test_creation.max_attempts
         ):
-            self._test_factory.insert_random_statement(test_case, test_case.size())
+            self.test_factory.insert_random_statement(test_case, test_case.size())
             attempts += 1
         return test_case
 
@@ -76,14 +76,14 @@ class SeededTestCaseFactory(TestCaseFactory):
     generation is delegated to the RandomLengthTestCaseFactory.
     """
 
-    def __init__(
+    def __init__(  # noqa: D107
         self, delegate: TestCaseFactory, population_provider: InitialPopulationProvider
     ):
-        super().__init__(delegate._test_factory)
+        super().__init__(delegate.test_factory)
         self._delegate = delegate
         self._population_provider = population_provider
 
-    def get_test_case(self) -> tc.TestCase:
+    def get_test_case(self) -> tc.TestCase:  # noqa: D102
         if (
             randomness.next_float()
             <= config.configuration.seeding.seeded_testcases_reuse_probability
