@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides a factory to create test suite chromosomes."""
+
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
@@ -14,16 +15,15 @@ import pynguin.ga.chromosomefactory as cf
 import pynguin.ga.testsuitechromosome as tsc
 
 from pynguin.utils import randomness
-from pynguin.utils.orderedset import OrderedSet
 
 
 if TYPE_CHECKING:
     import pynguin.ga.computations as ff
 
+    from pynguin.utils.orderedset import OrderedSet
 
-class TestSuiteChromosomeFactory(
-    cf.ChromosomeFactory[tsc.TestSuiteChromosome]
-):  # pylint:disable=too-few-public-methods.
+
+class TestSuiteChromosomeFactory(cf.ChromosomeFactory[tsc.TestSuiteChromosome]):
     """A factory that provides new test suite chromosomes of random length."""
 
     def __init__(
@@ -48,7 +48,7 @@ class TestSuiteChromosomeFactory(
         self._fitness_functions = fitness_functions
         self._coverage_functions = coverage_functions
 
-    def get_chromosome(self) -> tsc.TestSuiteChromosome:
+    def get_chromosome(self) -> tsc.TestSuiteChromosome:  # noqa: D102
         chromosome = tsc.TestSuiteChromosome(self._test_case_chromosome_factory)
         num_tests = randomness.next_int(
             config.configuration.search_algorithm.min_initial_tests,
@@ -56,9 +56,7 @@ class TestSuiteChromosomeFactory(
         )
 
         for _ in range(num_tests):
-            chromosome.add_test_case_chromosome(
-                self._test_case_chromosome_factory.get_chromosome()
-            )
+            chromosome.add_test_case_chromosome(self._test_case_chromosome_factory.get_chromosome())
         for fitness_function in self._fitness_functions:
             chromosome.add_fitness_function(fitness_function)
         for coverage_function in self._coverage_functions:
