@@ -1,15 +1,17 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
-"""
-Provides classes for runtime instrumentation.
+"""Provides classes for runtime instrumentation.
+
 Inspired by https://github.com/agronholm/typeguard/blob/master/typeguard/importhook.py.
 """
+
 from __future__ import annotations
 
+import contextlib
 import logging
 import sys
 
@@ -18,7 +20,6 @@ from importlib.abc import MetaPathFinder
 from importlib.machinery import ModuleSpec
 from importlib.machinery import SourceFileLoader
 from inspect import isclass
-from types import CodeType
 from typing import TYPE_CHECKING
 from typing import cast
 
@@ -27,40 +28,41 @@ import pynguin.configuration as config
 from pynguin.analyses.constants import ConstantPool
 from pynguin.analyses.constants import DynamicConstantProvider
 from pynguin.analyses.constants import EmptyConstantProvider
-from pynguin.instrumentation.instrumentation import BranchCoverageInstrumentation
-from pynguin.instrumentation.instrumentation import CheckedCoverageInstrumentation
-from pynguin.instrumentation.instrumentation import DynamicSeedingInstrumentation
-from pynguin.instrumentation.instrumentation import InstrumentationTransformer
-from pynguin.instrumentation.instrumentation import LineCoverageInstrumentation
+from pynguin.instrumentation.transformer import InstrumentationTransformer
+from pynguin.instrumentation.version import BranchCoverageInstrumentation
+from pynguin.instrumentation.version import CheckedCoverageInstrumentation
+from pynguin.instrumentation.version import DynamicSeedingInstrumentation
+from pynguin.instrumentation.version import LineCoverageInstrumentation
 
 
 if TYPE_CHECKING:
-    from pynguin.instrumentation.instrumentation import InstrumentationAdapter
-    from pynguin.testcase.execution import ExecutionTracer
+    from types import CodeType
+
+    from pynguin.instrumentation.tracer import SubjectProperties
+    from pynguin.instrumentation.transformer import InstrumentationAdapter
 
 
 class InstrumentationLoader(SourceFileLoader):
     """A loader that instruments the module after execution."""
 
-    def __init__(
+    def __init__(  # noqa: D107
         self,
         fullname,
         path,
-        tracer: ExecutionTracer,
         transformer: InstrumentationTransformer,
     ):
         super().__init__(fullname, path)
-        self._tracer = tracer
         self._transformer = transformer
 
-    def exec_module(self, module):
-        self._tracer.reset()
+    def exec_module(self, module):  # noqa: D102
+        self._transformer.subject_properties.reset()
         super().exec_module(module)
-        self._tracer.store_import_trace()
+        self._transformer.subject_properties.instrumentation_tracer.store_import_trace()
 
     def get_code(self, fullname) -> CodeType:
-        """Add instrumentation instructions to the code of the module
-        before it is executed.
+        """Add instrumentation instructions to the code of the module.
+
+        This happens before the module is executed.
 
         Args:
             fullname: The name of the module
@@ -68,21 +70,20 @@ class InstrumentationLoader(SourceFileLoader):
         Returns:
             The modules code blocks
         """
-        to_instrument = cast(CodeType, super().get_code(fullname))
+        to_instrument = cast("CodeType", super().get_code(fullname))
         assert to_instrument is not None, "Failed to get code object of module."
         return self._transformer.instrument_module(to_instrument)
 
 
 def build_transformer(
-    tracer: ExecutionTracer,
+    subject_properties: SubjectProperties,
     coverage_metrics: set[config.CoverageMetric],
     dynamic_constant_provider: DynamicConstantProvider | None = None,
 ) -> InstrumentationTransformer:
-    """Create an instrumentation transformer that applies the configured
-    instrumentations.
+    """Build a transformer that applies the configured instrumentation.
 
     Args:
-        tracer: The tracer to use.
+        subject_properties: The properties of the subject under test.
         coverage_metrics: The coverage metrics to use.
         dynamic_constant_provider: The dynamic constant provider to use.
             When such a provider is passed, we apply the instrumentation for dynamic
@@ -93,33 +94,33 @@ def build_transformer(
     """
     adapters: list[InstrumentationAdapter] = []
     if config.CoverageMetric.BRANCH in coverage_metrics:
-        adapters.append(BranchCoverageInstrumentation(tracer))
+        adapters.append(BranchCoverageInstrumentation(subject_properties))
     if config.CoverageMetric.LINE in coverage_metrics:
-        adapters.append(LineCoverageInstrumentation(tracer))
+        adapters.append(LineCoverageInstrumentation(subject_properties))
     if config.CoverageMetric.CHECKED in coverage_metrics:
-        adapters.append(CheckedCoverageInstrumentation(tracer))
+        adapters.append(CheckedCoverageInstrumentation(subject_properties))
 
     if dynamic_constant_provider is not None:
         adapters.append(DynamicSeedingInstrumentation(dynamic_constant_provider))
 
-    return InstrumentationTransformer(tracer, adapters)
+    return InstrumentationTransformer(subject_properties, adapters)
 
 
 class InstrumentationFinder(MetaPathFinder):
-    """
-    A meta pathfinder which wraps another pathfinder.
+    """A meta pathfinder which wraps another pathfinder.
+
     It receives all import requests and intercepts the ones for the modules that
     should be instrumented.
     """
 
     _logger = logging.getLogger(__name__)
 
-    # pylint:disable=too-many-arguments
     def __init__(
         self,
+        *,
         original_pathfinder,
         module_to_instrument: str,
-        tracer: ExecutionTracer,
+        subject_properties: SubjectProperties,
         coverage_metrics: set[config.CoverageMetric],
         dynamic_constant_provider: DynamicConstantProvider | None = None,
     ) -> None:
@@ -128,31 +129,41 @@ class InstrumentationFinder(MetaPathFinder):
         Args:
             original_pathfinder: the original pathfinder that is wrapped.
             module_to_instrument: the name of the module, that should be instrumented.
-            tracer: the execution tracer
+            subject_properties: the properties of the subject under test.
             coverage_metrics: the coverage metrics to be used for instrumentation.
             dynamic_constant_provider: Used for dynamic constant seeding
         """
         self._module_to_instrument = module_to_instrument
         self._original_pathfinder = original_pathfinder
-        self._tracer = tracer
+        self._subject_properties = subject_properties
         self._coverage_metrics = coverage_metrics
         self._dynamic_constant_provider = dynamic_constant_provider
 
+    @property
+    def subject_properties(self) -> SubjectProperties:
+        """Get the properties of the subject under test.
+
+        Returns:
+            The subject properties
+        """
+        return self._subject_properties
+
     def update_instrumentation_metrics(
         self,
-        tracer: ExecutionTracer,
+        subject_properties: SubjectProperties,
         coverage_metrics: set[config.CoverageMetric],
         dynamic_constant_provider: DynamicConstantProvider | None,
     ) -> None:
-        """Update the coverage instrumentation. Useful for re-applying a different
-        instrumentation.
+        """Update the coverage instrumentation.
+
+        Useful for re-applying a different instrumentation.
 
         Args:
-            tracer: The new execution tracer
+            subject_properties: The new subject properties
             coverage_metrics: The new coverage metrics
             dynamic_constant_provider: The dynamic constant provider, if any.
         """
-        self._tracer = tracer
+        self._subject_properties = subject_properties
         self._coverage_metrics = coverage_metrics
         self._dynamic_constant_provider = dynamic_constant_provider
 
@@ -174,25 +185,21 @@ class InstrumentationFinder(MetaPathFinder):
             An optional ModuleSpec
         """
         if self._should_instrument(fullname):
-            spec: ModuleSpec = self._original_pathfinder.find_spec(
-                fullname, path, target
-            )
+            spec: ModuleSpec = self._original_pathfinder.find_spec(fullname, path, target)
             if spec is not None:
                 if isinstance(spec.loader, FileLoader):
                     spec.loader = InstrumentationLoader(
                         spec.loader.name,
                         spec.loader.path,
-                        self._tracer,
                         build_transformer(
-                            self._tracer,
+                            self._subject_properties,
                             self._coverage_metrics,
                             self._dynamic_constant_provider,
                         ),
                     )
                     return spec
                 self._logger.error(
-                    "Loader for module under test is not a FileLoader,"
-                    " can not instrument."
+                    "Loader for module under test is not a FileLoader, can not instrument."
                 )
 
         return None
@@ -201,7 +208,7 @@ class InstrumentationFinder(MetaPathFinder):
 class ImportHookContextManager:
     """A simple context manager for using the import hook."""
 
-    def __init__(self, hook: MetaPathFinder):
+    def __init__(self, hook: MetaPathFinder):  # noqa: D107
         self.hook = hook
 
     def __enter__(self):
@@ -212,15 +219,13 @@ class ImportHookContextManager:
 
     def uninstall(self):
         """Remove the installed hook."""
-        try:
+        with contextlib.suppress(ValueError):  # suppress error if already removed
             sys.meta_path.remove(self.hook)
-        except ValueError:
-            pass  # already removed
 
 
 def install_import_hook(
     module_to_instrument: str,
-    tracer: ExecutionTracer,
+    subject_properties: SubjectProperties,
     coverage_metrics: set[config.CoverageMetric] | None = None,
     dynamic_constant_provider: DynamicConstantProvider | None = None,
 ) -> ImportHookContextManager:
@@ -228,7 +233,7 @@ def install_import_hook(
 
     Args:
         module_to_instrument: The module that shall be instrumented.
-        tracer: The tracer where the instrumentation should report its data.
+        subject_properties: The properties of the subject under test.
         coverage_metrics: the coverage metrics to be used for instrumentation, falls
             back to the configured metrics in the configuration, if not specified.
         dynamic_constant_provider: Used for dynamic constant seeding
@@ -253,11 +258,7 @@ def install_import_hook(
 
     to_wrap = None
     for finder in sys.meta_path:
-        if (
-            isclass(finder)
-            and finder.__name__ == "PathFinder"
-            and hasattr(finder, "find_spec")
-        ):
+        if isclass(finder) and finder.__name__ == "PathFinder" and hasattr(finder, "find_spec"):
             to_wrap = finder
             break
 
@@ -265,9 +266,9 @@ def install_import_hook(
         raise RuntimeError("Cannot find a PathFinder in sys.meta_path")
 
     hook = InstrumentationFinder(
-        to_wrap,
-        module_to_instrument,
-        tracer,
+        original_pathfinder=to_wrap,
+        module_to_instrument=module_to_instrument,
+        subject_properties=subject_properties,
         coverage_metrics=coverage_metrics,
         dynamic_constant_provider=dynamic_constant_provider,
     )
