@@ -1,17 +1,17 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides a default implementation of a test case."""
+
 from __future__ import annotations
 
 import logging
 
 from itertools import islice
 from typing import TYPE_CHECKING
-from typing import Any
 
 import pynguin.testcase.testcase as tc
 
@@ -25,16 +25,15 @@ if TYPE_CHECKING:
     import pynguin.testcase.variablereference as vr
 
 
-# pylint:disable=too-many-public-methods
-class DefaultTestCase(tc.TestCase):
+class DefaultTestCase(tc.TestCase):  # noqa: PLR0904
     """A default implementation of a test case."""
 
     _logger = logging.getLogger(__name__)
 
-    def accept(self, visitor: tcv.TestCaseVisitor) -> None:
+    def accept(self, visitor: tcv.TestCaseVisitor) -> None:  # noqa: D102
         visitor.visit_default_test_case(self)
 
-    def add_statement(
+    def add_statement(  # noqa: D102
         self, statement: stmt.Statement, position: int = -1
     ) -> vr.VariableReference | None:
         if position == -1:
@@ -43,7 +42,7 @@ class DefaultTestCase(tc.TestCase):
             self._statements.insert(position, statement)
         return statement.ret_val
 
-    def add_variable_creating_statement(
+    def add_variable_creating_statement(  # noqa: D102
         self, statement: stmt.VariableCreatingStatement, position: int = -1
     ) -> vr.VariableReference:
         if position == -1:
@@ -52,10 +51,10 @@ class DefaultTestCase(tc.TestCase):
             self._statements.insert(position, statement)
         return statement.ret_val
 
-    def add_statements(self, statements: list[stmt.Statement]) -> None:
+    def add_statements(self, statements: list[stmt.Statement]) -> None:  # noqa: D102
         self._statements.extend(statements)
 
-    def append_test_case(self, test_case: tc.TestCase) -> None:
+    def append_test_case(self, test_case: tc.TestCase) -> None:  # noqa: D102
         memo: dict[vr.VariableReference, vr.VariableReference] = {}
         for statement in test_case.statements:
             clone = statement.clone(self, memo)
@@ -65,38 +64,58 @@ class DefaultTestCase(tc.TestCase):
                 memo[statement.ret_val] = clone.ret_val  # type: ignore[assignment]
             self._statements.append(clone)
 
-    def remove(self, position: int) -> None:
+    def remove(self, position: int) -> None:  # noqa: D102
         self._logger.debug("Removing statement at position %d", position)
         if position >= self.size():
             return
         del self._statements[position]
 
-    def remove_statement(self, statement: stmt.Statement) -> None:
+    def remove_statement(self, statement: stmt.Statement) -> None:  # noqa: D102
         self._statements.remove(statement)
 
-    def chop(self, pos: int) -> None:
+    def remove_with_forward_dependencies(self, position: int) -> list[int]:  # noqa: D102
+        if position >= self.size():
+            raise ValueError(
+                f"Position {position} is out of bounds for test case of size {self.size()}."
+            )
+        statement = self.get_statement(position)
+        return self.remove_statement_with_forward_dependencies(statement)
+
+    def remove_statement_with_forward_dependencies(self, statement: stmt.Statement) -> list[int]:  # noqa: D102
+        if not self.contains(statement):
+            raise ValueError(f"Statement {statement} not found in test case.")
+        ret_val = statement.ret_val
+        forward_dependencies = []
+        if ret_val is not None:
+            forward_dependencies = list(self.get_forward_dependencies(ret_val))
+        positions_to_remove = tc.TestCase.positions_to_remove(statement, forward_dependencies)
+        for pos in positions_to_remove:
+            self.remove(pos)
+        return positions_to_remove
+
+    def chop(self, pos: int) -> None:  # noqa: D102
         assert pos >= 0
         while len(self._statements) > pos + 1:
             del self._statements[-1]
 
-    def contains(self, statement: stmt.Statement) -> bool:
+    def contains(self, statement: stmt.Statement) -> bool:  # noqa: D102
         return statement in self._statements
 
-    def get_statement(self, position: int) -> stmt.Statement:
+    def get_statement(self, position: int) -> stmt.Statement:  # noqa: D102
         assert 0 <= position < len(self._statements)
         return self._statements[position]
 
-    def set_statement(
+    def set_statement(  # noqa: D102
         self, statement: stmt.Statement, position: int
     ) -> vr.VariableReference | None:
         assert 0 <= position < len(self._statements)
         self._statements[position] = statement
         return statement.ret_val
 
-    def has_statement(self, position: int) -> bool:
+    def has_statement(self, position: int) -> bool:  # noqa: D102
         return 0 <= position < len(self._statements)
 
-    def clone(self, limit: int | None = None) -> tc.TestCase:
+    def clone(self, limit: int | None = None) -> tc.TestCase:  # noqa: D102
         test_case = DefaultTestCase(self.test_cluster)
         memo: dict[vr.VariableReference, vr.VariableReference] = {}
         for statement in islice(self._statements, limit):
@@ -109,7 +128,7 @@ class DefaultTestCase(tc.TestCase):
             copy.assertions = statement.copy_assertions(memo)
         return test_case
 
-    def get_dependencies(
+    def get_dependencies(  # noqa: D102
         self, var: vr.VariableReference
     ) -> OrderedSet[vr.VariableReference]:
         dependencies: OrderedSet[vr.VariableReference] = OrderedSet()
@@ -129,20 +148,42 @@ class DefaultTestCase(tc.TestCase):
 
         return dependencies
 
-    def get_assertions(self) -> list[ass.Assertion]:
+    def get_forward_dependencies(  # noqa: D102
+        self, var: vr.VariableReference
+    ) -> OrderedSet[vr.VariableReference]:
+        dependencies: OrderedSet[vr.VariableReference] = OrderedSet()
+
+        # Start with the variable itself
+        dependencies.add(var)
+
+        # Iterate forward from the variable's statement position
+        for idx in range(var.get_statement_position() + 1, self.size()):
+            statement = self.get_statement(idx)
+            # Check if this statement references any of the dependencies we've found so far
+            if (
+                any(statement.references(dep) for dep in dependencies)
+                and statement.ret_val is not None
+            ):
+                dependencies.add(statement.ret_val)
+
+        # Remove the original variable from the dependencies
+        dependencies.remove(var)
+
+        return dependencies
+
+    def get_assertions(self) -> list[ass.Assertion]:  # noqa: D102
         assertions: list[ass.Assertion] = []
         for statement in self._statements:
             assertions.extend(statement.assertions)
         return assertions
 
-    def size_with_assertions(self) -> int:
+    def size_with_assertions(self) -> int:  # noqa: D102
         return self.size() + len(self.get_assertions())
 
-    def size(self) -> int:
+    def size(self) -> int:  # noqa: D102
         return len(self._statements)
 
-    # pylint: disable=too-many-return-statements
-    def __eq__(self, other: Any) -> bool:
+    def __eq__(self, other: object) -> bool:
         if self is other:
             return True
         if not isinstance(other, DefaultTestCase):
