@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
-"""Provides a visitor that transforms statements to ast"""
+"""Provides a visitor that transforms statements to AST."""
+
 from __future__ import annotations
 
 import ast
@@ -16,16 +17,15 @@ from typing import cast
 
 import pynguin.utils.ast_util as au
 
+from pynguin.testcase.statement import ASTAssignStatement
 from pynguin.testcase.statement import ClassPrimitiveStatement
 from pynguin.testcase.statement import StatementVisitor
-from pynguin.utils.generic.genericaccessibleobject import (
-    GenericCallableAccessibleObject,
-)
 
 
 if TYPE_CHECKING:
     import pynguin.utils.namingscope as ns
 
+    from pynguin.testcase.statement import AllowedValuesStatement
     from pynguin.testcase.statement import AssignmentStatement
     from pynguin.testcase.statement import BooleanPrimitiveStatement
     from pynguin.testcase.statement import BytesPrimitiveStatement
@@ -39,25 +39,31 @@ if TYPE_CHECKING:
     from pynguin.testcase.statement import IntPrimitiveStatement
     from pynguin.testcase.statement import ListStatement
     from pynguin.testcase.statement import MethodStatement
+    from pynguin.testcase.statement import NdArrayStatement
     from pynguin.testcase.statement import NoneStatement
     from pynguin.testcase.statement import ParametrizedStatement
     from pynguin.testcase.statement import PrimitiveStatement
     from pynguin.testcase.statement import SetStatement
     from pynguin.testcase.statement import StringPrimitiveStatement
     from pynguin.testcase.statement import TupleStatement
+    from pynguin.utils.generic.genericaccessibleobject import (
+        GenericCallableAccessibleObject,
+    )
 
 
-class StatementToAstVisitor(StatementVisitor):
+class StatementToAstVisitor(StatementVisitor):  # noqa: PLR0904
     """Visitor that transforms statements into a list of AST nodes."""
 
     def __init__(
         self,
         module_aliases: ns.AbstractNamingScope,
         variable_names: ns.AbstractNamingScope,
+        *,
         store_call_return: bool = True,
     ) -> None:
-        """Creates a new transformation visitor that transforms our internal
-        statements to Python AST nodes.
+        """Creates a new transformation visitor.
+
+        The visitor transforms our internal statements to Python AST nodes.
 
         Args:
             module_aliases: A naming scope for module alias names.
@@ -82,35 +88,43 @@ class StatementToAstVisitor(StatementVisitor):
         assert self._ast_node, "No statement visited"
         return self._ast_node
 
-    def visit_int_primitive_statement(self, stmt: IntPrimitiveStatement) -> None:
+    def visit_int_primitive_statement(  # noqa: D102
+        self, stmt: IntPrimitiveStatement
+    ) -> None:
         self._ast_node = self._create_constant(stmt)
 
-    def visit_float_primitive_statement(self, stmt: FloatPrimitiveStatement) -> None:
+    def visit_float_primitive_statement(  # noqa: D102
+        self, stmt: FloatPrimitiveStatement
+    ) -> None:
         self._ast_node = self._create_constant(stmt)
 
-    def visit_complex_primitive_statement(
+    def visit_complex_primitive_statement(  # noqa: D102
         self, stmt: ComplexPrimitiveStatement
     ) -> None:
         self._ast_node = self._create_constant(stmt)
 
-    def visit_string_primitive_statement(self, stmt: StringPrimitiveStatement) -> None:
+    def visit_string_primitive_statement(  # noqa: D102
+        self, stmt: StringPrimitiveStatement
+    ) -> None:
         self._ast_node = self._create_constant(stmt)
 
-    def visit_bytes_primitive_statement(self, stmt: BytesPrimitiveStatement) -> None:
+    def visit_bytes_primitive_statement(  # noqa: D102
+        self, stmt: BytesPrimitiveStatement
+    ) -> None:
         self._ast_node = self._create_constant(stmt)
 
-    def visit_boolean_primitive_statement(
+    def visit_boolean_primitive_statement(  # noqa: D102
         self, stmt: BooleanPrimitiveStatement
     ) -> None:
         self._ast_node = self._create_constant(stmt)
 
-    def visit_enum_statement(self, stmt: EnumPrimitiveStatement) -> None:
+    def visit_enum_statement(self, stmt: EnumPrimitiveStatement) -> None:  # noqa: D102
         owner = stmt.accessible_object().owner
         assert owner
         self._ast_node = ast.Assign(
             targets=[
                 au.create_full_name(
-                    self._variable_names, self._module_aliases, stmt.ret_val, False
+                    self._variable_names, self._module_aliases, stmt.ret_val, load=False
                 )
             ],
             value=ast.Attribute(
@@ -124,12 +138,14 @@ class StatementToAstVisitor(StatementVisitor):
             ),
         )
 
-    def visit_class_primitive_statement(self, stmt: ClassPrimitiveStatement) -> None:
+    def visit_class_primitive_statement(  # noqa: D102
+        self, stmt: ClassPrimitiveStatement
+    ) -> None:
         clazz = stmt.type_info
         self._ast_node = ast.Assign(
             targets=[
                 au.create_full_name(
-                    self._variable_names, self._module_aliases, stmt.ret_val, False
+                    self._variable_names, self._module_aliases, stmt.ret_val, load=False
                 )
             ],
             # TODO(fk) think about nested classes, also for enums.
@@ -140,10 +156,12 @@ class StatementToAstVisitor(StatementVisitor):
             ),
         )
 
-    def visit_none_statement(self, stmt: NoneStatement) -> None:
+    def visit_none_statement(self, stmt: NoneStatement) -> None:  # noqa: D102
         self._ast_node = self._create_constant(stmt)
 
-    def visit_constructor_statement(self, stmt: ConstructorStatement) -> None:
+    def visit_constructor_statement(  # noqa: D102
+        self, stmt: ConstructorStatement
+    ) -> None:
         owner = stmt.accessible_object().owner
         assert owner
         args, kwargs = self._create_args(stmt)
@@ -160,7 +178,10 @@ class StatementToAstVisitor(StatementVisitor):
             self._ast_node = ast.Assign(
                 targets=[
                     au.create_full_name(
-                        self._variable_names, self._module_aliases, stmt.ret_val, False
+                        self._variable_names,
+                        self._module_aliases,
+                        stmt.ret_val,
+                        load=False,
                     )
                 ],
                 value=call,
@@ -168,14 +189,14 @@ class StatementToAstVisitor(StatementVisitor):
         else:
             self._ast_node = ast.Expr(value=call)
 
-    def visit_method_statement(self, stmt: MethodStatement) -> None:
+    def visit_method_statement(self, stmt: MethodStatement) -> None:  # noqa: D102
         args, kwargs = self._create_args(stmt)
         call = ast.Call(
             func=ast.Attribute(
                 attr=stmt.accessible_object().callable.__name__,
                 ctx=ast.Load(),
                 value=au.create_full_name(
-                    self._variable_names, self._module_aliases, stmt.callee, True
+                    self._variable_names, self._module_aliases, stmt.callee, load=True
                 ),
             ),
             args=args,
@@ -187,21 +208,22 @@ class StatementToAstVisitor(StatementVisitor):
             self._ast_node = ast.Assign(
                 targets=[
                     au.create_full_name(
-                        self._variable_names, self._module_aliases, stmt.ret_val, False
+                        self._variable_names,
+                        self._module_aliases,
+                        stmt.ret_val,
+                        load=False,
                     )
                 ],
                 value=call,
             )
 
-    def visit_function_statement(self, stmt: FunctionStatement) -> None:
+    def visit_function_statement(self, stmt: FunctionStatement) -> None:  # noqa: D102
         args, kwargs = self._create_args(stmt)
         call = ast.Call(
             func=ast.Attribute(
                 attr=stmt.accessible_object().callable.__name__,
                 ctx=ast.Load(),
-                value=self._create_module_alias(
-                    stmt.accessible_object().callable.__module__
-                ),
+                value=self._create_module_alias(stmt.accessible_object().callable.__module__),
             ),
             args=args,
             keywords=kwargs,
@@ -212,13 +234,16 @@ class StatementToAstVisitor(StatementVisitor):
             self._ast_node = ast.Assign(
                 targets=[
                     au.create_full_name(
-                        self._variable_names, self._module_aliases, stmt.ret_val, False
+                        self._variable_names,
+                        self._module_aliases,
+                        stmt.ret_val,
+                        load=False,
                     )
                 ],
                 value=call,
             )
 
-    def visit_field_statement(self, stmt: FieldStatement) -> None:
+    def visit_field_statement(self, stmt: FieldStatement) -> None:  # noqa: D102
         self._ast_node = ast.Assign(
             targets=[
                 ast.Name(
@@ -230,54 +255,79 @@ class StatementToAstVisitor(StatementVisitor):
                 attr=stmt.field.field,
                 ctx=ast.Load(),
                 value=au.create_full_name(
-                    self._variable_names, self._module_aliases, stmt.source, True
+                    self._variable_names, self._module_aliases, stmt.source, load=True
                 ),
             ),
         )
 
-    def visit_assignment_statement(self, stmt: AssignmentStatement) -> None:
+    def visit_assignment_statement(  # noqa: D102
+        self, stmt: AssignmentStatement
+    ) -> None:
         self._ast_node = ast.Assign(
             targets=[
                 au.create_full_name(
-                    self._variable_names, self._module_aliases, stmt.lhs, False
+                    self._variable_names, self._module_aliases, stmt.lhs, load=False
                 )
             ],
             value=au.create_full_name(
-                self._variable_names, self._module_aliases, stmt.rhs, True
+                self._variable_names, self._module_aliases, stmt.rhs, load=True
             ),
         )
 
-    def visit_list_statement(self, stmt: ListStatement) -> None:
+    def visit_ast_assign_statement(  # noqa: D102
+        self, stmt: ASTAssignStatement
+    ) -> None:
+        self._ast_node = ast.Assign(  # type: ignore[call-overload,unused-ignore]
+            targets=[
+                au.create_full_name(
+                    self._variable_names, self._module_aliases, stmt.ret_val, load=False
+                )
+            ],
+            value=stmt.get_rhs_as_normal_ast(  # type: ignore[arg-type,unused-ignore]
+                lambda x: au.create_full_name(
+                    self._variable_names, self._module_aliases, x, load=True
+                )
+            ),
+        )
+
+    def visit_list_statement(self, stmt: ListStatement) -> None:  # noqa: D102
         self._ast_node = ast.Assign(
             targets=[
                 au.create_full_name(
-                    self._variable_names, self._module_aliases, stmt.ret_val, False
+                    self._variable_names, self._module_aliases, stmt.ret_val, load=False
                 )
             ],
             value=ast.List(
                 elts=[
-                    au.create_full_name(
-                        self._variable_names, self._module_aliases, x, True
-                    )
+                    au.create_full_name(self._variable_names, self._module_aliases, x, load=True)
                     for x in stmt.elements
                 ],
                 ctx=ast.Load(),
             ),
         )
 
-    def visit_set_statement(self, stmt: SetStatement) -> None:
+    def visit_ndarray_statement(self, stmt: NdArrayStatement) -> None:  # noqa: D102
+        self._ast_node = ast.Assign(
+            targets=[
+                au.create_full_name(
+                    self._variable_names, self._module_aliases, stmt.ret_val, load=False
+                )
+            ],
+            value=au.create_ast_for_nested_list(stmt.elements),
+        )
+
+    def visit_allowed_values_statement(self, stmt: AllowedValuesStatement) -> None:  # noqa: D102
+        self._ast_node = self._create_constant(stmt)
+
+    def visit_set_statement(self, stmt: SetStatement) -> None:  # noqa: D102
         # There is no literal for empty sets, so we have to write "set()"
         inner: Any
         if len(stmt.elements) == 0:
-            inner = ast.Call(
-                func=ast.Name(id="set", ctx=ast.Load()), args=[], keywords=[]
-            )
+            inner = ast.Call(func=ast.Name(id="set", ctx=ast.Load()), args=[], keywords=[])
         else:
-            inner = ast.Set(
+            inner = ast.Set(  # type: ignore[call-arg]
                 elts=[
-                    au.create_full_name(
-                        self._variable_names, self._module_aliases, x, True
-                    )
+                    au.create_full_name(self._variable_names, self._module_aliases, x, load=True)
                     for x in stmt.elements
                 ],
                 ctx=ast.Load(),
@@ -286,48 +336,42 @@ class StatementToAstVisitor(StatementVisitor):
         self._ast_node = ast.Assign(
             targets=[
                 au.create_full_name(
-                    self._variable_names, self._module_aliases, stmt.ret_val, False
+                    self._variable_names, self._module_aliases, stmt.ret_val, load=False
                 )
             ],
             value=inner,
         )
 
-    def visit_tuple_statement(self, stmt: TupleStatement) -> None:
+    def visit_tuple_statement(self, stmt: TupleStatement) -> None:  # noqa: D102
         self._ast_node = ast.Assign(
             targets=[
                 au.create_full_name(
-                    self._variable_names, self._module_aliases, stmt.ret_val, False
+                    self._variable_names, self._module_aliases, stmt.ret_val, load=False
                 )
             ],
             value=ast.Tuple(
                 elts=[
-                    au.create_full_name(
-                        self._variable_names, self._module_aliases, x, True
-                    )
+                    au.create_full_name(self._variable_names, self._module_aliases, x, load=True)
                     for x in stmt.elements
                 ],
                 ctx=ast.Load(),
             ),
         )
 
-    def visit_dict_statement(self, stmt: DictStatement) -> None:
+    def visit_dict_statement(self, stmt: DictStatement) -> None:  # noqa: D102
         self._ast_node = ast.Assign(
             targets=[
                 au.create_full_name(
-                    self._variable_names, self._module_aliases, stmt.ret_val, False
+                    self._variable_names, self._module_aliases, stmt.ret_val, load=False
                 )
             ],
             value=ast.Dict(
                 keys=[
-                    au.create_full_name(
-                        self._variable_names, self._module_aliases, x[0], True
-                    )
+                    au.create_full_name(self._variable_names, self._module_aliases, x[0], load=True)
                     for x in stmt.elements
                 ],
                 values=[
-                    au.create_full_name(
-                        self._variable_names, self._module_aliases, x[1], True
-                    )
+                    au.create_full_name(self._variable_names, self._module_aliases, x[1], load=True)
                     for x in stmt.elements
                 ],
             ),
@@ -345,16 +389,16 @@ class StatementToAstVisitor(StatementVisitor):
         return ast.Assign(
             targets=[
                 au.create_full_name(
-                    self._variable_names, self._module_aliases, stmt.ret_val, False
+                    self._variable_names, self._module_aliases, stmt.ret_val, load=False
                 )
             ],
             value=ast.Constant(value=stmt.value),
         )
 
-    def _create_args(
-        self, stmt: ParametrizedStatement
-    ) -> tuple[list[ast.expr], list[ast.keyword]]:
-        """Creates the positional arguments, i.e., POSITIONAL_ONLY,
+    def _create_args(self, stmt: ParametrizedStatement) -> tuple[list[ast.expr], list[ast.keyword]]:
+        """Creates the AST nodes for arguments.
+
+        Creates the positional arguments, i.e., POSITIONAL_ONLY,
         POSITIONAL_OR_KEYWORD and VAR_POSITIONAL as well as the keyword arguments,
         i.e., KEYWORD_ONLY or VAR_KEYWORD.
 
@@ -368,7 +412,7 @@ class StatementToAstVisitor(StatementVisitor):
         kwargs = []
 
         gen_callable: GenericCallableAccessibleObject = cast(
-            GenericCallableAccessibleObject, stmt.accessible_object()
+            "GenericCallableAccessibleObject", stmt.accessible_object()
         )
 
         left_of_current: list[str] = []
@@ -382,7 +426,7 @@ class StatementToAstVisitor(StatementVisitor):
                     self._variable_names,
                     self._module_aliases,
                     stmt.args[param_name],
-                    True,
+                    load=True,
                 )
                 match param.kind:
                     case Parameter.POSITIONAL_ONLY:
