@@ -1,19 +1,19 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides an implementation for a test case."""
+
 from __future__ import annotations
 
-from abc import ABCMeta
+from abc import ABC
 from abc import abstractmethod
 from typing import TYPE_CHECKING
 
 from pynguin.utils import randomness
 from pynguin.utils.exceptions import ConstructionFailedException
-from pynguin.utils.orderedset import OrderedSet
 
 
 if TYPE_CHECKING:
@@ -24,10 +24,10 @@ if TYPE_CHECKING:
 
     from pynguin.analyses.module import TestCluster
     from pynguin.analyses.typesystem import ProperType
+    from pynguin.utils.orderedset import OrderedSet
 
 
-# pylint: disable=too-many-public-methods
-class TestCase(metaclass=ABCMeta):
+class TestCase(ABC):  # noqa: PLR0904
     """An abstract base implementation for a test case.
 
     Serves as an interface for test-case implementations
@@ -85,8 +85,7 @@ class TestCase(metaclass=ABCMeta):
     def add_variable_creating_statement(
         self, statement: stmt.VariableCreatingStatement, position: int = -1
     ) -> vr.VariableReference:
-        """Overloaded version of add_statement that adds a statement
-        which creates a variable.
+        """Overloaded version of add_statement that adds a statement.
 
         Args:
             statement: The new statement
@@ -130,6 +129,34 @@ class TestCase(metaclass=ABCMeta):
             statement: The statement to remove.
         """
 
+    @abstractmethod
+    def remove_with_forward_dependencies(self, position: int) -> list[int]:
+        """Removes a statement at the given position along with all its forward dependencies.
+
+        Args:
+            position: The position of the statement to remove
+
+        Returns:
+            A list of positions of statements that have been deleted
+
+        Raises:
+            ValueError: If the position is out of bounds for this test case
+        """
+
+    @abstractmethod
+    def remove_statement_with_forward_dependencies(self, statement: stmt.Statement) -> list[int]:
+        """Removes the given statement along with all its forward dependencies.
+
+        Args:
+            statement: The statement to remove
+
+        Returns:
+            A list of positions of statements that have been deleted
+
+        Raises:
+            ValueError: If the statement is not contained in the test case
+        """
+
     @abstractmethod
     def chop(self, pos: int) -> None:
         """Remove all statements after a given position.
@@ -217,9 +244,7 @@ class TestCase(metaclass=ABCMeta):
         """Get all assertions that exist for this test case."""
 
     @abstractmethod
-    def get_dependencies(
-        self, var: vr.VariableReference
-    ) -> OrderedSet[vr.VariableReference]:
+    def get_dependencies(self, var: vr.VariableReference) -> OrderedSet[vr.VariableReference]:
         """Provides all variables on which var depends.
 
         Args:
@@ -229,11 +254,21 @@ class TestCase(metaclass=ABCMeta):
             a set of variables on which var depends on. # noqa: DAR202
         """
 
-    def get_objects(
-        self, parameter_type: ProperType, position: int
-    ) -> list[vr.VariableReference]:
-        """Provides a list of variable references satisfying a certain type before a
-        given position.
+    @abstractmethod
+    def get_forward_dependencies(
+        self, var: vr.VariableReference
+    ) -> OrderedSet[vr.VariableReference]:
+        """Provides all variables that depend on var.
+
+        Args:
+            var: the variable for which we look for all dependent statements of
+
+        Returns:
+            a set of variables that depend on var. # noqa: DAR202
+        """
+
+    def get_objects(self, parameter_type: ProperType, position: int) -> list[vr.VariableReference]:
+        """Provides a list of variable references satisfying a certain type.
 
         If the position value is larger than the number of statements, only these
         statements will be considered.  Otherwise, the first `position` statements of
@@ -280,9 +315,7 @@ class TestCase(metaclass=ABCMeta):
                 variables.append(var)
         return variables
 
-    def get_random_object(
-        self, parameter_type: ProperType, position: int
-    ) -> vr.VariableReference:
+    def get_random_object(self, parameter_type: ProperType, position: int) -> vr.VariableReference:
         """Get a random object of the given type up to the given position (exclusive).
 
         Args:
@@ -301,3 +334,22 @@ class TestCase(metaclass=ABCMeta):
                 f"Found no variables of type {parameter_type} at position {position}"
             )
         return randomness.choice(variables)
+
+    @staticmethod
+    def positions_to_remove(
+        statement: stmt.Statement, dependencies: list[vr.VariableReference]
+    ) -> list[int]:
+        """Get the positions to remove and its forward dependencies in reverse order.
+
+        This is done to avoid index issues when removing multiple statements.
+
+        Args:
+            statement: The statement to remove
+            dependencies: The forward dependencies of the statement
+
+        Returns:
+            A list of positions to remove
+        """
+        positions = {dep.get_statement_position() for dep in dependencies}
+        positions.add(statement.get_position())
+        return sorted(positions, reverse=True)
