@@ -1,13 +1,13 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides a visitor that transforms test cases to asts."""
+
 from __future__ import annotations
 
-from ast import stmt
 from typing import TYPE_CHECKING
 
 import pynguin.assertion.assertion as ass
@@ -20,6 +20,8 @@ from pynguin.testcase.testcasevisitor import TestCaseVisitor
 
 
 if TYPE_CHECKING:
+    from ast import stmt
+
     import pynguin.testcase.defaulttestcase as dtc
     import pynguin.testcase.execution as ex
 
@@ -36,6 +38,8 @@ class TestCaseToAstVisitor(TestCaseVisitor):
         module_aliases: ns.NamingScope,
         common_modules: set[str],
         exec_result: ex.ExecutionResult | None = None,
+        *,
+        store_call_return: bool = False,
     ) -> None:
         """The module aliases are shared between test cases.
 
@@ -43,6 +47,7 @@ class TestCaseToAstVisitor(TestCaseVisitor):
             module_aliases: The aliases for used modules
             common_modules: The names of common modules that are not aliased
             exec_result: An optional execution result for the test case.
+            store_call_return: Whether to store the call return.
         """
         self._module_aliases: ns.NamingScope = module_aliases
         # Common modules (e.g. math) are not aliased.
@@ -50,13 +55,14 @@ class TestCaseToAstVisitor(TestCaseVisitor):
         self._exec_result = exec_result
         self._test_case_ast: list[stmt] = []
         self._is_failing_test: bool = False
+        self._store_call_return: bool = store_call_return
 
-    def visit_default_test_case(self, test_case: dtc.DefaultTestCase) -> None:
+    def visit_default_test_case(  # noqa: D102
+        self, test_case: dtc.DefaultTestCase
+    ) -> None:
         self._test_case_ast = []
         return_type_trace = (
-            None
-            if self._exec_result is None
-            else self._exec_result.proper_return_type_trace
+            None if self._exec_result is None else self._exec_result.proper_return_type_trace
         )
         variables = ns.VariableTypeNamingScope(return_type_trace=return_type_trace)
         for idx, statement in enumerate(test_case.statements):
@@ -65,9 +71,18 @@ class TestCaseToAstVisitor(TestCaseVisitor):
                 self._exec_result is not None
                 and self._exec_result.get_first_position_of_thrown_exception() == idx
             ):
-                # If a statement causes an exception and defines a new name, we don't
-                # actually want to create that name, as it will not be stored anyway.
+                store_call_return = self._store_call_return
+
+            # Only store the return value if it's used by subsequent statements or has assertions
+            # If store_call_return is True, we always store the return value
+            elif (
+                not self._store_call_return
+                and statement.ret_val is not None
+                and not test_case.get_forward_dependencies(statement.ret_val)
+                and not statement.assertions  # No assertions
+            ):
                 store_call_return = False
+
             statement_visitor = stmt_to_ast.StatementToAstVisitor(
                 self._module_aliases, variables, store_call_return=store_call_return
             )
