@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Utility methods for AST manipulation."""
+
 from __future__ import annotations
 
 import ast
@@ -20,6 +21,7 @@ def create_full_name(
     variable_names: ns.AbstractNamingScope,
     module_names: ns.AbstractNamingScope,
     var: vr.Reference,
+    *,
     load: bool,
 ) -> ast.Name | ast.Attribute:
     """Create a name node for the corresponding variable.
@@ -37,14 +39,14 @@ def create_full_name(
     loads = [True for _ in names[:-1]]
     loads.append(load)
     # First has to be ast.Name
-    res: ast.Name | ast.Attribute = create_ast_name(names[0], not loads[0])
+    res: ast.Name | ast.Attribute = create_ast_name(names[0], store=not loads[0])
     # Remaining are ast.Attribute
-    for name, loa in zip(names[1:], loads[1:]):
-        res = create_ast_attribute(name, res, not loa)
+    for name, loa in zip(names[1:], loads[1:], strict=True):
+        res = create_ast_attribute(name, res, store=not loa)
     return res
 
 
-def create_ast_name(name_id: str, store: bool = False) -> ast.Name:
+def create_ast_name(name_id: str, *, store: bool = False) -> ast.Name:
     """Creates an AST name node.
 
     Args:
@@ -70,7 +72,7 @@ def create_ast_assign(target, value) -> ast.Assign:
     return ast.Assign(targets=[target], value=value)
 
 
-def create_ast_attribute(attr, value, store: bool = False) -> ast.Attribute:
+def create_ast_attribute(attr, value, *, store: bool = False) -> ast.Attribute:
     """Creates an AST attribute node.
 
     Args:
@@ -81,12 +83,10 @@ def create_ast_attribute(attr, value, store: bool = False) -> ast.Attribute:
     Returns:
         the attribute node
     """
-    return ast.Attribute(
-        attr=attr, ctx=ast.Store() if store else ast.Load(), value=value
-    )
+    return ast.Attribute(attr=attr, ctx=ast.Store() if store else ast.Load(), value=value)
 
 
-def create_ast_list(elts, store: bool = False) -> ast.List:
+def create_ast_list(elts, *, store: bool = False) -> ast.List:
     """Creates an AST list node.
 
     Args:
@@ -124,7 +124,7 @@ def create_ast_set(elts) -> ast.Set:
     return ast.Set(elts=elts)
 
 
-def create_ast_tuple(elts, store: bool = False) -> ast.Tuple:
+def create_ast_tuple(elts, *, store: bool = False) -> ast.Tuple:
     """Creates an AST tuple node.
 
     Args:
@@ -200,3 +200,20 @@ def create_ast_keyword(arg, value) -> ast.keyword:
         the keyword node
     """
     return ast.keyword(arg=arg, value=value)
+
+
+def create_ast_for_nested_list(nested: list) -> ast.List | ast.Constant:
+    """Recursively convert a nested list into a corresponding AST node.
+
+    Args:
+        nested: nested list
+
+    Returns:
+        the nested list node
+    """
+    if isinstance(nested, list):
+        return ast.List(
+            elts=[create_ast_for_nested_list(item) for item in nested],
+            ctx=ast.Load(),
+        )
+    return ast.Constant(value=nested)
