@@ -1,18 +1,18 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Utility methods for collections."""
+
 from __future__ import annotations
 
 from typing import Any
 
 
 def dict_without_keys(dict_to_change: dict[Any, Any], keys: set[Any]) -> dict[Any, Any]:
-    """
-    Removes the given keys from the given dict.
+    """Removes the given keys from the given dict.
 
     Args:
         dict_to_change: The dict where the keys should be removed.
