@@ -1,25 +1,27 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides a naming scope."""
+
 from __future__ import annotations
 
 import typing
 
 from abc import abstractmethod
 from collections import defaultdict
-from collections.abc import Callable
 from typing import Any
 
-import pynguin.testcase.variablereference as vr
-
 from pynguin.analyses.typesystem import TypeVisitor
 
 
 if typing.TYPE_CHECKING:
+    from collections.abc import Callable
+
+    import pynguin.testcase.variablereference as vr
+
     from pynguin.analyses.typesystem import AnyType
     from pynguin.analyses.typesystem import Instance
     from pynguin.analyses.typesystem import NoneType
@@ -81,7 +83,7 @@ class NamingScope(AbstractNamingScope):
         self._prefix = prefix
         self._new_name_callback = new_name_callback
 
-    def get_name(self, obj: Any) -> str:
+    def get_name(self, obj: Any) -> str:  # noqa: D102
         if obj in self._known_names:
             return self._known_names[obj]
 
@@ -98,7 +100,7 @@ class NamingScope(AbstractNamingScope):
         for obj in self._known_names:
             yield obj, self.get_name(obj)
 
-    def is_known_name(self, obj) -> bool:
+    def is_known_name(self, obj) -> bool:  # noqa: D102
         return obj in self._known_names
 
 
@@ -111,12 +113,18 @@ class VariableTypeNamingScope(AbstractNamingScope):
         return_type_trace: dict[int, ProperType] | None = None,
         prefix: str = "var",
     ):
+        """Constructs the naming scope.
+
+        Args:
+            return_type_trace: A dictionary of return-type traces
+            prefix: The prefix for the variable names
+        """
         self._known_variable_names: dict[vr.VariableReference, str] = {}
         self._type_counter: dict[str, int] = defaultdict(int)
         self._prefix = prefix
         self._return_type_trace = return_type_trace
 
-    def get_name(self, obj: vr.VariableReference) -> str:
+    def get_name(self, obj: vr.VariableReference) -> str:  # noqa: D102
         if (name := self._known_variable_names.get(obj)) is not None:
             return name
 
@@ -126,11 +134,7 @@ class VariableTypeNamingScope(AbstractNamingScope):
         # Lookup runtime type if available from trace:
         if (
             self._return_type_trace is not None
-            and (
-                runtime_type := self._return_type_trace.get(
-                    obj.get_statement_position()
-                )
-            )
+            and (runtime_type := self._return_type_trace.get(obj.get_statement_position()))
             is not None
         ):
             type_ = runtime_type
@@ -148,7 +152,7 @@ class VariableTypeNamingScope(AbstractNamingScope):
     def __iter__(self):
         yield from self._known_variable_names.items()
 
-    def is_known_name(self, obj) -> bool:
+    def is_known_name(self, obj) -> bool:  # noqa: D102
         return obj in self._known_variable_names
 
 
