@@ -1,21 +1,26 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides a singleton instance of Random that can be seeded."""
+
 from __future__ import annotations
 
 import random
 import string
 
-from collections.abc import Sequence
+from typing import TYPE_CHECKING
 from typing import Any
 from typing import TypeVar
 
 
-class Random(random.Random):
+if TYPE_CHECKING:
+    from collections.abc import Sequence
+
+
+class Random(random.Random):  # noqa: S311
     """Override Random to allow querying for the seed value.
 
     It generates a seed if none was given from `time.time_ns()`.  This is NOT
@@ -24,15 +29,14 @@ class Random(random.Random):
     use the current time stamp in nano seconds as seed.
     """
 
-    def __init__(self, x=None) -> None:
+    def __init__(self, x=None) -> None:  # noqa: D107
         super().__init__(x)
         self._current_seed: int | None = None
         self.seed(x)
 
-    # pylint: disable=import-outside-toplevel
-    def seed(self, a=None, version: int = 2) -> None:
+    def seed(self, a=None, version: int = 2) -> None:  # noqa: D102
         if a is None:
-            import time
+            import time  # noqa: PLC0415
 
             a = time.time_ns()
 
@@ -107,8 +111,9 @@ def next_float(lower_bound=0, upper_bound=1) -> float:
 
 
 def next_gaussian() -> float:
-    """Returns the next pseudorandom, Gaussian ("normally") distributed
-    value with mu 0.0 and sigma 1.0.
+    """Returns the next pseudorandom.
+
+    Use a Gaussian ("normally") distribution value with mu 0.0 and sigma 1.0.
 
     Returns:
         The next random number
@@ -116,7 +121,7 @@ def next_gaussian() -> float:
     return RNG.gauss(0, 1)
 
 
-_T = TypeVar("_T")  # pylint:disable=invalid-name
+_T = TypeVar("_T")
 
 
 def choice(sequence: Sequence[_T]) -> _T:
