@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides capabilities to create a coverage report."""
+
 from __future__ import annotations
 
 import dataclasses
@@ -13,9 +14,7 @@ import importlib.resources
 import inspect
 import sys
 import typing
-import xml.etree.ElementTree as ET
-
-from pathlib import Path
+import xml.etree.ElementTree as ET  # noqa: S405
 
 import pygments
 
@@ -31,6 +30,8 @@ from pynguin.utils.orderedset import OrderedSet
 
 
 if typing.TYPE_CHECKING:
+    from pathlib import Path
+
     import pynguin.ga.testsuitechromosome as tsc
 
     from pynguin.testcase.execution import TestCaseExecutor
@@ -52,9 +53,7 @@ class CoverageEntry:
         Returns:
             A new coverage entry with the summed up elements of self and other.
         """
-        return CoverageEntry(
-            self.covered + other.covered, self.existing + other.existing
-        )
+        return CoverageEntry(self.covered + other.covered, self.existing + other.existing)
 
 
 @dataclasses.dataclass
@@ -72,17 +71,14 @@ class LineAnnotation:
     lines: CoverageEntry
 
     def message(self):
-        """Compute the message that should be displayed as a tool tip
-        when hovering over this line number.
+        """Compute the message that should be displayed as a tool tip.
 
         Returns:
             The message for this line.
         """
         msgs = []
         if self.branches.existing > 0:
-            msgs.append(
-                f"{self.branches.covered}/{self.branches.existing} branches covered"
-            )
+            msgs.append(f"{self.branches.covered}/{self.branches.existing} branches covered")
         if self.branchless_code_objects.existing > 0:
             msgs.append(
                 f"{self.branchless_code_objects.covered}/"
@@ -91,10 +87,7 @@ class LineAnnotation:
             )
         if self.lines.existing > 0:
             # No need to say something like "1 out of 1 lines executed".
-            msgs.append(
-                f"Line {self.line_no}"
-                f"{'' if self.lines.covered == 1 else ' not'} covered"
-            )
+            msgs.append(f"Line {self.line_no}{'' if self.lines.covered == 1 else ' not'} covered")
         return "; ".join(msgs)
 
     def __add__(self, other: LineAnnotation) -> LineAnnotation:
@@ -117,7 +110,7 @@ class LineAnnotation:
 
 
 @dataclasses.dataclass
-class CoverageReport:  # pylint:disable=too-many-instance-attributes
+class CoverageReport:
     """All coverage related data required to create a coverage report."""
 
     module: str
@@ -144,7 +137,6 @@ class CoverageReport:  # pylint:disable=too-many-instance-attributes
     line_coverage: float | None = None
 
 
-# pylint:disable=too-many-locals
 def get_coverage_report(
     suite: tsc.TestSuiteChromosome,
     executor: TestCaseExecutor,
@@ -166,12 +158,16 @@ def get_coverage_report(
         assert result is not None
         results.append(result)
     trace = ff.analyze_results(results)
-    subject_properties = executor.tracer.get_subject_properties()
-    source = inspect.getsourcelines(sys.modules[config.configuration.module_name])[0]
+    subject_properties = executor.subject_properties
+    try:
+        source = inspect.getsourcelines(sys.modules[config.configuration.module_name])[0]
+    except Exception as e:
+        # circular references in module might cause errors
+        raise RuntimeError(
+            f"Could not get source code for module {config.configuration.module_name}. "
+        ) from e
     line_annotations = [
-        LineAnnotation(
-            idx + 1, CoverageEntry(), CoverageEntry(), CoverageEntry(), CoverageEntry()
-        )
+        LineAnnotation(idx + 1, CoverageEntry(), CoverageEntry(), CoverageEntry(), CoverageEntry())
         for idx in range(len(source))
     ]
 
@@ -179,12 +175,10 @@ def get_coverage_report(
     branchless_code_objects = CoverageEntry()
     branches = CoverageEntry()
     if config.CoverageMetric.BRANCH in metrics:
-        line_to_branchless_code_object_coverage = (
-            _get_line_to_branchless_code_object_coverage(subject_properties, trace)
-        )
-        line_to_branch_coverage = _get_line_to_branch_coverage(
+        line_to_branchless_code_object_coverage = _get_line_to_branchless_code_object_coverage(
             subject_properties, trace
         )
+        line_to_branch_coverage = _get_line_to_branch_coverage(subject_properties, trace)
 
         branch_coverage = ff.compute_branch_coverage(trace, subject_properties)
         for cov in line_to_branchless_code_object_coverage.values():
@@ -204,8 +198,8 @@ def get_coverage_report(
     lines = CoverageEntry()
     if config.CoverageMetric.LINE in metrics:
         line_coverage = ff.compute_line_coverage(trace, subject_properties)
-        covered_lines = executor.tracer.lineids_to_linenos(trace.covered_line_ids)
-        existing_lines = executor.tracer.lineids_to_linenos(
+        covered_lines = subject_properties.lineids_to_linenos(trace.covered_line_ids)
+        existing_lines = subject_properties.lineids_to_linenos(
             OrderedSet(subject_properties.existing_lines.keys())
         )
         lines += CoverageEntry(len(covered_lines), len(existing_lines))
@@ -265,7 +259,7 @@ def render_coverage_report(
         )
 
 
-def render_xml_coverage_report(
+def render_xml_coverage_report(  # noqa: PLR0914
     cov_report: CoverageReport, report_path: Path, timestamp: datetime.datetime
 ) -> None:
     """Render the given coverage report to the given file using Cobertura XML style.
@@ -279,12 +273,8 @@ def render_xml_coverage_report(
     branch_rate = f"{cov_report.branch_coverage}"
     lines_covered = f"{cov_report.lines.covered}"
     lines_valid = f"{cov_report.lines.existing}"
-    branches_covered = (
-        f"{cov_report.branches.covered + cov_report.branchless_code_objects.covered}"
-    )
-    branches_valid = (
-        f"{cov_report.branches.existing + cov_report.branchless_code_objects.existing}"
-    )
+    branches_covered = f"{cov_report.branches.covered + cov_report.branchless_code_objects.covered}"
+    branches_valid = f"{cov_report.branches.existing + cov_report.branchless_code_objects.existing}"
     complexity = "0.0"
     version = f"pynguin-{ver.__version__}"
 
@@ -347,12 +337,10 @@ def render_xml_coverage_report(
             or line_annotation.branchless_code_objects.existing > 0
         ):
             covered = (
-                line_annotation.branches.covered
-                + line_annotation.branchless_code_objects.covered
+                line_annotation.branches.covered + line_annotation.branchless_code_objects.covered
             )
             existing = (
-                line_annotation.branches.existing
-                + line_annotation.branchless_code_objects.existing
+                line_annotation.branches.existing + line_annotation.branchless_code_objects.existing
             )
             cov = covered / existing
             cov_string = f"{cov:.0%} ({covered}/{existing})"
@@ -366,8 +354,7 @@ def render_xml_coverage_report(
     with report_path.open(mode="w", encoding="utf-8") as xml_file:
         xml_file.write('<?xml version="1.0" encoding="UTF-8"?>')
         xml_file.write(
-            "<!DOCTYPE coverage SYSTEM "
-            '"http://cobertura.sourceforge.net/xml/coverage-04.dtd">'
+            '<!DOCTYPE coverage SYSTEM "http://cobertura.sourceforge.net/xml/coverage-04.dtd">'
         )
         tree.write(xml_file, encoding="unicode")
 
@@ -389,9 +376,7 @@ def _get_line_to_branch_coverage(subject_properties, trace):
 def _get_line_to_branchless_code_object_coverage(subject_properties, trace):
     line_to_branchless_code_object_coverage = {}
     for code in subject_properties.branch_less_code_objects:
-        lineno = subject_properties.existing_code_objects[
-            code
-        ].code_object.co_firstlineno
+        lineno = subject_properties.existing_code_objects[code].code_object.co_firstlineno
         if lineno not in line_to_branchless_code_object_coverage:
             line_to_branchless_code_object_coverage[lineno] = CoverageEntry()
         line_to_branchless_code_object_coverage[lineno] += CoverageEntry(existing=1)
@@ -424,6 +409,4 @@ def _get_line_annotations_for_branch_coverage(
     if lineno in predicate_coverage:
         branches = predicate_coverage[lineno]
         total += branches
-    return LineAnnotation(
-        lineno, total, branches, branchless_code_objects, CoverageEntry()
-    )
+    return LineAnnotation(lineno, total, branches, branchless_code_objects, CoverageEntry())
