@@ -1,14 +1,14 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides runtime variables for output."""
+
 import enum
 
 
-# pylint: disable=invalid-name
 @enum.unique
 class RuntimeVariable(str, enum.Enum):
     """Defines all runtime variables we want to store in the result CSV files.
@@ -45,9 +45,6 @@ class RuntimeVariable(str, enum.Enum):
     # Number of iterations of the test-generation algorithm
     AlgorithmIterations = "AlgorithmIterations"
 
-    # Execution results
-    ExecutionResults = "ExecutionResults"
-
     # The random seed used during the search.
     # A random one was used if none was specified in the beginning
     RandomSeed = "RandomSeed"
@@ -184,5 +181,54 @@ class RuntimeVariable(str, enum.Enum):
     # they do not increase the resulting checked coverage
     DeletedAssertions = "DeletedAssertions"
 
+    # Which LLM strategy is applied. This is mainly
+    # for final result grouping
+    LLMStrategy = "LLMStrategy"
+
+    # Number of total LLM calls
+    TotalLLMCalls = "TotalLLMCalls"
+
+    # Number of input tokens sent LLM to model
+    TotalLLMInputTokens = "TotalLLMInputTokens"
+
+    # Number of input tokens sent LLM to model
+    TotalLLMOutputTokens = "TotalLLMOutputTokens"
+
+    # Number of LLM responses with no python code within them
+    TotalCodelessLLMResponses = "TotalCodelessLLMResponses"
+
+    # Number of seconds LLM queries took
+    LLMQueryTime = "LLMQueryTime"
+
+    # Total of LLM test cases tha are merged into the population
+    TotalLTCs = "TotalLTCs"
+
+    # Number of parsed statements from LLM output
+    LLMTotalParsedStatements = "LLMTotalParsedStatements"
+
+    # Total number of statements from LLM output
+    LLMTotalStatements = "LLMTotalStatements"
+
+    # Number of uninterpreted statements
+    LLMUninterpretedStatements = "LLMUninterpretedStatements"
+
+    # The coverage before LLM call for uncovered targets (initial coverage)
+    CoverageBeforeLLMCall = "CoverageBeforeLLMCall"
+
+    # The coverage after LLM call for uncovered targets
+    CoverageAfterLLMCall = "CoverageAfterLLMCall"
+
+    # Total assertions added to test cases that were received from the LLM
+    TotalAssertionsAddedFromLLM = "TotalAssertionsAddedFromLLM"
+
+    # Total assertions added to test cases
+    TotalAssertionsReceivedFromLLM = "TotalAssertionsReceivedFromLLM"
+
+    # Discovered non-whitelisted C-extension modules in the SUT
+    CExtensionModules = "CExtensionModules"
+
+    # Whether the subprocess mode was used or not
+    SubprocessMode = "SubprocessMode"
+
     def __repr__(self):
         return f"{self.name}"
