@@ -1,17 +1,18 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides an interface for a statistics writer."""
+
 from __future__ import annotations
 
 import csv
 import ctypes
 import logging
 
-from abc import ABCMeta
+from abc import ABC
 from abc import abstractmethod
 from dataclasses import dataclass
 from pathlib import Path
@@ -21,7 +22,7 @@ from typing import TypeVar
 import pynguin.configuration as config
 
 
-T = TypeVar("T")  # pylint: disable=invalid-name
+T = TypeVar("T")
 
 
 @dataclass(frozen=True)
@@ -32,8 +33,7 @@ class OutputVariable(Generic[T]):
     value: T
 
 
-# pylint: disable=too-few-public-methods
-class AbstractStatisticsBackend(metaclass=ABCMeta):
+class AbstractStatisticsBackend(ABC):
     """An interface for a statistics writer."""
 
     @abstractmethod
@@ -45,20 +45,17 @@ class AbstractStatisticsBackend(metaclass=ABCMeta):
         """
 
 
-# pylint: disable=too-few-public-methods
 class CSVStatisticsBackend(AbstractStatisticsBackend):
     """A statistics backend writing all (selected) output variables to a CSV file."""
 
     _logger = logging.getLogger(__name__)
 
-    def __init__(self) -> None:
+    def __init__(self) -> None:  # noqa: D107
         csv.field_size_limit(int(ctypes.c_ulong(-1).value // 2))
 
-    def write_data(self, data: dict[str, OutputVariable]) -> None:
+    def write_data(self, data: dict[str, OutputVariable]) -> None:  # noqa: D102
         try:
-            output_dir = Path(
-                config.configuration.statistics_output.report_dir
-            ).resolve()
+            output_dir = Path(config.configuration.statistics_output.report_dir).resolve()
             output_file = output_dir / "statistics.csv"
             with output_file.open(mode="a") as csv_file:
                 field_names = [k for k, _ in data.items()]
@@ -69,13 +66,12 @@ class CSVStatisticsBackend(AbstractStatisticsBackend):
                     csv_writer.writeheader()
                 csv_writer.writerow({k: str(v.value) for k, v in data.items()})
         except OSError as error:
-            logging.exception("Error while writing statistics: %s", error)
+            self._logger.exception("Error while writing statistics: %s", error)
 
 
-# pylint: disable=too-few-public-methods
 class ConsoleStatisticsBackend(AbstractStatisticsBackend):
     """Simple dummy backend that just outputs all output variables to the console."""
 
-    def write_data(self, data: dict[str, OutputVariable]) -> None:
+    def write_data(self, data: dict[str, OutputVariable]) -> None:  # noqa: D102
         for key, value in data.items():
             print(f"{key}: {value}")  # noqa: T201
