@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 """Provides utilities when working with types."""
+
 from __future__ import annotations
 
 import enum
@@ -91,7 +92,7 @@ def is_none_type(typ: type | None) -> bool:
     Returns:
         Whether or not the given type is NoneType
     """
-    return typ is type(None)  # noqa: E721
+    return typ is type(None)
 
 
 def is_numeric(value: Any) -> bool:
@@ -127,7 +128,7 @@ def is_bytes(value: Any) -> bool:
     Returns:
         Whether or not the given value is of type bytes or bytearray
     """
-    return isinstance(value, (bytes, bytearray))
+    return isinstance(value, bytes | bytearray)
 
 
 def is_list(typ: type) -> bool:
@@ -191,8 +192,7 @@ def is_enum(value: Any) -> bool:
 
 
 def is_assertable(obj: Any, recursion_depth: int = 0) -> bool:
-    """Check if we can generate an assertion with the given object as an
-    exact comparison value.
+    """Returns whether we can generate an assertion using an exact comparison value.
 
     Primitives (except float) are assertable.
     Enum values are assertable.
@@ -223,8 +223,7 @@ def is_assertable(obj: Any, recursion_depth: int = 0) -> bool:
         return all(is_assertable(elem, recursion_depth + 1) for elem in obj)
     if is_dict(tp_):
         return all(
-            is_assertable(key, recursion_depth + 1)
-            and is_assertable(value, recursion_depth + 1)
+            is_assertable(key, recursion_depth + 1) and is_assertable(value, recursion_depth + 1)
             for key, value in obj.items()
         )
     return False
@@ -260,7 +259,9 @@ def get_class_that_defined_method(method: object) -> object | None:
 
 
 def is_optional_parameter(inf_sig: InferredSignature, parameter_name: str) -> bool:
-    """There are some parameter types (*args, **kwargs, parameters with default) that
+    """Returns, whether a parameter is optional.
+
+    There are some parameter types (*args, **kwargs, parameters with default) that
     are optional.
 
     This is a simple utility method to check if the given parameter is optional.
@@ -273,18 +274,36 @@ def is_optional_parameter(inf_sig: InferredSignature, parameter_name: str) -> bo
         Whether or not this parameter is optional.
     """
     parameter: inspect.Parameter = inf_sig.signature.parameters[parameter_name]
+    # parameter.kind might not be hashable
     return (
-        parameter.kind
-        in (
-            inspect.Parameter.VAR_POSITIONAL,
-            inspect.Parameter.VAR_KEYWORD,
-        )
+        parameter.kind == inspect.Parameter.VAR_POSITIONAL  # noqa: PLR1714
+        or parameter.kind == inspect.Parameter.VAR_KEYWORD
         or parameter.default is not inspect.Parameter.empty
     )
 
 
+def is_arg_or_kwarg(inf_sig: InferredSignature, parameter_name: str) -> bool:
+    """Returns, whether a parameter is *args or **kwarg.
+
+    Args:
+        inf_sig: the inferred signature
+        parameter_name: the name of the parameter
+
+    Returns:
+        Whether this parameter is *arg or **kwarg.
+    """
+    parameter: inspect.Parameter = inf_sig.signature.parameters[parameter_name]
+    # parameter.kind might not be hashable
+    return (
+        parameter.kind == inspect.Parameter.VAR_POSITIONAL  # noqa: PLR1714
+        or parameter.kind == inspect.Parameter.VAR_KEYWORD
+    )
+
+
 def given_exception_matches(err, exc) -> bool:
-    """This is a naive approach to figure out if an exception matches, similar to
+    """Returns whether a raised exception matches an exception class.
+
+    This is a naive approach to figure out if an exception matches, similar to
     what CPython does here:
     https://github.com/python/cpython/blob/ae3c66acb89a6104fcd0eea760f80a0287327cc4/Python/errors.c#L231.
 
@@ -300,3 +319,54 @@ def given_exception_matches(err, exc) -> bool:
     if not isclass(err):
         err = type(err)
     return issubclass(err, exc)
+
+
+def string_distance(string1: str, string2: str) -> float:
+    """Returns the distance between two strings.
+
+    Unlike the Levenshtein distance, this calculates not just the
+    number of edits, but the character distance for left aligned strings.
+    Since we count each missing character as distance 1, character distances
+    are normalised in [0,1].
+    """
+    if string1 == string2:
+        return 0.0
+
+    min_length = min(len(string1), len(string2))
+    max_length = max(len(string1), len(string2))
+    differences: float = max_length - min_length
+
+    for pos in range(min_length):
+        if string1[pos] != string2[pos]:
+            difference = abs(ord(string1[pos]) - ord(string2[pos]))
+            differences += difference / (difference + 1.0)
+
+    return differences
+
+
+def string_lt_distance(string1: str, string2: str) -> int:
+    """Returns the strict ordered distance between two strings."""
+    if string1 < string2:
+        return 0
+
+    min_length = min(len(string1), len(string2))
+
+    for pos in range(min_length):
+        if string1[pos] > string2[pos]:
+            return ord(string1[pos]) - ord(string2[pos]) + 1
+
+    return 1
+
+
+def string_le_distance(string1: str, string2: str) -> int:
+    """Returns the ordered distance between two strings."""
+    if string1 <= string2:
+        return 0
+
+    min_length = min(len(string1), len(string2))
+
+    for pos in range(min_length):
+        if string1[pos] > string2[pos]:
+            return ord(string1[pos]) - ord(string2[pos])
+
+    return 1
