@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -30,6 +30,7 @@ from asciitree import LeftAligned
 from asciitree.drawing import BOX_LIGHT
 
 from pynguin.utils.orderedset import OrderedSet
+from pynguin.utils.orderedset import OrderedTypeSet
 
 
 LOGGER = logging.getLogger(__name__)
@@ -52,7 +53,7 @@ class UsageTraceNode:
     children: dict[str, UsageTraceNode] = dataclasses.field(init=False)
 
     # The type against which this proxy was checked.
-    type_checks: OrderedSet[type] = dataclasses.field(default_factory=OrderedSet)
+    type_checks: OrderedTypeSet = dataclasses.field(default_factory=OrderedTypeSet)
 
     # Maps argument positions to their types.
     arg_types: dict[int, OrderedSet[type]] = dataclasses.field(
@@ -60,6 +61,7 @@ class UsageTraceNode:
     )
 
     def __post_init__(self):
+        """Initialize the attribute with a specific dictionary."""
         self.children = DepthDefaultDict(self.depth)
 
     def find_path(self, path: tuple[str, ...]) -> UsageTraceNode | None:
@@ -91,42 +93,44 @@ class UsageTraceNode:
         )
         return tree({self._format_str(): self._format_children()})
 
-    def __len__(self):
+    def __len__(self) -> int:
+        """Yield the length of a usage-trace node.
+
+        The length is defined by the length of its children, argument types, and type
+        checks.
+
+        Returns:
+            The length of a usage-trace node
+        """
         return len(self.children) + len(self.arg_types) + len(self.type_checks)
 
     def _format_str(self):
         output = f"'{self.name}'"
         if len(self.type_checks) > 0:
             output += (
-                ", type_checks: {"
-                + ", ".join([check.__name__ for check in self.type_checks])
-                + "}"
+                ", type_checks: {" + ", ".join([check.__name__ for check in self.type_checks]) + "}"
             )
         if len(self.arg_types) > 0:
             output += (
                 ", arg_types: {"
-                + ", ".join(
-                    [
-                        str(idx)
-                        + ": {"
-                        + ", ".join([tp.__name__ for tp in types])
-                        + "}"
-                        for idx, types in self.arg_types.items()
-                    ]
-                )
+                + ", ".join([
+                    str(idx) + ": {" + ", ".join([tp.__name__ for tp in types]) + "}"
+                    for idx, types in self.arg_types.items()
+                ])
                 + "}"
             )
         return output
 
     def _format_children(self):
         return {
-            child._format_str(): child._format_children()
+            child._format_str(): child._format_children()  # noqa: SLF001
             for child in self.children.values()
         }
 
     @staticmethod
     def from_proxy(obj: ObjectProxy) -> UsageTraceNode:
         """Extract knowledge from the given proxy.
+
         This is a convenience method, because the knowledge attribute is not visible
         on a proxy.
 
@@ -136,7 +140,7 @@ class UsageTraceNode:
         Returns:
             The extracted knowledge.
         """
-        return obj._self_usage_trace_node
+        return obj._self_usage_trace_node  # noqa: SLF001
 
     def merge(self, other: UsageTraceNode) -> None:
         """Merge the knowledge from the other proxy into this one.
@@ -152,22 +156,35 @@ class UsageTraceNode:
             self.children[attr].merge(knowledge)
 
 
-class DepthDefaultDict(dict[str, UsageTraceNode]):
-    """Default dict which automatically creates a UsageTraceNode for each requested
-    and non-existing key.
+class DepthDefaultDict(dict[str, UsageTraceNode]):  # noqa: FURB189
+    """A dictionary creating a UsageTraceNode automatically for each key.
+
+    The implementation creates a UsageTraceNode for each requested and non-existing key.
     """
 
-    def __init__(self, depth: int):
+    def __init__(self, depth: int) -> None:
+        """Initializes the dictionary.
+
+        Args:
+            depth: The depth of the trace
+        """
         super().__init__()
         self._depth = depth
 
-    def __missing__(self, key):
-        # Create knowledge for missing attribute
+    def __missing__(self, key: str) -> UsageTraceNode:
+        """Creates the knowledge for a missing key.
+
+        Args:
+            key: The name of the key
+
+        Returns:
+            The instance of this dictionary
+        """
         res = self[key] = UsageTraceNode(key, depth=self._depth + 1)
         return res
 
 
-def proxify(log_arg_types=False, no_wrap_return=False):
+def proxify(*, log_arg_types=False, no_wrap_return=False):
     """Decorator to wrap the result of a dunder method in a proxy.
 
     Args:
@@ -202,7 +219,6 @@ def proxify(log_arg_types=False, no_wrap_return=False):
 
 
 class _ObjectProxyMethods:
-
     # We use properties to override the values of __module__ and
     # __doc__. If we add these in ObjectProxy, the derived class
     # __dict__ will still be setup to have string variants of these
@@ -214,24 +230,18 @@ class _ObjectProxyMethods:
 
     @property
     def __module__(self):
-        return (  # pylint:disable=no-member
-            self.__wrapped__.__module__  # type: ignore[attr-defined]
-        )
+        return self.__wrapped__.__module__  # type: ignore[attr-defined]
 
     @__module__.setter
     def __module__(self, value):
-        # pylint:disable=no-member
         self.__wrapped__.__module__ = value  # type: ignore[attr-defined]
 
     @property
     def __doc__(self):
-        return (  # pylint:disable=no-member
-            self.__wrapped__.__doc__  # type: ignore[attr-defined]
-        )
+        return self.__wrapped__.__doc__  # type: ignore[attr-defined]
 
     @__doc__.setter
     def __doc__(self, value):
-        # pylint:disable=no-member
         self.__wrapped__.__doc__ = value  # type: ignore[attr-defined]
 
     # We similar use a property for __dict__. We need __dict__ to be
@@ -239,9 +249,7 @@ class _ObjectProxyMethods:
 
     @property
     def __dict__(self):
-        return (  # pylint:disable=no-member
-            self.__wrapped__.__dict__  # type: ignore[attr-defined]
-        )
+        return self.__wrapped__.__dict__  # type: ignore[attr-defined]
 
     # Need to also propagate the special __weakref__ attribute for case
     # where decorating classes which will define this. If do not define
@@ -250,9 +258,7 @@ class _ObjectProxyMethods:
 
     @property
     def __weakref__(self):
-        return (  # pylint:disable=no-member
-            self.__wrapped__.__weakref__  # type: ignore[attr-defined]
-        )
+        return self.__wrapped__.__weakref__  # type: ignore[attr-defined]
 
 
 class _ObjectProxyMetaType(type):
@@ -281,7 +287,7 @@ def unwrap(obj):
     return obj
 
 
-class ObjectProxy(metaclass=_ObjectProxyMetaType):
+class ObjectProxy(metaclass=_ObjectProxyMetaType):  # noqa: PLR0904
     """A proxy for (almost) any Python object.
 
     Native types implemented in C might be problematic.
@@ -290,9 +296,17 @@ class ObjectProxy(metaclass=_ObjectProxyMetaType):
     def __init__(
         self,
         wrapped,
+        *,
         usage_trace: UsageTraceNode | None = None,
         is_kwargs: bool = False,
-    ):
+    ) -> None:
+        """Initializes the proxy around a wrapped object.
+
+        Args:
+            wrapped: The wrapped object
+            usage_trace: An optional usage-trace node
+            is_kwargs: Whether the proxy is passed as **kwargs
+        """
         object.__setattr__(self, "__wrapped__", wrapped)
         # What does this proxy know?
         object.__setattr__(
@@ -311,26 +325,20 @@ class ObjectProxy(metaclass=_ObjectProxyMetaType):
         # Python 3.2+ has the __qualname__ attribute, but it does not
         # allow it to be overridden using a property and it must instead
         # be an actual string object instead.
-
-        try:
+        with contextlib.suppress(AttributeError):
             object.__setattr__(self, "__qualname__", wrapped.__qualname__)
-        except AttributeError:
-            pass
 
         # Python 3.10 onwards also does not allow itself to be overridden
         # using a property and it must instead be set explicitly.
-
-        try:
+        with contextlib.suppress(AttributeError):
             object.__setattr__(self, "__annotations__", wrapped.__annotations__)
-        except AttributeError:
-            pass
 
     @property
-    def __name__(self):
+    def __name__(self):  # noqa: PLW3201
         return self.__wrapped__.__name__  # type:ignore[has-type]
 
     @__name__.setter
-    def __name__(self, value):
+    def __name__(self, value):  # noqa: PLW3201
         self.__wrapped__.__name__ = value  # type:ignore[has-type]
 
     @property
@@ -338,7 +346,7 @@ class ObjectProxy(metaclass=_ObjectProxyMetaType):
         return self.__wrapped__.__class__  # type:ignore[has-type]
 
     @__class__.setter
-    def __class__(self, value):  # noqa: F811
+    def __class__(self, value):
         self.__wrapped__.__class__ = value  # type:ignore[has-type]
 
     def __dir__(self):
@@ -361,7 +369,7 @@ class ObjectProxy(metaclass=_ObjectProxyMetaType):
     def __round__(self, *args):
         return round(self.__wrapped__, *args)  # type:ignore[has-type]
 
-    def __mro_entries__(self, bases):  # pylint:disable=unused-argument
+    def __mro_entries__(self, bases):
         return (self.__wrapped__,)  # type:ignore[has-type]
 
     @proxify(log_arg_types=True, no_wrap_return=True)
@@ -401,28 +409,16 @@ class ObjectProxy(metaclass=_ObjectProxyMetaType):
 
         elif name == "__wrapped__":
             object.__setattr__(self, name, value)
-            try:
+            with contextlib.suppress(AttributeError):
                 object.__delattr__(self, "__qualname__")
-            except AttributeError:
-                pass
-            try:
+            with contextlib.suppress(AttributeError):
                 object.__setattr__(self, "__qualname__", value.__qualname__)
-            except AttributeError:
-                pass
-            try:
+            with contextlib.suppress(AttributeError):
                 object.__delattr__(self, "__annotations__")
-            except AttributeError:
-                pass
-            try:
+            with contextlib.suppress(AttributeError):
                 object.__setattr__(self, "__annotations__", value.__annotations__)
-            except AttributeError:
-                pass
 
-        elif name == "__qualname__":
-            setattr(self.__wrapped__, name, value)  # type:ignore[has-type]
-            object.__setattr__(self, name, value)
-
-        elif name == "__annotations__":
+        elif name in {"__qualname__", "__annotations__"}:
             setattr(self.__wrapped__, name, value)  # type:ignore[has-type]
             object.__setattr__(self, name, value)
 
@@ -597,9 +593,9 @@ class ObjectProxy(metaclass=_ObjectProxyMetaType):
 
     @proxify(log_arg_types=True)
     def __itruediv__(self, other):  # type:ignore[misc]
-        # pylint:disable=attribute-defined-outside-init
         self.__wrapped__ = operator.itruediv(
-            self.__wrapped__, other  # type: ignore[has-type]
+            self.__wrapped__,  # type: ignore[has-type]
+            other,
         )
         return self
 
@@ -714,10 +710,12 @@ class ObjectProxy(metaclass=_ObjectProxyMetaType):
 
     # These do not give us any hint.
     # def __copy__(self):
-    #     raise NotImplementedError('object proxy must define __copy__()')
+    #     raise NotImplementedError(
+    #             'object proxy must define __copy__()')
     #
     # def __deepcopy__(self, memo):
-    #     raise NotImplementedError('object proxy must define __deepcopy__()')
+    #     raise NotImplementedError(
+    #             'object proxy must define __deepcopy__()')
     #
     # def __reduce__(self):
     #     raise NotImplementedError(
@@ -728,13 +726,14 @@ class ObjectProxy(metaclass=_ObjectProxyMetaType):
     #             'object proxy must define __reduce_ex__()')
 
     @proxify(log_arg_types=True)
-    def __call__(self, *args, **kwargs):
+    def __call__(self, *args, **kwargs):  # noqa: D102
         return self.__wrapped__(*args, **kwargs)
 
 
 @contextlib.contextmanager
 def shim_isinstance():
     """Context manager that temporarily replaces isinstance with a shim.
+
     The shim is aware of ObjectProxies.
 
     Yields:
@@ -743,15 +742,11 @@ def shim_isinstance():
     orig_isinstance = builtins.isinstance
 
     def shim(inst, types):
-        # pylint:disable=unidiomatic-typecheck
         if type(inst) is ObjectProxy:
             if types is ObjectProxy or orig_isinstance(types, ObjectProxy):
                 return orig_isinstance(inst, types)
             if orig_isinstance(types, tuple):
-                if any(
-                    typ is ObjectProxy or orig_isinstance(typ, ObjectProxy)
-                    for typ in types
-                ):
+                if any(typ is ObjectProxy or orig_isinstance(typ, ObjectProxy) for typ in types):
                     return orig_isinstance(inst, types)
                 UsageTraceNode.from_proxy(inst).type_checks.update(types)
             else:
