@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -15,12 +15,12 @@ from pynguin.analyses.constants import RestrictedConstantPool
 from pynguin.utils.orderedset import OrderedSet
 
 
-@pytest.fixture()
+@pytest.fixture
 def pool() -> ConstantPool:
     return ConstantPool()
 
 
-@pytest.fixture()
+@pytest.fixture
 def rpool() -> ConstantPool:
     return RestrictedConstantPool(max_size=5)
 
