@@ -1,17 +1,18 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 
+import operator
+
 #  This file is part of Pynguin.
 #
 #
 #  SPDX-License-Identifier: MIT
 #
-import os
-
+from pathlib import Path
 from unittest import mock
 from unittest.mock import MagicMock
 
@@ -26,40 +27,32 @@ from pynguin.analyses import seeding
 from pynguin.analyses.constants import EmptyConstantProvider
 from pynguin.analyses.module import ModuleTestCluster
 from pynguin.analyses.module import generate_test_cluster
-from pynguin.generation.generationalgorithmfactory import (
-    TestSuiteGenerationAlgorithmFactory,
-)
+from pynguin.ga.generationalgorithmfactory import TestSuiteGenerationAlgorithmFactory
 
 
-@pytest.fixture()
+@pytest.fixture
 def seed_modules_path():
-    dummy_test_file = os.path.join(
-        os.path.dirname(__file__),
-        "",
-        "..",
-        "fixtures",
-        "seeding",
-        "initialpopulationseeding",
-        "seedmodules",
+    return (
+        Path(__file__).parent
+        / ".."
+        / "fixtures"
+        / "seeding"
+        / "initialpopulationseeding"
+        / "seedmodules"
     )
-    return dummy_test_file
 
 
-@pytest.fixture()
+@pytest.fixture
 def triangle_test_cluster() -> ModuleTestCluster:
-    test_cluster = generate_test_cluster("tests.fixtures.examples.triangle")
-    return test_cluster
+    return generate_test_cluster("tests.fixtures.examples.triangle")
 
 
-@pytest.fixture()
+@pytest.fixture
 def dummy_test_cluster() -> ModuleTestCluster:
-    test_cluster = generate_test_cluster(
-        "tests.fixtures.seeding.initialpopulationseeding.dummycontainer"
-    )
-    return test_cluster
+    return generate_test_cluster("tests.fixtures.seeding.initialpopulationseeding.dummycontainer")
 
 
-@pytest.fixture()
+@pytest.fixture
 def constant_provider():
     return EmptyConstantProvider()
 
@@ -76,9 +69,7 @@ def test_get_testcases(constant_provider, seed_modules_path, triangle_test_clust
     assert len(provider) == 2
 
 
-def test_get_seeded_testcase(
-    constant_provider, seed_modules_path, triangle_test_cluster
-):
+def test_get_seeded_testcase(constant_provider, seed_modules_path, triangle_test_cluster):
     config.configuration.module_name = "triangle"
     provider = seeding.InitialPopulationProvider(
         triangle_test_cluster,
@@ -115,7 +106,7 @@ def test_get_seeded_testcase(
     ],
 )
 @mock.patch("pynguin.utils.randomness.choice")
-def test_collect_different_types(
+def test_collect_different_types(  # noqa: PLR0917
     rand_mock,
     constant_provider,
     seed_modules_path,
@@ -133,7 +124,7 @@ def test_collect_different_types(
         constant_provider,
     )
     provider.collect_testcases(seed_modules_path)
-    rand_mock.side_effect = lambda x: x[testcase_pos]
+    rand_mock.side_effect = operator.itemgetter(testcase_pos)
 
     seeded_testcase = provider.random_testcase()
     assert seeded_testcase is not None
@@ -150,7 +141,7 @@ def test_collect_different_types(
     ],
 )
 @mock.patch("pynguin.utils.randomness.choice")
-def test_create_assertion(
+def test_create_assertion(  # noqa: PLR0917
     rand_mock,
     constant_provider,
     seed_modules_path,
@@ -166,7 +157,7 @@ def test_create_assertion(
         constant_provider,
     )
     provider.collect_testcases(seed_modules_path)
-    rand_mock.side_effect = lambda x: x[testcase_pos]
+    rand_mock.side_effect = operator.itemgetter(testcase_pos)
 
     seeded_testcase = provider.random_testcase()
     assert len(seeded_testcase.statements[position].assertions) == num_assertions
@@ -181,9 +172,7 @@ def test_create_assertion(
         pytest.param("wrongassignseed"),
     ],
 )
-def test_not_working_cases(
-    constant_provider, seed_modules_path, dummy_test_cluster, module_name
-):
+def test_not_working_cases(constant_provider, seed_modules_path, dummy_test_cluster, module_name):
     config.configuration.module_name = module_name
     provider = seeding.InitialPopulationProvider(
         dummy_test_cluster,
@@ -199,7 +188,7 @@ def test_not_working_cases(
 def test_seeded_test_case_factory_no_delegation(
     rand_mock, constant_provider, seed_modules_path, dummy_test_cluster
 ):
-    rand_mock.side_effect = lambda x: x[2]
+    rand_mock.side_effect = operator.itemgetter(2)
     test_factory = tf.TestFactory(dummy_test_cluster, constant_provider)
     provider = seeding.InitialPopulationProvider(
         dummy_test_cluster, test_factory, constant_provider
@@ -220,7 +209,7 @@ def test_seeded_test_case_factory_no_delegation(
 def test_seeded_test_case_factory_with_delegation(
     rand_mock, constant_provider, seed_modules_path, dummy_test_cluster
 ):
-    rand_mock.side_effect = lambda x: x[2]  # pragma: no cover
+    rand_mock.side_effect = operator.itemgetter(2)  # pragma: no cover
     test_factory = tf.TestFactory(dummy_test_cluster, constant_provider)
     provider = seeding.InitialPopulationProvider(
         dummy_test_cluster, test_factory, constant_provider
@@ -253,21 +242,17 @@ def test_algorithm_generation_factory(
     tsfactory = TestSuiteGenerationAlgorithmFactory(
         mock_class.return_value, dummy_test_cluster, constant_provider
     )
-    with mock.patch(
-        "pynguin.analyses.seeding.InitialPopulationProvider.__len__"
-    ) as len_mock:
+    with mock.patch("pynguin.analyses.seeding.InitialPopulationProvider.__len__") as len_mock:
         len_mock.return_value = 1
         chromosome_factory = tsfactory._get_chromosome_factory(
             MagicMock(test_case_fitness_functions=[], test_suite_fitness_functions=[])
         )
     test_case_factory = chromosome_factory._test_case_factory
-    assert type(test_case_factory) == fac_type
+    assert type(test_case_factory) is fac_type
 
 
 @mock.patch("ast.parse")
-def test_module_not_readable(
-    parse_mock, constant_provider, seed_modules_path, dummy_test_cluster
-):
+def test_module_not_readable(parse_mock, constant_provider, seed_modules_path, dummy_test_cluster):
     test_factory = tf.TestFactory(dummy_test_cluster, constant_provider)
     provider = seeding.InitialPopulationProvider(
         dummy_test_cluster, test_factory, constant_provider
@@ -279,9 +264,7 @@ def test_module_not_readable(
 
 
 @mock.patch("pynguin.ga.testcasechromosome.TestCaseChromosome.mutate")
-def test_initial_mutation(
-    mutate_mock, constant_provider, seed_modules_path, dummy_test_cluster
-):
+def test_initial_mutation(mutate_mock, constant_provider, seed_modules_path, dummy_test_cluster):
     config.configuration.seeding.initial_population_mutations = 2
     config.configuration.module_name = "primitiveseed"
     test_factory = tf.TestFactory(dummy_test_cluster, constant_provider)
