@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -11,6 +11,7 @@ from logging import Logger
 from typing import Union
 from typing import cast
 from unittest.mock import MagicMock
+from unittest.mock import patch
 
 import pytest
 
@@ -28,12 +29,14 @@ from pynguin.analyses.typesystem import ProperType
 from pynguin.analyses.typesystem import TypeInfo
 from pynguin.analyses.typesystem import UnionType
 from pynguin.utils.exceptions import ConstructionFailedException
+from pynguin.utils.exceptions import CoroutineFoundException
 from pynguin.utils.generic.genericaccessibleobject import GenericAccessibleObject
 from pynguin.utils.generic.genericaccessibleobject import GenericConstructor
 from pynguin.utils.generic.genericaccessibleobject import GenericEnum
 from pynguin.utils.generic.genericaccessibleobject import GenericFunction
 from pynguin.utils.generic.genericaccessibleobject import GenericMethod
 from pynguin.utils.orderedset import OrderedSet
+from pynguin.utils.statistics.runtimevariable import RuntimeVariable
 from pynguin.utils.type_utils import COLLECTIONS
 from pynguin.utils.type_utils import PRIMITIVES
 
@@ -58,6 +61,16 @@ def parsed_module_nested_functions() -> _ModuleParseResult:
     return parse_module("tests.fixtures.cluster.nested_functions")
 
 
+@pytest.fixture(scope="module")
+def parsed_module_lambda() -> _ModuleParseResult:
+    return parse_module("tests.fixtures.cluster.lambda")
+
+
+@pytest.fixture(scope="module")
+def parsed_module_unnamed_lambda() -> _ModuleParseResult:
+    return parse_module("tests.fixtures.cluster.unnamed_lambda")
+
+
 @pytest.fixture
 def module_test_cluster() -> ModuleTestCluster:
     return ModuleTestCluster(linenos=-1)
@@ -71,9 +84,9 @@ def test_parse_module(parsed_module_no_dependencies):
     assert parse_result.syntax_tree is not None
 
 
-def test_parse_c_module():
+def test_parse_native_module():
     module.LOGGER = MagicMock(Logger)
-    module_name = "jellyfish.cjellyfish"
+    module_name = "libcst.native"
     parse_result = parse_module(module_name)
     assert parse_result.module.__name__ == module_name
     assert parse_result.module_name == module_name
@@ -81,6 +94,29 @@ def test_parse_c_module():
     module.LOGGER.debug.assert_called_once()
 
 
+@pytest.mark.parametrize("exception_type", [TypeError, OSError, RuntimeError])
+@patch("astroid.parse")
+def test_parse_module_exceptions(mock_parse, exception_type):
+    mock_parse.side_effect = exception_type("Mocked Exception")
+    module.LOGGER = MagicMock(Logger)
+    module_name = "tests.fixtures.cluster.no_dependencies"
+    result = parse_module(module_name)
+    assert result.module_name == module_name
+    assert result.syntax_tree is None
+    assert result.linenos == -1
+    module.LOGGER.debug.assert_called_once()
+
+
+def test_parse_alias_module():
+    module.LOGGER = MagicMock(Logger)
+    module_name = "tests.fixtures.cluster.dependency"
+    alias_name = "tests.fixtures.cluster.alias_module.dep"
+    parse_result = parse_module(alias_name)
+    assert parse_result.module.__name__ == module_name
+    assert parse_result.module_name == alias_name
+    assert parse_result.syntax_tree is not None
+
+
 def test_analyse_module(parsed_module_no_dependencies):
     test_cluster = analyse_module(parsed_module_no_dependencies)
     assert test_cluster.num_accessible_objects_under_test() == 4
@@ -95,19 +131,17 @@ def test_analyse_module_dependencies(parsed_module_complex_dependencies):
 
 def test_add_generator_primitive(module_test_cluster):
     generator = MagicMock(GenericMethod)
-    generator.generated_type.return_value = (
-        module_test_cluster.type_system.convert_type_hint(int)
-    )
+    generator.generated_type.return_value = module_test_cluster.type_system.convert_type_hint(int)
     module_test_cluster.add_generator(generator)
     assert module_test_cluster.get_generators_for(
         module_test_cluster.type_system.convert_type_hint(int)
     ) == (OrderedSet([]), True)
 
 
-def test_add_generator(module_test_cluster, type_system):
+def test_add_generator(module_test_cluster):
     generator = MagicMock(GenericMethod)
-    generator.generated_type.return_value = (
-        module_test_cluster.type_system.convert_type_hint(MagicMock)
+    generator.generated_type.return_value = module_test_cluster.type_system.convert_type_hint(
+        MagicMock
     )
     module_test_cluster.add_generator(generator)
     assert module_test_cluster.get_generators_for(
@@ -117,13 +151,13 @@ def test_add_generator(module_test_cluster, type_system):
 
 def test_add_generator_two(module_test_cluster):
     generator = MagicMock(GenericMethod)
-    generator.generated_type.return_value = (
-        module_test_cluster.type_system.convert_type_hint(MagicMock)
+    generator.generated_type.return_value = module_test_cluster.type_system.convert_type_hint(
+        MagicMock
     )
     module_test_cluster.add_generator(generator)
     generator_2 = MagicMock(GenericMethod)
-    generator_2.generated_type.return_value = (
-        module_test_cluster.type_system.convert_type_hint(MagicMock)
+    generator_2.generated_type.return_value = module_test_cluster.type_system.convert_type_hint(
+        MagicMock
     )
     module_test_cluster.add_generator(generator_2)
     assert module_test_cluster.get_generators_for(
@@ -141,8 +175,8 @@ def test_add_accessible_object_under_test(module_test_cluster):
 
 def test_add_modifier(module_test_cluster):
     modifier = MagicMock(GenericMethod)
-    modifier.generated_type.return_value = (
-        module_test_cluster.type_system.convert_type_hint(MagicMock)
+    modifier.generated_type.return_value = module_test_cluster.type_system.convert_type_hint(
+        MagicMock
     )
     module_test_cluster.add_modifier(
         module_test_cluster.type_system.to_type_info(MagicMock), modifier
@@ -155,14 +189,10 @@ def test_add_modifier(module_test_cluster):
 def test_add_modifier_two(module_test_cluster):
     modifier = MagicMock(GenericMethod)
     modifier.generated_type.return_value = MagicMock
-    module_test_cluster.add_modifier(
-        module_test_cluster.type_system.to_type_info(int), modifier
-    )
+    module_test_cluster.add_modifier(module_test_cluster.type_system.to_type_info(int), modifier)
     modifier2 = MagicMock(GenericMethod)
     modifier2.generated_type.return_value = MagicMock
-    module_test_cluster.add_modifier(
-        module_test_cluster.type_system.to_type_info(int), modifier2
-    )
+    module_test_cluster.add_modifier(module_test_cluster.type_system.to_type_info(int), modifier2)
     assert module_test_cluster.get_modifiers_for(
         module_test_cluster.type_system.convert_type_hint(int)
     ) == OrderedSet([modifier, modifier2])
@@ -171,14 +201,10 @@ def test_add_modifier_two(module_test_cluster):
 def test_get_random_modifier(module_test_cluster):
     modifier = MagicMock(GenericMethod)
     modifier.generated_type.return_value = MagicMock
-    module_test_cluster.add_modifier(
-        module_test_cluster.type_system.to_type_info(int), modifier
-    )
+    module_test_cluster.add_modifier(module_test_cluster.type_system.to_type_info(int), modifier)
     modifier2 = MagicMock(GenericMethod)
     modifier2.generated_type.return_value = MagicMock
-    module_test_cluster.add_modifier(
-        module_test_cluster.type_system.to_type_info(int), modifier2
-    )
+    module_test_cluster.add_modifier(module_test_cluster.type_system.to_type_info(int), modifier2)
     assert module_test_cluster.get_random_call_for(
         module_test_cluster.type_system.convert_type_hint(int)
     ) in {modifier, modifier2}
@@ -216,7 +242,7 @@ def test_get_random_accessible_two(module_test_cluster):
     "type_, result",
     [
         pytest.param(bool, [bool]),
-        pytest.param(Union[int, float], [int, float]),
+        pytest.param(Union[int, float], [int, float]),  # noqa: UP007
     ],
 )
 def test_select_concrete_type_union_unary(type_, result, module_test_cluster):
@@ -227,8 +253,8 @@ def test_select_concrete_type_union_unary(type_, result, module_test_cluster):
 
 def test_select_concrete_type_any(module_test_cluster):
     generator = MagicMock(GenericMethod)
-    generator.generated_type.return_value = (
-        module_test_cluster.type_system.convert_type_hint(MagicMock)
+    generator.generated_type.return_value = module_test_cluster.type_system.convert_type_hint(
+        MagicMock
     )
     module_test_cluster.add_generator(generator)
     assert (
@@ -247,8 +273,8 @@ def test_get_all_generatable_types_only_builtin(module_test_cluster):
 
 def test_get_all_generatable_types(module_test_cluster):
     generator = MagicMock(GenericMethod)
-    generator.generated_type.return_value = (
-        module_test_cluster.type_system.convert_type_hint(MagicMock)
+    generator.generated_type.return_value = module_test_cluster.type_system.convert_type_hint(
+        MagicMock
     )
     module_test_cluster.add_generator(generator)
     expected = {
@@ -266,9 +292,11 @@ def __extract_method_names(
     accessible_objects: OrderedSet[GenericAccessibleObject],
 ) -> set[str]:
     return {
-        f"{elem.owner.name}.{elem.callable.__name__}"
-        if isinstance(elem, GenericMethod)
-        else f"{elem.owner.name}.__init__"
+        (
+            f"{elem.owner.name}.{elem.callable.__name__}"
+            if isinstance(elem, GenericMethod)
+            else f"{elem.owner.name}.__init__"
+        )
         for elem in accessible_objects
     }
 
@@ -306,14 +334,8 @@ def test_nothing_included_multiple_times():
 
 def test_generators():
     cluster = generate_test_cluster("tests.fixtures.cluster.no_dependencies")
-    assert (
-        len(cluster.get_generators_for(cluster.type_system.convert_type_hint(int))[0])
-        == 0
-    )
-    assert (
-        len(cluster.get_generators_for(cluster.type_system.convert_type_hint(float))[0])
-        == 0
-    )
+    assert len(cluster.get_generators_for(cluster.type_system.convert_type_hint(int))[0]) == 0
+    assert len(cluster.get_generators_for(cluster.type_system.convert_type_hint(float))[0]) == 0
     assert __convert_to_str_count_dict(cluster.generators) == {"Test": 1, "object": 1}
     assert cluster.num_accessible_objects_under_test() == 4
 
@@ -335,17 +357,13 @@ def test_complex_dependencies():
 
 def test_inheritance_generator():
     cluster = generate_test_cluster("tests.fixtures.cluster.inheritance")
-    from tests.fixtures.cluster.inheritance import Bar
-    from tests.fixtures.cluster.inheritance import Foo
+    from tests.fixtures.cluster.inheritance import Bar  # noqa: PLC0415
+    from tests.fixtures.cluster.inheritance import Foo  # noqa: PLC0415
 
-    res_foo, only_any = cluster.get_generators_for(
-        cluster.type_system.convert_type_hint(Foo)
-    )
+    res_foo, only_any = cluster.get_generators_for(cluster.type_system.convert_type_hint(Foo))
     assert len(res_foo) == 2
     assert not only_any
-    res_bar, only_any = cluster.get_generators_for(
-        cluster.type_system.convert_type_hint(Bar)
-    )
+    res_bar, only_any = cluster.get_generators_for(cluster.type_system.convert_type_hint(Bar))
     assert len(res_bar) == 1
     assert not only_any
 
@@ -370,8 +388,8 @@ def test_only_any_generator_3(module_test_cluster):
     generator.generated_type.return_value = ANY
     module_test_cluster.add_generator(generator)
     generator2 = MagicMock(GenericMethod)
-    generator2.generated_type.return_value = (
-        module_test_cluster.type_system.convert_type_hint(MagicMock)
+    generator2.generated_type.return_value = module_test_cluster.type_system.convert_type_hint(
+        MagicMock
     )
     module_test_cluster.add_generator(generator2)
     assert module_test_cluster.get_generators_for(
@@ -381,15 +399,11 @@ def test_only_any_generator_3(module_test_cluster):
 
 def test_inheritance_modifier():
     cluster = generate_test_cluster("tests.fixtures.cluster.inheritance")
-    from tests.fixtures.cluster.inheritance import Bar
-    from tests.fixtures.cluster.inheritance import Foo
+    from tests.fixtures.cluster.inheritance import Bar  # noqa: PLC0415
+    from tests.fixtures.cluster.inheritance import Foo  # noqa: PLC0415
 
-    assert (
-        len(cluster.get_modifiers_for(cluster.type_system.convert_type_hint(Bar))) == 2
-    )
-    assert (
-        len(cluster.get_modifiers_for(cluster.type_system.convert_type_hint(Foo))) == 1
-    )
+    assert len(cluster.get_modifiers_for(cluster.type_system.convert_type_hint(Bar))) == 2
+    assert len(cluster.get_modifiers_for(cluster.type_system.convert_type_hint(Foo))) == 1
 
 
 def test_modifier():
@@ -416,15 +430,11 @@ def test_resolve_optional():
 def test_private_method_not_added():
     cluster = generate_test_cluster("tests.fixtures.examples.private_methods")
     assert len(cluster.accessible_objects_under_test) == 1
-    assert isinstance(
-        next(iter(cluster.accessible_objects_under_test)), GenericConstructor
-    )
+    assert isinstance(next(iter(cluster.accessible_objects_under_test)), GenericConstructor)
 
 
 def test_overridden_inherited_methods():
-    cluster = generate_test_cluster(
-        "tests.fixtures.cluster.overridden_inherited_methods"
-    )
+    cluster = generate_test_cluster("tests.fixtures.cluster.overridden_inherited_methods")
     accessible_objects = cluster.accessible_objects_under_test
     methods = __extract_method_names(accessible_objects)
     expected = {"Foo.__init__", "Foo.foo", "Foo.__iter__", "Bar.__init__", "Bar.foo"}
@@ -434,15 +444,13 @@ def test_overridden_inherited_methods():
 def test_conditional_import_forward_ref():
     cluster = generate_test_cluster("tests.fixtures.cluster.conditional_import")
     accessible_objects = list(cluster.accessible_objects_under_test)
-    constructor = cast(GenericConstructor, accessible_objects[0])
+    constructor = cast("GenericConstructor", accessible_objects[0])
     assert constructor.inferred_signature.original_parameters["arg0"] == AnyType()
 
 
 def test_enums():
     cluster = generate_test_cluster("tests.fixtures.cluster.enums")
-    accessible_objects = cast(
-        list[GenericEnum], list(cluster.accessible_objects_under_test)
-    )
+    accessible_objects = cast("list[GenericEnum]", list(cluster.accessible_objects_under_test))
     assert {enum.owner.name: set(enum.names) for enum in accessible_objects} == {
         "Color": {"RED", "BLUE", "GREEN"},
         "Foo": {"FOO", "BAR"},
@@ -455,7 +463,7 @@ def test_enums():
     ["async_func", "async_gen", "async_class_gen", "async_class_method"],
 )
 def test_analyse_async_function_or_method(module_name):
-    with pytest.raises(ValueError):
+    with pytest.raises(CoroutineFoundException):
         generate_test_cluster(f"tests.fixtures.cluster.{module_name}")
 
 
@@ -527,10 +535,7 @@ def test_inheritance_graph():
 )
 def test_instance_attrs(mod, typ, attributes):
     cluster = generate_test_cluster(mod)
-    assert (
-        cluster.type_system.find_type_info(f"{mod}.{typ}").instance_attributes
-        == attributes
-    )
+    assert cluster.type_system.find_type_info(f"{mod}.{typ}").instance_attributes == attributes
 
 
 @pytest.mark.parametrize(
@@ -552,3 +557,118 @@ def test__add_or_make_union_2(type_system):
     assert ModuleTestCluster._add_or_make_union(
         ANY, type_system.convert_type_hint(int)
     ) == UnionType((type_system.convert_type_hint(int),))
+
+
+class CustomError(Exception):
+    pass
+
+
+def test_exception_during_inspect_getmembers(parsed_module_no_dependencies):
+    with patch("inspect.getmembers", side_effect=CustomError):
+        test_cluster = analyse_module(parsed_module_no_dependencies)
+        assert test_cluster.num_accessible_objects_under_test() == 3  # one less (failed)
+
+
+def test_analyse_module_lambda(parsed_module_lambda):
+    test_cluster = analyse_module(parsed_module_lambda)
+    assert test_cluster.num_accessible_objects_under_test() == 3
+    objects = list(test_cluster.accessible_objects_under_test)
+    lambda1 = cast("GenericFunction", objects[0])
+    assert lambda1.function_name == "y"
+    lambda2 = cast("GenericFunction", objects[1])
+    assert lambda2.function_name == "abc"
+    lambda3 = cast("GenericFunction", objects[2])
+    assert lambda3.function_name == "salam_aleykum"
+
+
+def test_analyse_function_lambda_no_name():
+    """Test that __analyse_function returns early when no name for a lambda is found."""
+    # Create a test cluster
+    test_cluster = ModuleTestCluster(linenos=-1)
+
+    # Create a lambda function
+    lambda_func = lambda x: x  # noqa: E731
+
+    # Mock the _get_lambda_assigned_name function to return None
+    with patch("pynguin.analyses.module._get_lambda_assigned_name", return_value=None):
+        # Call __analyse_function directly
+        module.__analyse_function(
+            func_name="<lambda>",
+            func=lambda_func,
+            type_inference_strategy=TypeInferenceStrategy.TYPE_HINTS,
+            module_tree=None,
+            test_cluster=test_cluster,
+            add_to_test=True,
+        )
+
+    # Verify that the lambda was not added to the test cluster
+    assert test_cluster.num_accessible_objects_under_test() == 0
+
+
+@pytest.mark.parametrize(
+    "subprocess,subprocess_if_recommended,expected_subprocess_mode",
+    [
+        (False, False, "False"),
+        (True, False, "True"),
+        (False, True, "True"),
+        (True, True, "True"),
+    ],
+)
+def test_analyse_module_not_sets_c_if_not_recommended(
+    monkeypatch, subprocess, subprocess_if_recommended, expected_subprocess_mode
+):
+    tracked = {}
+
+    def fake_track_output_variable(var, value):
+        tracked[var] = value
+
+    monkeypatch.setattr(module.stat, "track_output_variable", fake_track_output_variable)
+    monkeypatch.setattr(module.config.configuration, "subprocess", subprocess)
+    monkeypatch.setattr(
+        module.config.configuration, "subprocess_if_recommended", subprocess_if_recommended
+    )
+    parse_result = module.parse_module("_ctypes")
+    module.analyse_module(parse_result)
+    assert tracked[RuntimeVariable.SubprocessMode] == expected_subprocess_mode, (
+        f"Expected subprocess mode to be {expected_subprocess_mode}, "
+        f"but got {tracked[RuntimeVariable.SubprocessMode]}"
+    )
+
+
+@pytest.mark.parametrize(
+    "module_name,expected_subprocess_value,expected_c_ext_present",
+    [
+        ("_ctypes", True, "_ctypes"),
+        ("tests.fixtures.c.my_ctypes", True, "_ctypes"),  # imports _ctypes
+        ("_abc", False, None),  # _abc is whitelisted
+    ],
+)
+def test_analyse_module_sets_c_extension_and_subprocess(
+    monkeypatch, module_name, expected_subprocess_value, expected_c_ext_present
+):
+    tracked = {}
+
+    def fake_track_output_variable(var, value):
+        tracked[var] = value
+
+    monkeypatch.setattr(module.stat, "track_output_variable", fake_track_output_variable)
+    monkeypatch.setattr(module.config.configuration, "subprocess", False)
+    monkeypatch.setattr(module.config.configuration, "subprocess_if_recommended", True)
+
+    parse_result = module.parse_module(module_name)
+    module.analyse_module(parse_result)
+
+    # Check C extension tracking
+    if expected_c_ext_present is not None:
+        assert RuntimeVariable.CExtensionModules in tracked, "CExtensionModules not tracked"
+        assert expected_c_ext_present in tracked[RuntimeVariable.CExtensionModules], (
+            f"Expected {expected_c_ext_present} to be tracked as C extension, "
+            f"but got {tracked[RuntimeVariable.CExtensionModules]}"
+        )
+
+    # Check subprocess mode tracking
+    assert RuntimeVariable.SubprocessMode in tracked, "SubprocessMode not tracked"
+    assert tracked[RuntimeVariable.SubprocessMode] == str(expected_subprocess_value), (
+        f"Expected subprocess mode to be {expected_subprocess_value}, "
+        f"but got {tracked[RuntimeVariable.SubprocessMode]}"
+    )
