@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -13,7 +13,7 @@ import pynguin.ga.testcasechromosome as tcc
 from pynguin.analyses.constants import EmptyConstantProvider
 from pynguin.analyses.module import generate_test_cluster
 from pynguin.analyses.seeding import AstToTestCaseTransformer
-from pynguin.generation import export
+from pynguin.testcase import export
 
 
 @pytest.mark.parametrize(
@@ -48,13 +48,16 @@ def test_case_0():
         + testcase_seed
     )
     test_cluster = generate_test_cluster("tests.fixtures.grammar.parameters")
-    transformer = AstToTestCaseTransformer(test_cluster, False, EmptyConstantProvider())
+    transformer = AstToTestCaseTransformer(
+        test_cluster,
+        False,  # noqa: FBT003
+        EmptyConstantProvider(),
+    )
     transformer.visit(ast.parse(testcase_seed))
     export_path = tmp_path / "export.py"
     chromosome = tcc.TestCaseChromosome(transformer.testcases[0])
-    exporter = export.PyTestChromosomeToAstVisitor()
+    exporter = export.PyTestChromosomeToAstVisitor(store_call_return=True)
     chromosome.accept(exporter)
     export.save_module_to_file(exporter.to_module(), export_path)
-    with open(export_path) as f:
-        content = f.read()
-        assert content == testcase_seed
+    content = export_path.read_text(encoding="locale")
+    assert content == testcase_seed
