@@ -1,10 +1,10 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
-import os
+from pathlib import Path
 
 import pytest
 
@@ -13,14 +13,7 @@ from pynguin.analyses.constants import collect_static_constants
 
 @pytest.fixture
 def fixture_dir():
-    return os.path.join(
-        os.path.dirname(__file__),
-        "",
-        "..",
-        "fixtures",
-        "seeding",
-        "staticconstantseeding",
-    )
+    return Path(__file__).parent / ".." / "fixtures" / "seeding" / "staticconstantseeding"
 
 
 @pytest.mark.parametrize(
