@@ -1,12 +1,16 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
-"""The implementation of this module contains some code adopted from the ``darglint``
+"""Analyses for the syntax tree.
+
+The implementation of this module contains some code adopted from the ``darglint``
 library (https://github.com/terrencepreilly/darglint), which was released by Terrence
-Reilly under MIT license."""
+Reilly under MIT license.
+"""
+
 import ast
 import importlib
 import inspect
@@ -28,16 +32,13 @@ def comments_tree() -> astroid.Module:
     return astroid.parse(inspect.getsource(module), path="comments.py")
 
 
-@pytest.fixture(scope="function")
+@pytest.fixture
 def function_analysis() -> FunctionAnalysisVisitor:
     return FunctionAnalysisVisitor()
 
 
 def __parse_ast(code: str) -> ast.Module:
-    tree = ast.parse(
-        code, filename="dummy.py", type_comments=True, feature_version=(3, 8)
-    )
-    return tree
+    return ast.parse(code, filename="dummy.py", type_comments=True, feature_version=(3, 8))
 
 
 @pytest.mark.parametrize(
@@ -59,9 +60,7 @@ def test__has_decorator(comments_tree, decorators, expected):
     ],
 )
 def test_get_function_description(comments_tree, function):
-    descriptions = get_function_description(
-        get_function_node_from_ast(comments_tree, function)
-    )
+    descriptions = get_function_description(get_function_node_from_ast(comments_tree, function))
     assert descriptions.name == function
 
 
@@ -79,9 +78,7 @@ def test_get_function_description(comments_tree, function):
 )
 def test_get_method_description(comments_tree, function):
     descriptions = get_function_description(
-        get_function_node_from_ast(
-            get_class_node_from_ast(comments_tree, "AClass"), function
-        )
+        get_function_node_from_ast(get_class_node_from_ast(comments_tree, "AClass"), function)
     )
     assert descriptions.name == function
 
