@@ -1,15 +1,15 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
+# ruff: noqa: PLC2701
+
 import inspect
+import operator
 
 from typing import Any
-from typing import List
-from typing import Set
-from typing import Tuple
 from typing import TypeVar
 from typing import Union
 from unittest import mock
@@ -19,6 +19,14 @@ import pytest
 import pynguin.configuration as config
 
 from pynguin.analyses.module import generate_test_cluster
+from pynguin.analyses.typesystem import _DICT_KEY_ATTRIBUTES
+from pynguin.analyses.typesystem import _DICT_KEY_FROM_ARGUMENT_TYPES
+from pynguin.analyses.typesystem import _DICT_VALUE_ATTRIBUTES
+from pynguin.analyses.typesystem import _DICT_VALUE_FROM_ARGUMENT_TYPES
+from pynguin.analyses.typesystem import _LIST_ELEMENT_ATTRIBUTES
+from pynguin.analyses.typesystem import _LIST_ELEMENT_FROM_ARGUMENT_TYPES
+from pynguin.analyses.typesystem import _SET_ELEMENT_ATTRIBUTES
+from pynguin.analyses.typesystem import _SET_ELEMENT_FROM_ARGUMENT_TYPES
 from pynguin.analyses.typesystem import UNSUPPORTED
 from pynguin.analyses.typesystem import AnyType
 from pynguin.analyses.typesystem import InferredSignature
@@ -38,7 +46,7 @@ from tests.fixtures.types.subtyping import Sub
 from tests.fixtures.types.subtyping import Super
 
 
-def __dummy(x: int, y: int) -> int:
+def __dummy(x: int, y: int) -> int:  # noqa: FURB118
     return x * y  # pragma: no cover
 
 
@@ -54,7 +62,10 @@ def __untyped_dummy(a, b, c):
     return f"int {a} float {b} any {c}"  # pragma: no cover
 
 
-def __union_dummy(a: int | float, b: int | float) -> int | float:
+def __union_dummy(  # noqa: FURB118
+    a: int | float,  # noqa: PYI041
+    b: int | float,  # noqa: PYI041
+) -> int | float:
     return a + b  # pragma: no cover
 
 
@@ -191,7 +202,6 @@ A = TypeVar("A")
             list[int],
             Instance(TypeInfo(list), (Instance(TypeInfo(int)),)),
         ),
-        (list[int], Instance(TypeInfo(list), (Instance(TypeInfo(int)),))),
         (
             set[int],
             Instance(TypeInfo(set), (Instance(TypeInfo(int)),)),
@@ -200,18 +210,6 @@ A = TypeVar("A")
             set,
             Instance(TypeInfo(set), (AnyType(),)),
         ),
-        (set[int], Instance(TypeInfo(set), (Instance(TypeInfo(int)),))),
-        (
-            set[int],
-            Instance(TypeInfo(set), (Instance(TypeInfo(int)),)),
-        ),
-        (
-            dict[int, str],
-            Instance(
-                TypeInfo(dict),
-                (Instance(TypeInfo(int)), Instance(TypeInfo(str))),
-            ),
-        ),
         (
             dict[int, str],
             Instance(
@@ -226,13 +224,13 @@ A = TypeVar("A")
             ),
         ),
         (
-            Union[int, str],
+            Union[int, str],  # noqa: UP007
             UnionType(
                 (Instance(TypeInfo(int)), Instance(TypeInfo(str))),
             ),
         ),
         (
-            Union[int, type(None)],
+            Union[int, type(None)],  # noqa: UP007
             UnionType(
                 (NoneType(), Instance(TypeInfo(int))),
             ),
@@ -243,12 +241,6 @@ A = TypeVar("A")
                 (Instance(TypeInfo(int)), Instance(TypeInfo(str))),
             ),
         ),
-        (
-            tuple[int, str],
-            TupleType(
-                (Instance(TypeInfo(int)), Instance(TypeInfo(str))),
-            ),
-        ),
         (
             tuple,
             TupleType((AnyType(),), unknown_size=True),
@@ -262,11 +254,6 @@ A = TypeVar("A")
             NoneType(),
         ),
         (A, AnyType()),
-        (
-            list,
-            Instance(TypeInfo(list), (AnyType(),)),
-        ),
-        (tuple, TupleType((AnyType(),), unknown_size=True)),
     ],
 )
 def test_convert_type_hints(hint, expected):
@@ -279,7 +266,7 @@ def test_convert_type_hints(hint, expected):
     "hint, expected",
     [(A, UNSUPPORTED), (list[A], Instance(TypeInfo(list), (UNSUPPORTED,)))],
 )
-def test_convert_type_hint_unsupported(hint, expected):
+def test_convert_type_hint_unsupported(hint, expected):  # noqa: ARG001
     ts = TypeSystem()
     ts.convert_type_hint(hint, unsupported=UNSUPPORTED)
 
@@ -290,6 +277,7 @@ def test_unsupported_str():
 
 @pytest.fixture(scope="module")
 def subtyping_cluster():
+    config.configuration.pynguinml.ml_testing_enabled = False
     return generate_test_cluster("tests.fixtures.types.subtyping")
 
 
@@ -303,7 +291,6 @@ def subtyping_cluster():
         (tuple, int, False, False),
         (int, type(None), False, False),
         (type(None), type(None), True, True),
-        (int, type(None), False, False),
         (type(None), int, False, False),
         (tuple[str], tuple[str, int], False, False),
         (tuple[int, str], tuple[int, str], True, True),
@@ -320,7 +307,7 @@ def subtyping_cluster():
         (int | str, int | str, True, True),
         (int | str | float, int | str, False, True),
         (int | str, int | str | float, True, True),
-        (int, Union[int, None], True, True),
+        (int, Union[int, None], True, True),  # noqa: UP007
         (Sub, Super, True, True),
         (Sub, Super | int, True, True),
         (Sub, Sub | int, True, True),
@@ -346,9 +333,7 @@ def subtyping_cluster():
         (Any, Any, True, True),
     ],
 )
-def test_is_subtype(
-    subtyping_cluster, left_hint, right_hint, subtype_result, maybe_subtype_result
-):
+def test_is_subtype(subtyping_cluster, left_hint, right_hint, subtype_result, maybe_subtype_result):
     type_system = subtyping_cluster.type_system
     left = type_system.convert_type_hint(left_hint)
     right = type_system.convert_type_hint(right_hint)
@@ -514,9 +499,9 @@ def test_is_collection_type(typ, result):
 def test_find_by_symbols(symbol, types):
     test_cluster = generate_test_cluster("tests.fixtures.types.symbols")
     tps = test_cluster.type_system
-    assert test_cluster.type_system.find_by_attribute(symbol) == OrderedSet(
-        [tps.find_type_info("" + t) for t in types]
-    )
+    assert test_cluster.type_system.find_by_attribute(symbol) == OrderedSet([
+        tps.find_type_info("" + t) for t in types
+    ])
 
 
 @pytest.mark.parametrize(
@@ -623,17 +608,15 @@ def test_union_single_element():
 
 @pytest.mark.parametrize(
     "symbol, typ, result",
-    [(sym, list, list[int]) for sym in InferredSignature._LIST_ELEMENT_ATTRIBUTES]
-    + [(sym, set, set[int]) for sym in InferredSignature._SET_ELEMENT_ATTRIBUTES],
+    [(sym, list, list[int]) for sym in _LIST_ELEMENT_ATTRIBUTES]
+    + [(sym, set, set[int]) for sym in _SET_ELEMENT_ATTRIBUTES],
 )
-def test_guess_generic_types_list_set_from_elements(
-    inferred_signature, symbol, typ, result
-):
+def test_guess_generic_types_list_set_from_elements(inferred_signature, symbol, typ, result):
     config.configuration.test_creation.negate_type = 0.0
     knowledge = UsageTraceNode("ROOT")
     knowledge.children[symbol].type_checks.add(int)
     with mock.patch("pynguin.utils.randomness.choice") as choice_mock:
-        choice_mock.side_effect = lambda x: x[0]
+        choice_mock.side_effect = operator.itemgetter(0)
         assert inferred_signature._guess_generic_type_parameters_for_builtins(
             inferred_signature.type_system.convert_type_hint(typ), knowledge, 0
         ) == inferred_signature.type_system.convert_type_hint(result)
@@ -641,16 +624,14 @@ def test_guess_generic_types_list_set_from_elements(
 
 @pytest.mark.parametrize(
     "symbol, typ, result",
-    [(sym, dict, dict[int, Any]) for sym in InferredSignature._DICT_KEY_ATTRIBUTES],
+    [(sym, dict, dict[int, Any]) for sym in _DICT_KEY_ATTRIBUTES],
 )
-def test_guess_generic_types_dict_key_from_elements(
-    inferred_signature, symbol, typ, result
-):
+def test_guess_generic_types_dict_key_from_elements(inferred_signature, symbol, typ, result):
     config.configuration.test_creation.negate_type = 0.0
     knowledge = UsageTraceNode("ROOT")
     knowledge.children[symbol].type_checks.add(int)
     with mock.patch("pynguin.utils.randomness.choice") as choice_mock:
-        choice_mock.side_effect = lambda x: x[0]
+        choice_mock.side_effect = operator.itemgetter(0)
         assert inferred_signature._guess_generic_type_parameters_for_builtins(
             inferred_signature.type_system.convert_type_hint(typ), knowledge, 0
         ) == inferred_signature.type_system.convert_type_hint(result)
@@ -658,19 +639,14 @@ def test_guess_generic_types_dict_key_from_elements(
 
 @pytest.mark.parametrize(
     "symbol, typ, result",
-    [
-        (sym, dict, dict[int, Any])
-        for sym in InferredSignature._DICT_KEY_FROM_ARGUMENT_TYPES
-    ],
+    [(sym, dict, dict[int, Any]) for sym in _DICT_KEY_FROM_ARGUMENT_TYPES],
 )
-def test_guess_generic_types_dict_key_from_arguments(
-    inferred_signature, symbol, typ, result
-):
+def test_guess_generic_types_dict_key_from_arguments(inferred_signature, symbol, typ, result):
     config.configuration.test_creation.negate_type = 0.0
     knowledge = UsageTraceNode("ROOT")
     knowledge.children[symbol].arg_types[0].add(int)
     with mock.patch("pynguin.utils.randomness.choice") as choice_mock:
-        choice_mock.side_effect = lambda x: x[0]
+        choice_mock.side_effect = operator.itemgetter(0)
         assert inferred_signature._guess_generic_type_parameters_for_builtins(
             inferred_signature.type_system.convert_type_hint(typ), knowledge, 0
         ) == inferred_signature.type_system.convert_type_hint(result)
@@ -678,16 +654,14 @@ def test_guess_generic_types_dict_key_from_arguments(
 
 @pytest.mark.parametrize(
     "symbol, typ, result",
-    [(sym, dict, dict[Any, int]) for sym in InferredSignature._DICT_VALUE_ATTRIBUTES],
+    [(sym, dict, dict[Any, int]) for sym in _DICT_VALUE_ATTRIBUTES],
 )
-def test_guess_generic_types_dict_value_from_elements(
-    inferred_signature, symbol, typ, result
-):
+def test_guess_generic_types_dict_value_from_elements(inferred_signature, symbol, typ, result):
     config.configuration.test_creation.negate_type = 0.0
     knowledge = UsageTraceNode("ROOT")
     knowledge.children[symbol].type_checks.add(int)
     with mock.patch("pynguin.utils.randomness.choice") as choice_mock:
-        choice_mock.side_effect = lambda x: x[0]
+        choice_mock.side_effect = operator.itemgetter(0)
         assert inferred_signature._guess_generic_type_parameters_for_builtins(
             inferred_signature.type_system.convert_type_hint(typ), knowledge, 0
         ) == inferred_signature.type_system.convert_type_hint(result)
@@ -695,19 +669,14 @@ def test_guess_generic_types_dict_value_from_elements(
 
 @pytest.mark.parametrize(
     "symbol, typ, result",
-    [
-        (sym, dict, dict[Any, int])
-        for sym in InferredSignature._DICT_VALUE_FROM_ARGUMENT_TYPES
-    ],
+    [(sym, dict, dict[Any, int]) for sym in _DICT_VALUE_FROM_ARGUMENT_TYPES],
 )
-def test_guess_generic_types_dict_value_from_arguments(
-    inferred_signature, symbol, typ, result
-):
+def test_guess_generic_types_dict_value_from_arguments(inferred_signature, symbol, typ, result):
     config.configuration.test_creation.negate_type = 0.0
     knowledge = UsageTraceNode("ROOT")
     knowledge.children[symbol].arg_types[1].add(int)
     with mock.patch("pynguin.utils.randomness.choice") as choice_mock:
-        choice_mock.side_effect = lambda x: x[0]
+        choice_mock.side_effect = operator.itemgetter(0)
         assert inferred_signature._guess_generic_type_parameters_for_builtins(
             inferred_signature.type_system.convert_type_hint(typ), knowledge, 0
         ) == inferred_signature.type_system.convert_type_hint(result)
@@ -715,23 +684,15 @@ def test_guess_generic_types_dict_value_from_arguments(
 
 @pytest.mark.parametrize(
     "symbol, typ, result",
-    [
-        (sym, list, list[int])
-        for sym in InferredSignature._LIST_ELEMENT_FROM_ARGUMENT_TYPES
-    ]
-    + [
-        (sym, set, set[int])
-        for sym in InferredSignature._SET_ELEMENT_FROM_ARGUMENT_TYPES
-    ],
+    [(sym, list, list[int]) for sym in _LIST_ELEMENT_FROM_ARGUMENT_TYPES]
+    + [(sym, set, set[int]) for sym in _SET_ELEMENT_FROM_ARGUMENT_TYPES],
 )
-def test_guess_generic_types_list_set_from_arguments(
-    inferred_signature, symbol, typ, result
-):
+def test_guess_generic_types_list_set_from_arguments(inferred_signature, symbol, typ, result):
     config.configuration.test_creation.negate_type = 0.0
     knowledge = UsageTraceNode("ROOT")
     knowledge.children[symbol].arg_types[0].add(int)
     with mock.patch("pynguin.utils.randomness.choice") as choice_mock:
-        choice_mock.side_effect = lambda x: x[0]
+        choice_mock.side_effect = operator.itemgetter(0)
         assert inferred_signature._guess_generic_type_parameters_for_builtins(
             inferred_signature.type_system.convert_type_hint(typ), knowledge, 0
         ) == inferred_signature.type_system.convert_type_hint(result)
@@ -764,9 +725,7 @@ def test_choose_type_or_negate_negate(inferred_signature):
 
 def test_choose_type_or_negate_empty_2(inferred_signature):
     config.configuration.test_creation.negate_type = 1.0
-    with mock.patch.object(
-        inferred_signature.type_system, "get_type_outside_of"
-    ) as outside_mock:
+    with mock.patch.object(inferred_signature.type_system, "get_type_outside_of") as outside_mock:
         outside_mock.return_value = OrderedSet()
         assert inferred_signature._choose_type_or_negate(
             OrderedSet((inferred_signature.type_system.to_type_info(object),))
@@ -779,21 +738,15 @@ def test_update_guess(inferred_signature):
 
 
 def test_update_guess_single(inferred_signature):
-    inferred_signature._update_guess(
-        "x", inferred_signature.type_system.convert_type_hint(int)
-    )
+    inferred_signature._update_guess("x", inferred_signature.type_system.convert_type_hint(int))
     assert inferred_signature.current_guessed_parameters["x"] == [
         inferred_signature.type_system.convert_type_hint(int)
     ]
 
 
 def test_update_guess_multi(inferred_signature):
-    inferred_signature._update_guess(
-        "x", inferred_signature.type_system.convert_type_hint(int)
-    )
-    inferred_signature._update_guess(
-        "x", inferred_signature.type_system.convert_type_hint(int)
-    )
+    inferred_signature._update_guess("x", inferred_signature.type_system.convert_type_hint(int))
+    inferred_signature._update_guess("x", inferred_signature.type_system.convert_type_hint(int))
     assert inferred_signature.current_guessed_parameters["x"] == [
         inferred_signature.type_system.convert_type_hint(int)
     ]
@@ -801,24 +754,12 @@ def test_update_guess_multi(inferred_signature):
 
 def test_update_guess_multi_drop(inferred_signature):
     config.configuration.test_creation.type_tracing_kept_guesses = 5
-    inferred_signature._update_guess(
-        "x", inferred_signature.type_system.convert_type_hint(int)
-    )
-    inferred_signature._update_guess(
-        "x", inferred_signature.type_system.convert_type_hint(float)
-    )
-    inferred_signature._update_guess(
-        "x", inferred_signature.type_system.convert_type_hint(str)
-    )
-    inferred_signature._update_guess(
-        "x", inferred_signature.type_system.convert_type_hint(bytes)
-    )
-    inferred_signature._update_guess(
-        "x", inferred_signature.type_system.convert_type_hint(bool)
-    )
-    inferred_signature._update_guess(
-        "x", inferred_signature.type_system.convert_type_hint(complex)
-    )
+    inferred_signature._update_guess("x", inferred_signature.type_system.convert_type_hint(int))
+    inferred_signature._update_guess("x", inferred_signature.type_system.convert_type_hint(float))
+    inferred_signature._update_guess("x", inferred_signature.type_system.convert_type_hint(str))
+    inferred_signature._update_guess("x", inferred_signature.type_system.convert_type_hint(bytes))
+    inferred_signature._update_guess("x", inferred_signature.type_system.convert_type_hint(bool))
+    inferred_signature._update_guess("x", inferred_signature.type_system.convert_type_hint(complex))
     assert inferred_signature.current_guessed_parameters["x"] == [
         inferred_signature.type_system.convert_type_hint(tp)
         for tp in [float, str, bytes, bool, complex]
@@ -840,9 +781,7 @@ def test__guess_parameter_type(inferred_signature, symbol, kind):
         guess.assert_called_with(knowledge.children[symbol])
 
 
-@pytest.mark.parametrize(
-    "kind", [inspect.Parameter.VAR_KEYWORD, inspect.Parameter.VAR_POSITIONAL]
-)
+@pytest.mark.parametrize("kind", [inspect.Parameter.VAR_KEYWORD, inspect.Parameter.VAR_POSITIONAL])
 def test__guess_parameter_type_2(inferred_signature, kind):
     knowledge = UsageTraceNode("ROOT")
     assert inferred_signature._guess_parameter_type(knowledge, kind) is None
@@ -951,3 +890,53 @@ def test_no_partial_type_match(type_system, left, right):
         type_system.convert_type_hint(left), type_system.convert_type_hint(right)
     )
     assert match is None
+
+
+def test_to_type_info_union_type(subtyping_cluster):
+    type_system = subtyping_cluster.type_system
+    type_system.to_type_info(float | int)
+
+
+def test__guess_parameter_type_with_type_knowledge_simple(inferred_signature):
+    config.configuration.test_creation.negate_type = 0
+    knowledge = UsageTraceNode("ROOT")
+    kind = ""  # not inspect.Parameter.VAR_KEYWORD or inspect.Parameter.VAR_POSITIONAL
+    knowledge.type_checks.add(float)
+    expected = Instance(TypeInfo(float))
+    actual = inferred_signature._guess_parameter_type(knowledge, kind)
+    assert actual == expected
+
+
+def pick_0_generator():
+    while True:
+        yield 0
+
+
+def pick_1_generator():
+    while True:
+        yield 0
+        yield 1
+        yield 0
+
+
+pick_1 = pick_1_generator()
+pick_0 = pick_0_generator()
+
+
+@pytest.mark.parametrize(
+    "pick, expected_type",
+    [
+        (pick_0, Instance(TypeInfo(float))),
+        (pick_1, Instance(TypeInfo(int))),
+    ],
+)
+def test__guess_parameter_type_with_type_knowledge(inferred_signature, pick, expected_type):
+    config.configuration.test_creation.negate_type = 0
+    knowledge = UsageTraceNode("ROOT")
+    kind = ""  # not inspect.Parameter.VAR_KEYWORD or inspect.Parameter.VAR_POSITIONAL
+    knowledge.type_checks.add(float | int)
+
+    with mock.patch("pynguin.utils.randomness.choice") as choice_mock:
+        choice_mock.side_effect = lambda x: x[next(pick)]  # noqa: FURB118
+        actual = inferred_signature._guess_parameter_type(knowledge, kind)
+        assert actual == expected_type
