@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
-from typing import Any
+import ast
+
 from typing import cast
 from unittest.mock import MagicMock
 
@@ -21,12 +22,10 @@ class FooReferenceAssertion(ass.ReferenceAssertion):
     def accept(self, visitor: AssertionVisitor) -> None:
         pass  # pragma: no cover
 
-    def clone(
-        self, memo: dict[vr.VariableReference, vr.VariableReference]
-    ) -> Assertion:
+    def clone(self, memo: dict[vr.VariableReference, vr.VariableReference]) -> Assertion:
         pass  # pragma: no cover
 
-    def __eq__(self, other: Any) -> bool:
+    def __eq__(self, other: object) -> bool:
         pass  # pragma: no cover
 
     def __hash__(self) -> int:
@@ -39,12 +38,24 @@ def test_reference_assertion_source():
     assert foo.source == ref
 
 
+def test_reference_assertion_source_setter():
+    ref1 = MagicMock()
+    ref2 = MagicMock()
+    foo = FooReferenceAssertion(ref1)
+    foo.source = ref2
+    assert foo.source == ref2
+
+
 @pytest.mark.parametrize(
     "assertion,method",
     [
         (ass.TypeNameAssertion(MagicMock(), "", ""), "visit_type_name_assertion"),
         (ass.FloatAssertion(MagicMock(), 3.7), "visit_float_assertion"),
         (ass.ObjectAssertion(MagicMock(), [1]), "visit_object_assertion"),
+        (
+            ass.IsInstanceAssertion(MagicMock(), ast.Name(id="int", ctx=ast.Load())),
+            "visit_isinstance_assertion",
+        ),
         (
             ass.CollectionLengthAssertion(MagicMock(), 5),
             "visit_collection_length_assertion",
@@ -64,18 +75,19 @@ def test_assertion_accept(assertion, method):
 @pytest.mark.parametrize(
     "assertion",
     [
-        (
-            ass.TypeNameAssertion(
-                vr.VariableReference(MagicMock(), int), "builtins", "int"
-            )
-        ),
+        (ass.TypeNameAssertion(vr.VariableReference(MagicMock(), int), "builtins", "int")),
         (ass.FloatAssertion(vr.VariableReference(MagicMock(), int), 3.7)),
         (ass.ObjectAssertion(vr.VariableReference(MagicMock(), int), [1])),
+        (
+            ass.IsInstanceAssertion(
+                vr.VariableReference(MagicMock(), int), ast.Name(id="int", ctx=ast.Load())
+            )
+        ),
         (ass.CollectionLengthAssertion(vr.VariableReference(MagicMock(), int), 5)),
     ],
 )
 def test_assertion_clone(assertion):
-    source = cast(vr.VariableReference, assertion.source)
+    source = cast("vr.VariableReference", assertion.source)
     cloned = assertion.clone({source: source})
     assert cloned == assertion
     assert hash(cloned) == hash(assertion)
@@ -106,6 +118,14 @@ def test_assertion_clone(assertion):
         ),
         (ass.ObjectAssertion(0, [1]), ass.ObjectAssertion(1, [1])),
         (ass.ObjectAssertion(0, [1]), ass.ObjectAssertion(1, [2])),
+        (
+            ass.IsInstanceAssertion(0, ast.Name(id="int", ctx=ast.Load())),
+            ass.IsInstanceAssertion(1, ast.Name(id="int", ctx=ast.Load())),
+        ),
+        (
+            ass.IsInstanceAssertion(0, ast.Name(id="int", ctx=ast.Load())),
+            ass.IsInstanceAssertion(0, ast.Name(id="str", ctx=ast.Load())),
+        ),
         (
             ass.CollectionLengthAssertion(0, 5),
             ass.CollectionLengthAssertion(1, 5),
@@ -117,7 +137,7 @@ def test_assertion_clone(assertion):
     ],
 )
 def test_assertion_eq(assertion, different):
-    assert assertion == assertion
+    assert assertion == assertion  # noqa: PLR0124
     assert assertion != different
 
 
@@ -140,3 +160,9 @@ def test_exception_assertion():
     assertion = ass.ExceptionAssertion("builtin", "foo")
     assert assertion.module == "builtin"
     assert assertion.exception_type_name == "foo"
+
+
+def test_isinstance_assertion_expected_type():
+    expected_type = ast.Name(id="int", ctx=ast.Load())
+    assertion = ass.IsInstanceAssertion(MagicMock(), expected_type)
+    assert assertion.expected_type == expected_type
