@@ -1,16 +1,20 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
+# ruff: noqa: E501
 import ast
 import importlib
+import inspect
 import threading
 
 import pytest
 
 import pynguin.assertion.assertiongenerator as ag
+import pynguin.assertion.mutation_analysis.mutators as mu
+import pynguin.assertion.mutation_analysis.operators as mo
 import pynguin.configuration as config
 import pynguin.ga.testcasechromosome as tcc
 import pynguin.ga.testsuitechromosome as tsc
@@ -20,8 +24,9 @@ import pynguin.utils.namingscope as ns
 from pynguin.analyses.constants import EmptyConstantProvider
 from pynguin.analyses.module import generate_test_cluster
 from pynguin.analyses.seeding import AstToTestCaseTransformer
+from pynguin.assertion.mutation_analysis.transformer import ParentNodeTransformer
 from pynguin.instrumentation.machinery import install_import_hook
-from pynguin.testcase.execution import ExecutionTracer
+from pynguin.instrumentation.tracer import SubjectProperties
 from pynguin.testcase.execution import TestCaseExecutor
 
 
@@ -44,19 +49,25 @@ assert str_1 == 'foo bar'""",
 float_0 = 39.82
 human_0 = module_0.Human(str_0, float_0)
 assert module_0.static_state == 0
-str_1 = human_0.get_name()""",
+human_0.get_name()""",
         ),
     ],
 )
-def test_generate_mutation_assertions(generator, expected_result):
+def test_generate_mutation_assertions(
+    generator, expected_result, subject_properties: SubjectProperties
+):
     config.configuration.module_name = "tests.fixtures.examples.assertions"
     module_name = config.configuration.module_name
-    tracer = ExecutionTracer()
-    tracer.current_thread_identifier = threading.current_thread().ident
-    with install_import_hook(module_name, tracer):
-        importlib.reload(importlib.import_module(module_name))
+    with install_import_hook(module_name, subject_properties):
+        with subject_properties.instrumentation_tracer:
+            module = importlib.import_module(module_name)
+            importlib.reload(module)
         cluster = generate_test_cluster(module_name)
-        transformer = AstToTestCaseTransformer(cluster, False, EmptyConstantProvider())
+        transformer = AstToTestCaseTransformer(
+            cluster,
+            False,  # noqa: FBT003
+            EmptyConstantProvider(),
+        )
         transformer.visit(
             ast.parse(
                 """def test_case_0():
@@ -73,15 +84,29 @@ def test_generate_mutation_assertions(generator, expected_result):
         suite = tsc.TestSuiteChromosome()
         suite.add_test_case_chromosome(chromosome)
 
-        gen = generator(TestCaseExecutor(tracer))
+        if generator is ag.MutationAnalysisAssertionGenerator:
+            mutant_generator = mu.FirstOrderMutator([
+                *mo.standard_operators,
+                *mo.experimental_operators,
+            ])
+            module_source_code = inspect.getsource(module)
+            module_ast = ParentNodeTransformer.create_ast(module_source_code)
+            mutation_subject_properties = SubjectProperties()
+            mutation_controller = ag.InstrumentedMutationController(
+                mutant_generator,
+                module_ast,
+                module,
+                mutation_subject_properties,
+            )
+            gen = generator(TestCaseExecutor(subject_properties), mutation_controller)
+        else:
+            gen = generator(TestCaseExecutor(subject_properties))
         suite.accept(gen)
 
         visitor = tc_to_ast.TestCaseToAstVisitor(ns.NamingScope(prefix="module"), set())
         test_case.accept(visitor)
         source = ast.unparse(
-            ast.fix_missing_locations(
-                ast.Module(body=visitor.test_case_ast, type_ignores=[])
-            )
+            ast.fix_missing_locations(ast.Module(body=visitor.test_case_ast, type_ignores=[]))
         )
         assert source == expected_result
 
@@ -144,7 +169,8 @@ _MUTANTS = [
             "tests.fixtures.mutation.mutation",
             "def test_case_0():\n    int_0 = 0\n    int_1 = 0\n    int_2 = 0\n"
             "    int_3 = 1\n    float_0 = module_0.foo(int_3)",
-            "int_0 = 0\nint_1 = 0\nint_2 = 0\nint_3 = 1\nfloat_0 = module_0.foo(int_3)\n"
+            "int_0 = 0\nint_1 = 0\nint_2 = 0\nint_3 = 1\nfloat_0 = "
+            "module_0.foo(int_3)\n"
             "assert float_0 == pytest.approx(2.0, abs=0.01, rel=0.01)",
             _MUTANTS,
             ag._MutationMetrics(5, 4, 0),
@@ -155,7 +181,7 @@ _MUTANTS = [
             "tests.fixtures.mutation.mutation",
             "def test_case_0():\n    int_0 = 0\n    int_1 = 0\n    int_2 = 0\n"
             "    int_3 = 0\n    float_0 = module_0.foo(int_3)",
-            "int_0 = 0\nint_1 = 0\nint_2 = 0\nint_3 = 0\nfloat_0 = module_0.foo(int_3)",
+            "int_0 = 0\nint_1 = 0\nint_2 = 0\nint_3 = 0\nmodule_0.foo(int_3)",
             _MUTANTS,
             ag._MutationMetrics(5, 0, 0),
             set(),
@@ -164,7 +190,7 @@ _MUTANTS = [
         (
             "tests.fixtures.mutation.exception",
             "def test_case_0():\n    float_0 = module_0.foo()",
-            "none_type_0 = module_0.foo()",
+            "module_0.foo()",
             [
                 "def foo() -> None:\n"
                 "    alist = [1, 2]\n"
@@ -251,7 +277,7 @@ _MUTANTS = [
         ),
     ],
 )
-def test_mutation_analysis_integration_full(
+def test_mutation_analysis_integration_full(  # noqa: PLR0914, PLR0917
     module,
     test_case_str,
     test_case_str_with_assertions,
@@ -259,15 +285,20 @@ def test_mutation_analysis_integration_full(
     metrics,
     killed,
     timeout,
+    subject_properties: SubjectProperties,
 ):
     config.configuration.module_name = module
     module_name = config.configuration.module_name
-    tracer = ExecutionTracer()
-    tracer.current_thread_identifier = threading.current_thread().ident
-    with install_import_hook(module_name, tracer):
-        importlib.reload(importlib.import_module(module_name))
+    with install_import_hook(module_name, subject_properties):
+        with subject_properties.instrumentation_tracer:
+            module_type = importlib.import_module(module_name)
+            importlib.reload(module_type)
         cluster = generate_test_cluster(module_name)
-        transformer = AstToTestCaseTransformer(cluster, False, EmptyConstantProvider())
+        transformer = AstToTestCaseTransformer(
+            cluster,
+            False,  # noqa: FBT003
+            EmptyConstantProvider(),
+        )
         transformer.visit(ast.parse(test_case_str))
         test_case = transformer.testcases[0]
 
@@ -275,8 +306,22 @@ def test_mutation_analysis_integration_full(
         suite = tsc.TestSuiteChromosome()
         suite.add_test_case_chromosome(chromosome)
 
+        mutant_generator = mu.FirstOrderMutator([
+            *mo.standard_operators,
+            *mo.experimental_operators,
+        ])
+        module_source_code = inspect.getsource(module_type)
+        module_ast = ParentNodeTransformer.create_ast(module_source_code)
+        mutation_subject_properties = SubjectProperties()
+        mutation_controller = ag.InstrumentedMutationController(
+            mutant_generator,
+            module_ast,
+            module_type,
+            mutation_subject_properties,
+            testing=True,
+        )
         gen = ag.MutationAnalysisAssertionGenerator(
-            TestCaseExecutor(tracer), testing=True
+            TestCaseExecutor(subject_properties), mutation_controller, testing=True
         )
         suite.accept(gen)
 
@@ -287,18 +332,14 @@ def test_mutation_analysis_integration_full(
         assert kills == killed
         assert timeouts == timeout
         # Test for disjoint
-        assert len(kills | survived | timeouts) == len(kills) + len(timeouts) + len(
-            survived
-        )
+        assert len(kills | survived | timeouts) == len(kills) + len(timeouts) + len(survived)
 
         assert summary.get_metrics() == metrics
-        assert gen._testing_created_mutants == mutants
+        assert mutation_controller._testing_created_mutants == mutants
         visitor = tc_to_ast.TestCaseToAstVisitor(ns.NamingScope(prefix="module"), set())
         test_case.accept(visitor)
         source = ast.unparse(
-            ast.fix_missing_locations(
-                ast.Module(body=visitor.test_case_ast, type_ignores=[])
-            )
+            ast.fix_missing_locations(ast.Module(body=visitor.test_case_ast, type_ignores=[]))
         )
         assert source == test_case_str_with_assertions
         for thread in threading.enumerate():
