@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -20,9 +20,7 @@ from pynguin.utils.namingscope import NamingScope
 
 
 @pytest.fixture
-def assertion_to_ast_ref() -> tuple[
-    ata.PyTestAssertionToAstVisitor, vr.VariableReference
-]:
+def assertion_to_ast_ref() -> tuple[ata.PyTestAssertionToAstVisitor, vr.VariableReference]:
     scope = NamingScope()
     module_aliases = NamingScope(prefix="module")
     var = vr.VariableReference(MagicMock(), None)
@@ -40,9 +38,7 @@ def assertion_to_ast_ref() -> tuple[
 
 
 def __create_source_from_ast(module_body: list[ast.stmt]) -> str:
-    return ast.unparse(
-        ast.fix_missing_locations(ast.Module(body=module_body, type_ignores=[]))
-    )
+    return ast.unparse(ast.fix_missing_locations(ast.Module(body=module_body, type_ignores=[])))
 
 
 def test_type_name(assertion_to_ast_ref):
@@ -51,7 +47,8 @@ def test_type_name(assertion_to_ast_ref):
     assertion.accept(assertion_to_ast)
     assert (
         __create_source_from_ast(assertion_to_ast.nodes)
-        == "var_0 = 5\nassert f'{type(var_0).__module__}.{type(var_0).__qualname__}' == 'foo.bar'"
+        == "var_0 = 5\nassert f'{type(var_0).__module__}.{type(var_0).__qualname__}' "
+        "== 'foo.bar'"
     )
 
 
@@ -106,12 +103,21 @@ def test_collection_length(assertion_to_ast_ref, length, output):
 
 
 def test_raises_exception(assertion_to_ast_ref):
-    assertion_to_ast, ref = assertion_to_ast_ref
-    assertion = ass.ExceptionAssertion(
-        module="builtins", exception_type_name="AssertionError"
-    )
+    assertion_to_ast, _ref = assertion_to_ast_ref
+    assertion = ass.ExceptionAssertion(module="builtins", exception_type_name="AssertionError")
     assertion.accept(assertion_to_ast)
     assert (
         __create_source_from_ast(assertion_to_ast.nodes)
         == "with pytest.raises(AssertionError):\n    var_0 = 5"
     )
+
+
+def test_isinstance_assertion(assertion_to_ast_ref):
+    assertion_to_ast, ref = assertion_to_ast_ref
+    expected_type = ast.Name(id="int", ctx=ast.Load())
+    assertion = ass.IsInstanceAssertion(source=ref, expected_type=expected_type)
+    assertion.accept(assertion_to_ast)
+    assert (
+        __create_source_from_ast(assertion_to_ast.nodes)
+        == "var_0 = 5\nassert isinstance(var_0, int)"
+    )
