@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -9,9 +9,7 @@ import pytest
 import pynguin.assertion.assertiongenerator as ag
 
 
-@pytest.mark.parametrize(
-    "created,killed,timeout,score", [(5, 2, 1, 0.5), (1, 0, 1, 1.0)]
-)
+@pytest.mark.parametrize("created,killed,timeout,score", [(5, 2, 1, 0.5), (1, 0, 1, 1.0)])
 def test_mutation_score(created, killed, timeout, score):
     metrics = ag._MutationMetrics(created, killed, timeout)
     assert metrics.get_score() == score
