@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -29,7 +29,7 @@ class DependencyPair:
     recursion_level: int = dataclasses.field(compare=False, hash=False)
 
 
-class TestClusterGenerator:  # pylint: disable=too-few-public-methods
+class TestClusterGenerator:
     """Generate a new test cluster"""
 
     _logger = logging.getLogger(__name__)
