@@ -1,13 +1,13 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 from unittest import mock
 from unittest.mock import MagicMock
 
-import pynguin.ga.operators.crossover.singlepointrelativecrossover as cross
+import pynguin.ga.operators.crossover as cross
 import pynguin.ga.testsuitechromosome as tsc
 
 
