@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -11,9 +11,9 @@ import pytest
 import pynguin.configuration as config
 import pynguin.ga.chromosome as chrom
 
-from pynguin.ga.operators.ranking.rankingfunction import RankBasedPreferenceSorting
-from pynguin.ga.operators.ranking.rankingfunction import RankedFronts
-from pynguin.ga.operators.ranking.rankingfunction import RankingFunction
+from pynguin.ga.operators.ranking import RankBasedPreferenceSorting
+from pynguin.ga.operators.ranking import RankedFronts
+from pynguin.ga.operators.ranking import RankingFunction
 
 
 @pytest.fixture
