@@ -1,15 +1,15 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 from unittest.mock import MagicMock
 
-import pynguin.ga.operators.selection.selection as sel
+import pynguin.ga.operators.selection as sel
 import pynguin.ga.testsuitechromosome as tsc
 
-from pynguin.ga.operators.selection.selection import T
+from pynguin.ga.operators.selection import T
 from pynguin.utils import randomness
 
 
