@@ -1,12 +1,12 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 from unittest.mock import MagicMock
 
 import pynguin.ga.chromosome as chrom
-import pynguin.ga.operators.selection.tournamentselection as sel
+import pynguin.ga.operators.selection as sel
 
 from pynguin.utils import randomness
 
