@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -26,9 +26,7 @@ def chromosome():
         def clone(self) -> Chromosome:
             pass  # pragma: no cover
 
-        def cross_over(
-            self, other: chrom.Chromosome, position1: int, position2: int
-        ) -> None:
+        def cross_over(self, other: chrom.Chromosome, position1: int, position2: int) -> None:
             pass  # pragma: no cover
 
         def __hash__(self):
