@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -91,7 +91,7 @@ def test_mutation_insert_none(test_case_chromosome):
 def test_mutation_insert_two(default_test_case):
     test_factory = MagicMock(tf.TestFactory)
 
-    def side_effect(tc, pos):
+    def side_effect(tc, _):
         tc.add_statement(IntPrimitiveStatement(tc, 5))
         return 0
 
@@ -102,15 +102,16 @@ def test_mutation_insert_two(default_test_case):
     with mock.patch("pynguin.utils.randomness.next_float") as float_mock:
         float_mock.side_effect = [0.2, 0.2, 0.2]
         assert chromosome._mutation_insert()
-    test_factory.insert_random_statement.assert_has_calls(
-        [call(default_test_case, 0), call(default_test_case, 1)]
-    )
+    test_factory.insert_random_statement.assert_has_calls([
+        call(default_test_case, 0),
+        call(default_test_case, 1),
+    ])
 
 
 def test_mutation_insert_twice_no_success(default_test_case):
     test_factory = MagicMock(tf.TestFactory)
 
-    def side_effect(tc, pos):
+    def side_effect(_tc, _pos):
         return -1
 
     test_factory.insert_random_statement.side_effect = side_effect
@@ -120,15 +121,16 @@ def test_mutation_insert_twice_no_success(default_test_case):
     with mock.patch("pynguin.utils.randomness.next_float") as float_mock:
         float_mock.side_effect = [0.2, 0.2, 0.2]
         assert not chromosome._mutation_insert()
-    test_factory.insert_random_statement.assert_has_calls(
-        [call(default_test_case, 0), call(default_test_case, 0)]
-    )
+    test_factory.insert_random_statement.assert_has_calls([
+        call(default_test_case, 0),
+        call(default_test_case, 0),
+    ])
 
 
 def test_mutation_insert_max_length(default_test_case):
     test_factory = MagicMock(tf.TestFactory)
 
-    def side_effect(tc, pos):
+    def side_effect(tc, _):
         tc.add_statement(IntPrimitiveStatement(tc, 5))
         return 0
 
@@ -225,7 +227,7 @@ def test_mutation_delete_skipping(default_test_case):
 def test_mutate_chop(test_case_chromosome_with_test):
     chromosome, test_case = test_case_chromosome_with_test
     chromosome.changed = False
-    for i in range(50):
+    for _i in range(50):
         test_case.add_statement(IntPrimitiveStatement(test_case, 5))
     config.configuration.search_algorithm.test_insert_probability = 0.0
     config.configuration.search_algorithm.test_change_probability = 0.0
@@ -242,7 +244,7 @@ def test_mutate_chop(test_case_chromosome_with_test):
 
 def test_mutate_no_chop(test_case_chromosome_with_test):
     chromosome, test_case = test_case_chromosome_with_test
-    for i in range(50):
+    for _i in range(50):
         test_case.add_statement(IntPrimitiveStatement(test_case, 5))
     chromosome.changed = False
     config.configuration.search_algorithm.test_insert_probability = 0.0
@@ -275,9 +277,7 @@ def test_mutate_all(test_case_chromosome, func, rand, result):
         float_mock.side_effect = rand
         with mock.patch.object(test_case_chromosome, func) as mock_func:
             mock_func.return_value = result
-            with mock.patch.object(
-                test_case_chromosome, "_test_factory"
-            ) as factory_mock:
+            with mock.patch.object(test_case_chromosome, "_test_factory") as factory_mock:
                 factory_mock.has_call_on_sut.return_value = True
                 test_case_chromosome.mutate()
                 assert test_case_chromosome.changed == result
