@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -24,7 +24,7 @@ def test_get_test_case_max_attempts():
 def test_get_test_case_success():
     test_factory = MagicMock(tf.TestFactory)
     test_factory.insert_random_statement.side_effect = (
-        lambda test_case, pos: test_case.add_statement(MagicMock(), 0)
+        lambda test_case, _pos: test_case.add_statement(MagicMock(), 0)
     )
     test_case_factory = tcf.RandomLengthTestCaseFactory(test_factory, MagicMock())
     test_case_factory.get_test_case()
