@@ -1,12 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 import ast
 import importlib
-import threading
 
 import pytest
 
@@ -21,10 +20,10 @@ from pynguin.analyses.module import generate_test_cluster
 from pynguin.analyses.seeding import AstToTestCaseTransformer
 from pynguin.ga.computations import TestSuiteAssertionCheckedCoverageFunction
 from pynguin.instrumentation.machinery import install_import_hook
+from pynguin.instrumentation.tracer import SubjectProperties
 from pynguin.slicer.dynamicslicer import AssertionSlicer
 from pynguin.slicer.dynamicslicer import DynamicSlicer
-from pynguin.testcase.execution import AssertionExecutionObserver
-from pynguin.testcase.execution import ExecutionTracer
+from pynguin.testcase.execution import RemoteAssertionExecutionObserver
 from pynguin.testcase.execution import TestCaseExecutor
 from pynguin.testcase.variablereference import FieldReference
 from pynguin.testcase.variablereference import StaticFieldReference
@@ -33,18 +32,22 @@ from tests.fixtures.linecoverage.plus import Plus
 
 @pytest.fixture
 def full_cover_plus_three_test():
-    """
-    Produces the following testcase:
-        def test_case_0():
-            int_0 = 3360
-            plus_0 = module_0.Plus()
-            assert plus_0.calculations == 0
-            var_0 = plus_0.plus_three(int_0)
-            assert var_0 == 3363
-            assert plus_0.calculations == 1
+    """Produces the following testcase.
+
+    def test_case_0():
+        int_0 = 3360
+        plus_0 = module_0.Plus()
+        assert plus_0.calculations == 0
+        var_0 = plus_0.plus_three(int_0)
+        assert var_0 == 3363
+        assert plus_0.calculations == 1.
     """
     cluster = generate_test_cluster("tests.fixtures.linecoverage.plus")
-    transformer = AstToTestCaseTransformer(cluster, False, EmptyConstantProvider())
+    transformer = AstToTestCaseTransformer(
+        cluster,
+        False,  # noqa: FBT003
+        EmptyConstantProvider(),
+    )
     transformer.visit(
         ast.parse(
             """def test_case_0():
@@ -89,19 +92,22 @@ def full_cover_plus_three_test():
 
 @pytest.fixture
 def full_cover_plus_four_test():
-    """
-    Produces the following testcase:
-        def test_case_1():
-            int_0 = -3559
-            plus_0 = module_0.Plus()
-            assert plus_0.calculations == 0
-            var_0 = plus_0.plus_four(int_0)
-            assert var_0 == -3555
-            assert plus_0.calculations == 1
-    """
+    """Produces the following testcase.
 
+    def test_case_1():
+        int_0 = -3559
+        plus_0 = module_0.Plus()
+        assert plus_0.calculations == 0
+        var_0 = plus_0.plus_four(int_0)
+        assert var_0 == -3555
+        assert plus_0.calculations == 1.
+    """
     cluster = generate_test_cluster("tests.fixtures.linecoverage.plus")
-    transformer = AstToTestCaseTransformer(cluster, False, EmptyConstantProvider())
+    transformer = AstToTestCaseTransformer(
+        cluster,
+        False,  # noqa: FBT003
+        EmptyConstantProvider(),
+    )
     transformer.visit(
         ast.parse(
             """def test_case_0():
@@ -147,7 +153,11 @@ def full_cover_plus_four_test():
 @pytest.fixture
 def partial_cover_use_bool_as_int():
     cluster = generate_test_cluster("tests.fixtures.linecoverage.plus")
-    transformer = AstToTestCaseTransformer(cluster, False, EmptyConstantProvider())
+    transformer = AstToTestCaseTransformer(
+        cluster,
+        False,  # noqa: FBT003
+        EmptyConstantProvider(),
+    )
     transformer.visit(
         ast.parse(
             """def test_case_0():
@@ -204,9 +214,7 @@ def test_case_1():
             0,
         )
     )
-    tc_1.statements[2].add_assertion(
-        ass.ObjectAssertion(tc_1.statements[2].ret_val, 1004)
-    )
+    tc_1.statements[2].add_assertion(ass.ObjectAssertion(tc_1.statements[2].ret_val, 1004))
 
     test_suite = tsc.TestSuiteChromosome()
     test_suite.add_test_case_chromosome(tcc.TestCaseChromosome(tc_0))
@@ -261,22 +269,22 @@ def no_cover_plus_testsuite(default_test_case) -> tsc.TestSuiteChromosome:
     ],
 )
 def test_assertion_detection_on_test_case(
-    module_name, test_case_name, expected_assertions, request
+    module_name,
+    test_case_name,
+    expected_assertions,
+    request,
+    subject_properties: SubjectProperties,
 ):
     test_case = request.getfixturevalue(test_case_name)
-    config.configuration.statistics_output.coverage_metrics = [
-        config.CoverageMetric.CHECKED
-    ]
+    config.configuration.statistics_output.coverage_metrics = [config.CoverageMetric.CHECKED]
 
-    tracer = ExecutionTracer()
-    tracer.current_thread_identifier = threading.current_thread().ident
+    with install_import_hook(module_name, subject_properties):
+        with subject_properties.instrumentation_tracer:
+            module = importlib.import_module(module_name)
+            importlib.reload(module)
 
-    with install_import_hook(module_name, tracer):
-        module = importlib.import_module(module_name)
-        importlib.reload(module)
-
-        executor = TestCaseExecutor(tracer)
-        executor.add_observer(AssertionExecutionObserver(tracer))
+        executor = TestCaseExecutor(subject_properties)
+        executor.add_remote_observer(RemoteAssertionExecutionObserver())
         result = executor.execute(test_case)
         assert result.execution_trace.executed_assertions
         assert len(result.execution_trace.executed_assertions) == expected_assertions
@@ -303,29 +311,23 @@ def test_assertion_detection_on_test_case(
     ],
 )
 def test_slicing_after_test_execution(
-    module_name, test_case_name, expected_lines, request
+    module_name, test_case_name, expected_lines, request, subject_properties: SubjectProperties
 ):
     test_case = request.getfixturevalue(test_case_name)
-    config.configuration.statistics_output.coverage_metrics = [
-        config.CoverageMetric.CHECKED
-    ]
+    config.configuration.statistics_output.coverage_metrics = [config.CoverageMetric.CHECKED]
 
-    tracer = ExecutionTracer()
-    tracer.current_thread_identifier = threading.current_thread().ident
+    with install_import_hook(module_name, subject_properties):
+        with subject_properties.instrumentation_tracer:
+            module = importlib.import_module(module_name)
+            importlib.reload(module)
 
-    with install_import_hook(module_name, tracer):
-        module = importlib.import_module(module_name)
-        importlib.reload(module)
-
-        executor = TestCaseExecutor(tracer)
-        executor.add_observer(AssertionExecutionObserver(tracer))
+        executor = TestCaseExecutor(subject_properties)
+        executor.add_remote_observer(RemoteAssertionExecutionObserver())
         result = executor.execute(test_case)
         assert result.execution_trace.executed_assertions
 
         instructions_in_slice = []
-        assertion_slicer = AssertionSlicer(
-            tracer.get_subject_properties().existing_code_objects
-        )
+        assertion_slicer = AssertionSlicer(subject_properties.existing_code_objects)
         for assertion in result.execution_trace.executed_assertions:
             instructions_in_slice.extend(
                 assertion_slicer.slice_assertion(assertion, result.execution_trace)
@@ -333,7 +335,7 @@ def test_slicing_after_test_execution(
         assert instructions_in_slice
 
         checked_lines = DynamicSlicer.map_instructions_to_lines(
-            instructions_in_slice, tracer.get_subject_properties()
+            instructions_in_slice, subject_properties
         )
         assert checked_lines
         assert checked_lines == expected_lines
@@ -367,23 +369,23 @@ def test_slicing_after_test_execution(
     ],
 )
 def test_testsuite_assertion_checked_coverage_calculation(
-    module_name, test_suite_name, expected_coverage, request
+    module_name,
+    test_suite_name,
+    expected_coverage,
+    request,
+    subject_properties: SubjectProperties,
 ):
     test_suite = request.getfixturevalue(test_suite_name)
     config.configuration.statistics_output.coverage_metrics = [
         config.CoverageMetric.CHECKED,
     ]
 
-    tracer = ExecutionTracer()
-    tracer.current_thread_identifier = threading.current_thread().ident
+    with install_import_hook(module_name, subject_properties):
+        with subject_properties.instrumentation_tracer:
+            module = importlib.import_module(module_name)
+            importlib.reload(module)
 
-    with install_import_hook(module_name, tracer):
-        module = importlib.import_module(module_name)
-        importlib.reload(module)
-
-        executor = TestCaseExecutor(tracer)
-        executor.add_observer(AssertionExecutionObserver(tracer))
+        executor = TestCaseExecutor(subject_properties)
+        executor.add_remote_observer(RemoteAssertionExecutionObserver())
         ff = TestSuiteAssertionCheckedCoverageFunction(executor)
-        assert ff.compute_coverage(test_suite) == pytest.approx(
-            expected_coverage, 0.1, 0.1
-        )
+        assert ff.compute_coverage(test_suite) == pytest.approx(expected_coverage, 0.1, 0.1)
