@@ -1,13 +1,12 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 import ast
 import importlib
 import inspect
-import threading
 
 from unittest.mock import MagicMock
 
@@ -24,9 +23,9 @@ from pynguin.analyses.typesystem import InferredSignature
 from pynguin.ga.computations import TestCaseStatementCheckedCoverageFunction
 from pynguin.ga.computations import TestSuiteStatementCheckedCoverageFunction
 from pynguin.instrumentation.machinery import install_import_hook
+from pynguin.instrumentation.tracer import SubjectProperties
 from pynguin.slicer.dynamicslicer import DynamicSlicer
-from pynguin.slicer.statementslicingobserver import StatementSlicingObserver
-from pynguin.testcase.execution import ExecutionTracer
+from pynguin.slicer.statementslicingobserver import RemoteStatementSlicingObserver
 from pynguin.testcase.execution import TestCaseExecutor
 from pynguin.testcase.statement import MethodStatement
 from pynguin.utils.generic.genericaccessibleobject import GenericMethod
@@ -36,7 +35,11 @@ from tests.fixtures.linecoverage.setter_getter import SetterGetter
 @pytest.fixture
 def plus_three_test():
     cluster = generate_test_cluster("tests.fixtures.linecoverage.plus")
-    transformer = AstToTestCaseTransformer(cluster, False, EmptyConstantProvider())
+    transformer = AstToTestCaseTransformer(
+        cluster,
+        False,  # noqa: FBT003
+        EmptyConstantProvider(),
+    )
     transformer.visit(
         ast.parse(
             """def test_case_0():
@@ -49,57 +52,57 @@ def plus_three_test():
     return transformer.testcases[0]
 
 
-def test_testsuite_statement_checked_coverage_calculation(plus_three_test):
+def test_testsuite_statement_checked_coverage_calculation(
+    plus_three_test, subject_properties: SubjectProperties
+):
     module_name = "tests.fixtures.linecoverage.plus"
     test_suite = tsc.TestSuiteChromosome()
-    test_suite.add_test_case_chromosome(
-        tcc.TestCaseChromosome(test_case=plus_three_test)
-    )
+    test_suite.add_test_case_chromosome(tcc.TestCaseChromosome(test_case=plus_three_test))
     config.configuration.statistics_output.coverage_metrics = [
         config.CoverageMetric.CHECKED,
     ]
 
-    tracer = ExecutionTracer()
-    tracer.current_thread_identifier = threading.current_thread().ident
+    with install_import_hook(module_name, subject_properties):
+        with subject_properties.instrumentation_tracer:
+            module = importlib.import_module(module_name)
+            importlib.reload(module)
 
-    with install_import_hook(module_name, tracer):
-        module = importlib.import_module(module_name)
-        importlib.reload(module)
-
-        executor = TestCaseExecutor(tracer)
-        executor.add_observer(StatementSlicingObserver(tracer))
+        executor = TestCaseExecutor(subject_properties)
+        executor.add_remote_observer(RemoteStatementSlicingObserver())
 
         ff = TestSuiteStatementCheckedCoverageFunction(executor)
         assert ff.compute_coverage(test_suite) == pytest.approx(4 / 8, 0.1, 0.1)
 
 
-def test_testcase_statement_checked_coverage_calculation(plus_three_test):
+def test_testcase_statement_checked_coverage_calculation(
+    plus_three_test, subject_properties: SubjectProperties
+):
     module_name = "tests.fixtures.linecoverage.plus"
     test_case_chromosome = tcc.TestCaseChromosome(test_case=plus_three_test)
     config.configuration.statistics_output.coverage_metrics = [
         config.CoverageMetric.CHECKED,
     ]
 
-    tracer = ExecutionTracer()
-    tracer.current_thread_identifier = threading.current_thread().ident
+    with install_import_hook(module_name, subject_properties):
+        with subject_properties.instrumentation_tracer:
+            module = importlib.import_module(module_name)
+            importlib.reload(module)
 
-    with install_import_hook(module_name, tracer):
-        module = importlib.import_module(module_name)
-        importlib.reload(module)
-
-        executor = TestCaseExecutor(tracer)
-        executor.add_observer(StatementSlicingObserver(tracer))
+        executor = TestCaseExecutor(subject_properties)
+        executor.add_remote_observer(RemoteStatementSlicingObserver())
 
         ff = TestCaseStatementCheckedCoverageFunction(executor)
-        assert ff.compute_coverage(test_case_chromosome) == pytest.approx(
-            4 / 8, 0.1, 0.1
-        )
+        assert ff.compute_coverage(test_case_chromosome) == pytest.approx(4 / 8, 0.1, 0.1)
 
 
 @pytest.fixture
 def setter_test():
     cluster = generate_test_cluster("tests.fixtures.linecoverage.setter_getter")
-    transformer = AstToTestCaseTransformer(cluster, False, EmptyConstantProvider())
+    transformer = AstToTestCaseTransformer(
+        cluster,
+        False,  # noqa: FBT003
+        EmptyConstantProvider(),
+    )
     transformer.visit(
         ast.parse(
             """def test_case_0():
@@ -135,35 +138,33 @@ def setter_test():
     return tc
 
 
-def test_only_void_function(setter_test):
-    """Test if implicit return nones are correctly filtered from the sliced
-    assignment to a none type for methods with none return type."""
+def test_only_void_function(setter_test, subject_properties: SubjectProperties):
     module_name = "tests.fixtures.linecoverage.setter_getter"
     test_case_chromosome = tcc.TestCaseChromosome(test_case=setter_test)
     config.configuration.statistics_output.coverage_metrics = [
         config.CoverageMetric.CHECKED,
     ]
 
-    tracer = ExecutionTracer()
-    tracer.current_thread_identifier = threading.current_thread().ident
+    with install_import_hook(module_name, subject_properties):
+        with subject_properties.instrumentation_tracer:
+            module = importlib.import_module(module_name)
+            importlib.reload(module)
 
-    with install_import_hook(module_name, tracer):
-        module = importlib.import_module(module_name)
-        importlib.reload(module)
-
-        executor = TestCaseExecutor(tracer)
-        executor.add_observer(StatementSlicingObserver(tracer))
+        executor = TestCaseExecutor(subject_properties)
+        executor.add_remote_observer(RemoteStatementSlicingObserver())
 
         ff = TestCaseStatementCheckedCoverageFunction(executor)
-        assert ff.compute_coverage(test_case_chromosome) == pytest.approx(
-            3 / 6, 0.1, 0.1
-        )
+        assert ff.compute_coverage(test_case_chromosome) == pytest.approx(3 / 6, 0.1, 0.1)
 
 
 @pytest.fixture
 def getter_setter_test():
     cluster = generate_test_cluster("tests.fixtures.linecoverage.setter_getter")
-    transformer = AstToTestCaseTransformer(cluster, False, EmptyConstantProvider())
+    transformer = AstToTestCaseTransformer(
+        cluster,
+        False,  # noqa: FBT003
+        EmptyConstantProvider(),
+    )
     transformer.visit(
         ast.parse(
             """def test_case_0():
@@ -200,36 +201,33 @@ def getter_setter_test():
     return tc
 
 
-def test_getter_before_setter(getter_setter_test):
-    """If the getter is before after the setter, the value retrieved by the getter
-    is not depending on the new set value. Therefore, the body of the setter should
-    npt be included in the slice."""
+def test_getter_before_setter(getter_setter_test, subject_properties: SubjectProperties):
     module_name = "tests.fixtures.linecoverage.setter_getter"
     test_case_chromosome = tcc.TestCaseChromosome(test_case=getter_setter_test)
     config.configuration.statistics_output.coverage_metrics = [
         config.CoverageMetric.CHECKED,
     ]
 
-    tracer = ExecutionTracer()
-    tracer.current_thread_identifier = threading.current_thread().ident
+    with install_import_hook(module_name, subject_properties):
+        with subject_properties.instrumentation_tracer:
+            module = importlib.import_module(module_name)
+            importlib.reload(module)
 
-    with install_import_hook(module_name, tracer):
-        module = importlib.import_module(module_name)
-        importlib.reload(module)
-
-        executor = TestCaseExecutor(tracer)
-        executor.add_observer(StatementSlicingObserver(tracer))
+        executor = TestCaseExecutor(subject_properties)
+        executor.add_remote_observer(RemoteStatementSlicingObserver())
 
         ff = TestCaseStatementCheckedCoverageFunction(executor)
-        assert ff.compute_coverage(test_case_chromosome) == pytest.approx(
-            5 / 6, 0.1, 0.1
-        )
+        assert ff.compute_coverage(test_case_chromosome) == pytest.approx(5 / 6, 0.1, 0.1)
 
 
 @pytest.fixture
 def setter_getter_test():
     cluster = generate_test_cluster("tests.fixtures.linecoverage.setter_getter")
-    transformer = AstToTestCaseTransformer(cluster, False, EmptyConstantProvider())
+    transformer = AstToTestCaseTransformer(
+        cluster,
+        False,  # noqa: FBT003
+        EmptyConstantProvider(),
+    )
     transformer.visit(
         ast.parse(
             """def test_case_0():
@@ -282,30 +280,23 @@ def setter_getter_test():
     return tc
 
 
-def test_getter_after_setter(setter_getter_test):
-    """If the getter is called after the setter, the value retrieved by the getter
-    is depending on the new set value. Therefore, all lines of the setter should be included
-    in the slice, but the initial setting of the class attribute is not included."""
+def test_getter_after_setter(setter_getter_test, subject_properties: SubjectProperties):
     module_name = "tests.fixtures.linecoverage.setter_getter"
     test_case_chromosome = tcc.TestCaseChromosome(test_case=setter_getter_test)
     config.configuration.statistics_output.coverage_metrics = [
         config.CoverageMetric.CHECKED,
     ]
 
-    tracer = ExecutionTracer()
-    tracer.current_thread_identifier = threading.current_thread().ident
+    with install_import_hook(module_name, subject_properties):
+        with subject_properties.instrumentation_tracer:
+            module = importlib.import_module(module_name)
+            importlib.reload(module)
 
-    with install_import_hook(module_name, tracer):
-        module = importlib.import_module(module_name)
-        importlib.reload(module)
-
-        executor = TestCaseExecutor(tracer)
-        executor.add_observer(StatementSlicingObserver(tracer))
+        executor = TestCaseExecutor(subject_properties)
+        executor.add_remote_observer(RemoteStatementSlicingObserver())
 
         ff = TestCaseStatementCheckedCoverageFunction(executor)
-        assert ff.compute_coverage(test_case_chromosome) == pytest.approx(
-            5 / 6, 0.1, 0.1
-        )
+        assert ff.compute_coverage(test_case_chromosome) == pytest.approx(5 / 6, 0.1, 0.1)
 
 
 def test_get_line_id_by_instruction_throws_error():
@@ -324,7 +315,5 @@ def test_get_line_id_by_instruction_throws_error():
         }
     )
 
-    with pytest.raises(ValueError):
-        DynamicSlicer.get_line_id_by_instruction(
-            instruction_mock, subject_properties_mock
-        )
+    with pytest.raises(ValueError):  # noqa: PT011
+        DynamicSlicer.get_line_id_by_instruction(instruction_mock, subject_properties_mock)
