@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -8,6 +8,7 @@ import argparse
 import importlib
 import logging
 import os
+import re
 
 from pathlib import Path
 from unittest import mock
@@ -18,16 +19,16 @@ import pytest
 
 import pynguin.configuration as config
 
-from pynguin.cli import _DANGER_ENV
-from pynguin.cli import _create_argument_parser
-from pynguin.cli import _expand_arguments_if_necessary
-from pynguin.cli import _setup_logging
+from pynguin.cli import _DANGER_ENV  # noqa: PLC2701
+from pynguin.cli import _create_argument_parser  # noqa: PLC2701
+from pynguin.cli import _expand_arguments_if_necessary  # noqa: PLC2701
+from pynguin.cli import _setup_logging  # noqa: PLC2701
 from pynguin.cli import main
 from pynguin.generator import ReturnCode
 
 
 def test_main_empty_argv():
-    with mock.patch("pynguin.cli.run_pynguin") as generator_mock:
+    with mock.patch("pynguin.cli.run_pynguin") as generator_mock:  # noqa: SIM117
         with mock.patch("pynguin.cli._create_argument_parser") as parser_mock:
             with mock.patch("pynguin.cli._setup_logging"):
                 with mock.patch("pynguin.cli._setup_output_path"):
@@ -40,7 +41,7 @@ def test_main_empty_argv():
 
 
 def test_main_with_argv():
-    with mock.patch("pynguin.cli.run_pynguin") as generator_mock:
+    with mock.patch("pynguin.cli.run_pynguin") as generator_mock:  # noqa: SIM117
         with mock.patch("pynguin.cli._create_argument_parser") as parser_mock:
             with mock.patch("pynguin.cli._setup_logging"):
                 with mock.patch("pynguin.cli._setup_output_path"):
@@ -66,7 +67,7 @@ def test__create_argument_parser():
 def test__setup_logging_single_verbose_without_log_file():
     logging.shutdown()
     importlib.reload(logging)
-    _setup_logging(1, False)
+    _setup_logging(verbosity=1, no_rich=False, log_file=None)
     logger = logging.getLogger("")
     assert len(logger.handlers) == 1
     assert logger.level == logging.INFO
@@ -77,7 +78,7 @@ def test__setup_logging_single_verbose_without_log_file():
 def test__setup_logging_double_verbose_without_log_file():
     logging.shutdown()
     importlib.reload(logging)
-    _setup_logging(2, False)
+    _setup_logging(verbosity=2, no_rich=False, log_file=None)
     logger = logging.getLogger("")
     assert len(logger.handlers) == 1
     assert logger.level == logging.DEBUG
@@ -85,12 +86,27 @@ def test__setup_logging_double_verbose_without_log_file():
     importlib.reload(logging)
 
 
+def test__setup_logging_log_file(tmp_path: Path):
+    log_file = tmp_path / "pynguin.log"
+    logging.shutdown()
+    importlib.reload(logging)
+    _setup_logging(verbosity=1, no_rich=False, log_file=log_file)
+    logger = logging.getLogger("")
+    assert len(logger.handlers) == 1
+    logger.info("Test entry")
+    assert re.match(
+        r"\[[0-9]{2}:[0-9]{2}:[0-9]{2}\]\s\[INFO\]\(.+:"
+        r"test__setup_logging_log_file:[0-9]+\):\sTest\sentry",
+        log_file.read_text(),
+    )
+    logging.shutdown()
+    importlib.reload(logging)
+
+
 @pytest.mark.parametrize(
     "arguments, expected",
     [
-        pytest.param(
-            ["--foo", "bar", "--bar", "foo"], ["--foo", "bar", "--bar", "foo"]
-        ),
+        pytest.param(["--foo", "bar", "--bar", "foo"], ["--foo", "bar", "--bar", "foo"]),
         pytest.param(
             ["--foo", "bar", "--output_variables", "foo,bar,baz", "--bar", "foo"],
             ["--foo", "bar", "--output_variables", "foo", "bar", "baz", "--bar", "foo"],
@@ -235,24 +251,22 @@ def test__expand_arguments_if_necessary(arguments, expected):
 
 
 def test_load_configuration_from_file(tmp_path):
-    config_file = Path(".").absolute()
+    config_file = Path().absolute()
     if config_file.name != "tests":
         config_file /= "tests"  # pragma: no cover
     config_file = config_file / "fixtures" / "test.conf"
     parser = _create_argument_parser()
-    parsed = parser.parse_args(
-        [
-            f"@{config_file}",
-            "--module_name",
-            "hurz",
-            "--project_path",
-            str(tmp_path),
-            "--output_path",
-            str(tmp_path),
-            "--maximum_search_time",
-            "50",
-        ]
-    )
+    parsed = parser.parse_args([
+        f"@{config_file}",
+        "--module_name",
+        "hurz",
+        "--project_path",
+        str(tmp_path),
+        "--output_path",
+        str(tmp_path),
+        "--maximum_search_time",
+        "50",
+    ])
     configuration = parsed.config
     expected = config.Configuration(
         algorithm=config.Algorithm.MOSA,
@@ -264,3 +278,43 @@ def test_load_configuration_from_file(tmp_path):
     expected.stopping.maximum_search_time = 50
     expected.statistics_output.configuration_id = "merge checker"
     assert configuration == expected
+
+
+@pytest.mark.parametrize(
+    "input_value, expected_bool, expected_float",
+    [
+        ("True", True, 1),
+        ("False", False, 0),
+        ("0", False, 0),
+        ("1", True, 1),
+    ],
+)
+def test_parse_arguments_type_tracing_bool(tmp_path, input_value, expected_bool, expected_float):
+    parser = _create_argument_parser()
+    parsed = parser.parse_args([
+        "--module_name",
+        "hurz",
+        "--project_path",
+        str(tmp_path),
+        "--output_path",
+        str(tmp_path),
+        "--type-tracing",
+        input_value,
+    ])
+    assert parsed.config.type_inference.type_tracing == expected_bool
+    assert parsed.config.type_inference.type_tracing == expected_float
+
+
+def test_parse_arguments_type_tracing_float(tmp_path):
+    parser = _create_argument_parser()
+    parsed = parser.parse_args([
+        "--module_name",
+        "hurz",
+        "--project_path",
+        str(tmp_path),
+        "--output_path",
+        str(tmp_path),
+        "--type-tracing",
+        "0.1",
+    ])
+    assert parsed.config.type_inference.type_tracing == 0.1
