@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -16,11 +16,7 @@ def test_instruction_equal():
     arg = None
     lineno = 42
     offset = 42
-    instr1 = ExecutedInstruction(
-        module, code_object_id, node_id, opcode, arg, lineno, offset
-    )
-    instr2 = ExecutedInstruction(
-        module, code_object_id, node_id, opcode, arg, lineno, offset
-    )
+    instr1 = ExecutedInstruction(module, code_object_id, node_id, opcode, arg, lineno, offset)
+    instr2 = ExecutedInstruction(module, code_object_id, node_id, opcode, arg, lineno, offset)
 
     assert instr1 == instr2
