@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -32,28 +32,30 @@ def test_rhs(test_case_mock, variable_reference_mock):
 
 def test_structural_hash(assignment_statement):
     assert (
-        assignment_statement.structural_hash(
-            {assignment_statement.lhs.source: 0, assignment_statement.rhs: 1}
-        )
+        assignment_statement.structural_hash({
+            assignment_statement.lhs.source: 0,
+            assignment_statement.rhs: 1,
+        })
         != 0
     )
 
 
 def test_structural_hash_same(assignment_statement):
     memo = {assignment_statement.lhs.source: 0, assignment_statement.rhs: 1}
-    assert assignment_statement.structural_hash(
-        memo
-    ) == assignment_statement.structural_hash(memo)
+    assert assignment_statement.structural_hash(memo) == assignment_statement.structural_hash(memo)
 
 
 def test_structural_eq_same(assignment_statement):
+    # fmt: off
     assert assignment_statement.structural_eq(
         assignment_statement,
         {
-            assignment_statement.lhs.get_variable_reference(): assignment_statement.lhs.get_variable_reference(),
+            assignment_statement.lhs.get_variable_reference(): assignment_statement.lhs
+            .get_variable_reference(),
             assignment_statement.rhs: assignment_statement.rhs,
         },
     )
+    # fmt: on
 
 
 def test_structural_eq_other_type(test_case_mock, variable_reference_mock):
@@ -98,11 +100,11 @@ def test_accessible_object(assignment_statement):
 
 
 def test_mutate(assignment_statement):
-    with pytest.raises(Exception):
+    with pytest.raises(Exception):  # noqa: B017, PT011
         assignment_statement.mutate()
 
 
-def test_get_variable_references(test_case_mock, assignment_statement):
+def test_get_variable_references(assignment_statement):
     result = assignment_statement.get_variable_references()
     assert result == {
         assignment_statement.lhs.get_variable_reference(),
