@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -49,7 +49,7 @@ BENCHMARK_REPETITIONS = 2
 
 def test_benchmark_eq(benchmark_test_case):
     cloned = benchmark_test_case.clone()
-    res = all([benchmark_test_case == cloned for _ in range(BENCHMARK_REPETITIONS)])
+    res = all(benchmark_test_case == cloned for _ in range(BENCHMARK_REPETITIONS))
     assert res
 
 
@@ -59,6 +59,6 @@ def test_benchmark_hash(benchmark_test_case):
 
 def test_benchmark_clone(benchmark_test_case):
     cloned = benchmark_test_case.clone()
-    for i in range(BENCHMARK_REPETITIONS):
+    for _i in range(BENCHMARK_REPETITIONS):
         cloned = cloned.clone()
     assert cloned == benchmark_test_case
