@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -23,26 +23,22 @@ from pynguin.utils.generic.genericaccessibleobject import GenericMethod
 from pynguin.utils.namingscope import NamingScope
 
 
-@pytest.fixture()
+@pytest.fixture
 def statement_to_ast_visitor() -> stmt_to_ast.StatementToAstVisitor:
     var_names = NamingScope()
     module_aliases = NamingScope(prefix="module")
     return stmt_to_ast.StatementToAstVisitor(module_aliases, var_names)
 
 
-@pytest.fixture()
+@pytest.fixture
 def statement_to_ast_visitor_no_store() -> stmt_to_ast.StatementToAstVisitor:
     var_names = NamingScope()
     module_aliases = NamingScope(prefix="module")
-    return stmt_to_ast.StatementToAstVisitor(
-        module_aliases, var_names, store_call_return=False
-    )
+    return stmt_to_ast.StatementToAstVisitor(module_aliases, var_names, store_call_return=False)
 
 
 def __create_source_from_ast(module_body: ast.stmt) -> str:
-    return ast.unparse(
-        ast.fix_missing_locations(ast.Module(body=[module_body], type_ignores=[]))
-    )
+    return ast.unparse(ast.fix_missing_locations(ast.Module(body=[module_body], type_ignores=[])))
 
 
 def test_statement_to_ast_int(statement_to_ast_visitor, default_test_case):
@@ -60,23 +56,18 @@ def test_statement_to_ast_float(statement_to_ast_visitor, default_test_case):
 def test_statement_to_ast_str(statement_to_ast_visitor, default_test_case):
     str_stmt = stmt.StringPrimitiveStatement(default_test_case, "TestMe")
     statement_to_ast_visitor.visit_string_primitive_statement(str_stmt)
-    assert (
-        __create_source_from_ast(statement_to_ast_visitor.ast_node)
-        == "var_0 = 'TestMe'"
-    )
+    assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = 'TestMe'"
 
 
 def test_statement_to_ast_bytes(statement_to_ast_visitor, default_test_case):
     bytes_stmt = stmt.BytesPrimitiveStatement(default_test_case, b"TestMe")
     statement_to_ast_visitor.visit_bytes_primitive_statement(bytes_stmt)
-    assert (
-        __create_source_from_ast(statement_to_ast_visitor.ast_node)
-        == "var_0 = b'TestMe'"
-    )
+    assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = b'TestMe'"
 
 
 def test_statement_to_ast_bool(statement_to_ast_visitor, default_test_case):
-    bool_stmt = stmt.BooleanPrimitiveStatement(default_test_case, True)
+    bool_stmt = stmt.BooleanPrimitiveStatement(default_test_case, True)  # noqa: FBT003
+
     statement_to_ast_visitor.visit_boolean_primitive_statement(bool_stmt)
     assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = True"
 
@@ -84,10 +75,7 @@ def test_statement_to_ast_bool(statement_to_ast_visitor, default_test_case):
 def test_statement_to_ast_class(statement_to_ast_visitor, default_test_case):
     class_stmt = stmt.ClassPrimitiveStatement(default_test_case, 0)
     statement_to_ast_visitor.visit_class_primitive_statement(class_stmt)
-    assert (
-        __create_source_from_ast(statement_to_ast_visitor.ast_node)
-        == "var_0 = module_0.int"
-    )
+    assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = module_0.int"
 
 
 def test_statement_to_ast_none(statement_to_ast_visitor, default_test_case):
@@ -112,9 +100,7 @@ def test_statement_to_ast_enum(statement_to_ast_visitor, default_test_case):
     )
 
 
-def test_statement_to_ast_assignment(
-    variable_reference_mock, statement_to_ast_visitor, default_test_case
-):
+def test_statement_to_ast_assignment(statement_to_ast_visitor, default_test_case):
     string = stmt.StringPrimitiveStatement(default_test_case, "foo")
     field = gao.GenericField(
         default_test_case.test_cluster.type_system.to_type_info(str),
@@ -126,10 +112,7 @@ def test_statement_to_ast_assignment(
         default_test_case, vr.FieldReference(string.ret_val, field), int_0.ret_val
     )
     statement_to_ast_visitor.visit_assignment_statement(assign_stmt)
-    assert (
-        __create_source_from_ast(statement_to_ast_visitor.ast_node)
-        == "var_0.foo = var_1"
-    )
+    assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0.foo = var_1"
 
 
 def test_statement_to_ast_field(statement_to_ast_visitor, default_test_case):
@@ -141,10 +124,7 @@ def test_statement_to_ast_field(statement_to_ast_visitor, default_test_case):
     )
     field_stmt = stmt.FieldStatement(default_test_case, field, string.ret_val)
     statement_to_ast_visitor.visit_field_statement(field_stmt)
-    assert (
-        __create_source_from_ast(statement_to_ast_visitor.ast_node)
-        == "var_0 = var_1.foo"
-    )
+    assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = var_1.foo"
 
 
 def all_param_types_signature(type_system):
@@ -256,7 +236,25 @@ def no_default_args_signature(type_system):
     )
 
 
-@pytest.fixture()
+def no_types_signature(type_system):
+    return InferredSignature(
+        signature=inspect.Signature(
+            parameters=[
+                inspect.Parameter(
+                    name="x",
+                    kind=inspect.Parameter.POSITIONAL_OR_KEYWORD,
+                ),
+            ]
+        ),
+        original_return_type=type_system.convert_type_hint(int),
+        original_parameters={
+            "x": type_system.convert_type_hint(int),
+        },
+        type_system=type_system,
+    )
+
+
+@pytest.fixture
 def all_types_constructor(type_system):
     return GenericConstructor(
         owner=type_system.to_type_info(MagicMock),
@@ -264,7 +262,7 @@ def all_types_constructor(type_system):
     )
 
 
-@pytest.fixture()
+@pytest.fixture
 def all_types_method(type_system):
     return GenericMethod(
         owner=MagicMock(),
@@ -273,7 +271,7 @@ def all_types_method(type_system):
     )
 
 
-@pytest.fixture()
+@pytest.fixture
 def all_types_function(type_system):
     return GenericFunction(
         function=MagicMock(__name__="function"),
@@ -281,7 +279,7 @@ def all_types_function(type_system):
     )
 
 
-@pytest.fixture()
+@pytest.fixture
 def default_args_function(type_system):
     return GenericFunction(
         function=MagicMock(__name__="function"),
@@ -289,7 +287,7 @@ def default_args_function(type_system):
     )
 
 
-@pytest.fixture()
+@pytest.fixture
 def no_default_args_function(type_system):
     return GenericFunction(
         function=MagicMock(__name__="function"),
@@ -297,6 +295,14 @@ def no_default_args_function(type_system):
     )
 
 
+@pytest.fixture
+def lambda_function(type_system):
+    return GenericFunction(
+        function=MagicMock(__name__="lambda"),
+        inferred_signature=no_types_signature(type_system),
+    )
+
+
 @pytest.mark.parametrize(
     "args,expected",
     [
@@ -336,12 +342,8 @@ def no_default_args_function(type_system):
 def test_statement_to_ast_constructor_args(
     statement_to_ast_visitor, default_test_case, all_types_constructor, args, expected
 ):
-    args_stmts = {
-        a: stmt.IntPrimitiveStatement(default_test_case, 3).ret_val for a in args
-    }
-    constr_stmt = stmt.ConstructorStatement(
-        default_test_case, all_types_constructor, args_stmts
-    )
+    args_stmts = {a: stmt.IntPrimitiveStatement(default_test_case, 3).ret_val for a in args}
+    constr_stmt = stmt.ConstructorStatement(default_test_case, all_types_constructor, args_stmts)
     statement_to_ast_visitor.visit_constructor_statement(constr_stmt)
     assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == expected
 
@@ -416,10 +418,7 @@ def test_statement_to_ast_method_no_store(
         {},
     )
     statement_to_ast_visitor_no_store.visit_method_statement(method_stmt)
-    assert (
-        __create_source_from_ast(statement_to_ast_visitor_no_store.ast_node)
-        == "var_0.method()"
-    )
+    assert __create_source_from_ast(statement_to_ast_visitor_no_store.ast_node) == "var_0.method()"
 
 
 @pytest.mark.parametrize(
@@ -553,23 +552,30 @@ def test_statement_to_ast_function_no_store(
     )
 
 
-def test_statement_to_ast_list_single(
-    statement_to_ast_visitor, default_test_case, function_mock
+def test_statement_to_ast_function_lambda(
+    statement_to_ast_visitor, test_case_mock, lambda_function
 ):
+    func_stmt = stmt.FunctionStatement(
+        test_case_mock, lambda_function, {"x": stmt.IntPrimitiveStatement(MagicMock(), 3).ret_val}
+    )
+    statement_to_ast_visitor.visit_function_statement(func_stmt)
+    assert (
+        __create_source_from_ast(statement_to_ast_visitor.ast_node)
+        == "var_1 = module_0.lambda(var_0)"
+    )
+
+
+def test_statement_to_ast_list_single(statement_to_ast_visitor, default_test_case):
     list_stmt = stmt.ListStatement(
         default_test_case,
         default_test_case.test_cluster.type_system.convert_type_hint(list[int]),
         [stmt.IntPrimitiveStatement(default_test_case, 5).ret_val],
     )
     statement_to_ast_visitor.visit_list_statement(list_stmt)
-    assert (
-        __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = [var_1]"
-    )
+    assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = [var_1]"
 
 
-def test_statement_to_ast_list_empty(
-    statement_to_ast_visitor, default_test_case, function_mock
-):
+def test_statement_to_ast_list_empty(statement_to_ast_visitor, default_test_case):
     list_stmt = stmt.ListStatement(
         default_test_case,
         default_test_case.test_cluster.type_system.convert_type_hint(list[int]),
@@ -579,52 +585,37 @@ def test_statement_to_ast_list_empty(
     assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = []"
 
 
-def test_statement_to_ast_set_single(
-    statement_to_ast_visitor, default_test_case, function_mock
-):
+def test_statement_to_ast_set_single(statement_to_ast_visitor, default_test_case):
     set_stmt = stmt.SetStatement(
         default_test_case,
         default_test_case.test_cluster.type_system.convert_type_hint(set[int]),
         [stmt.IntPrimitiveStatement(default_test_case, 5).ret_val],
     )
     statement_to_ast_visitor.visit_set_statement(set_stmt)
-    assert (
-        __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_1 = {var_0}"
-    )
+    assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_1 = {var_0}"
 
 
-def test_statement_to_ast_set_empty(
-    statement_to_ast_visitor, default_test_case, function_mock
-):
+def test_statement_to_ast_set_empty(statement_to_ast_visitor, default_test_case):
     set_stmt = stmt.SetStatement(
         default_test_case,
         default_test_case.test_cluster.type_system.convert_type_hint(set[int]),
         [],
     )
     statement_to_ast_visitor.visit_set_statement(set_stmt)
-    assert (
-        __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = set()"
-    )
+    assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = set()"
 
 
-def test_statement_to_ast_tuple_single(
-    statement_to_ast_visitor, default_test_case, function_mock
-):
+def test_statement_to_ast_tuple_single(statement_to_ast_visitor, default_test_case):
     tuple_stmt = stmt.TupleStatement(
         default_test_case,
         default_test_case.test_cluster.type_system.convert_type_hint(tuple[int]),
         [stmt.IntPrimitiveStatement(default_test_case, 5).ret_val],
     )
     statement_to_ast_visitor.visit_tuple_statement(tuple_stmt)
-    assert (
-        __create_source_from_ast(statement_to_ast_visitor.ast_node)
-        == "var_0 = (var_1,)"
-    )
+    assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = (var_1,)"
 
 
-def test_statement_to_ast_tuple_empty(
-    statement_to_ast_visitor, default_test_case, function_mock
-):
+def test_statement_to_ast_tuple_empty(statement_to_ast_visitor, default_test_case):
     tuple_stmt = stmt.TupleStatement(
         default_test_case,
         default_test_case.test_cluster.type_system.convert_type_hint(tuple),
@@ -634,9 +625,7 @@ def test_statement_to_ast_tuple_empty(
     assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = ()"
 
 
-def test_statement_to_ast_dict_single(
-    statement_to_ast_visitor, default_test_case, function_mock
-):
+def test_statement_to_ast_dict_single(statement_to_ast_visitor, default_test_case):
     dict_stmt = stmt.DictStatement(
         default_test_case,
         default_test_case.test_cluster.type_system.convert_type_hint(dict[int, int]),
@@ -648,15 +637,10 @@ def test_statement_to_ast_dict_single(
         ],
     )
     statement_to_ast_visitor.visit_dict_statement(dict_stmt)
-    assert (
-        __create_source_from_ast(statement_to_ast_visitor.ast_node)
-        == "var_0 = {var_1: var_2}"
-    )
+    assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = {var_1: var_2}"
 
 
-def test_statement_to_ast_dict_empty(
-    statement_to_ast_visitor, default_test_case, function_mock
-):
+def test_statement_to_ast_dict_empty(statement_to_ast_visitor, default_test_case):
     dict_stmt = stmt.DictStatement(
         default_test_case,
         default_test_case.test_cluster.type_system.convert_type_hint(dict[int, int]),
@@ -664,3 +648,19 @@ def test_statement_to_ast_dict_empty(
     )
     statement_to_ast_visitor.visit_dict_statement(dict_stmt)
     assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = {}"
+
+
+def test_statement_to_ast_ast_assign(statement_to_ast_visitor, default_test_case):
+    """Test the visit_ast_assign_statement method."""
+    # Create a simple AST node (constant)
+    ast_node = ast.Constant(value=42)
+    ref_dict = {}
+
+    # Create the ASTAssignStatement
+    ast_assign_stmt = stmt.ASTAssignStatement(default_test_case, ast_node, ref_dict)
+
+    # Visit the statement
+    statement_to_ast_visitor.visit_ast_assign_statement(ast_assign_stmt)
+
+    # Verify the generated AST node
+    assert __create_source_from_ast(statement_to_ast_visitor.ast_node) == "var_0 = 42"
