@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -29,9 +29,7 @@ class DummyReference(vr.Reference):
     def clone(self, memo: dict[VariableReference, VariableReference]) -> Reference:
         pass  # pragma: no cover
 
-    def structural_eq(
-        self, other: Any, memo: dict[VariableReference, VariableReference]
-    ) -> bool:
+    def structural_eq(self, other: Any, memo: dict[VariableReference, VariableReference]) -> bool:
         pass  # pragma: no cover
 
     def structural_hash(self, memo: dict[VariableReference, int]) -> int:
@@ -40,19 +38,17 @@ class DummyReference(vr.Reference):
     def get_variable_reference(self) -> VariableReference | None:
         pass  # pragma: no cover
 
-    def replace_variable_reference(
-        self, old: VariableReference, new: VariableReference
-    ) -> None:
+    def replace_variable_reference(self, old: VariableReference, new: VariableReference) -> None:
         pass  # pragma: no cover
 
 
-@pytest.fixture()
+@pytest.fixture
 def dummy_reference():
     return DummyReference(int)
 
 
 def test_type(dummy_reference):
-    assert dummy_reference.type == int
+    assert dummy_reference.type is int
 
 
 @pytest.mark.parametrize(
@@ -99,7 +95,7 @@ def test_var_get_position(test_case_mock):
 def test_var_get_position_no_statements(test_case_mock):
     ref = vr.VariableReference(test_case_mock, int)
     test_case_mock.statements = []
-    with pytest.raises(Exception):
+    with pytest.raises(Exception):  # noqa: B017, PT011
         ref.get_statement_position()
 
 
@@ -125,7 +121,7 @@ def test_var_distance(test_case_mock):
     assert ref.distance == 42
 
 
-@pytest.fixture()
+@pytest.fixture
 def field_mock():
     return gao.GenericField(MagicMock, "foo", int)
 
@@ -213,7 +209,7 @@ def test_field_replace_var(field_mock):
     assert ref.source == var_2
 
 
-@pytest.fixture()
+@pytest.fixture
 def static_field_mock(type_system):
     return gao.GenericStaticField(
         type_system.to_type_info(MagicMock), "foo", type_system.convert_type_hint(int)
@@ -269,7 +265,7 @@ def test_static_field_replace_var(static_field_mock):
     assert ref.get_variable_reference() is None
 
 
-@pytest.fixture()
+@pytest.fixture
 def static_module_field_mock():
     return gao.GenericStaticModuleField("foomod", "foo", int)
 
@@ -301,9 +297,7 @@ def test_module_field_eq(static_module_field_mock):
 
 def test_module_field_eq_2(static_module_field_mock):
     ref = vr.StaticModuleFieldReference(static_module_field_mock)
-    ref_2 = vr.StaticModuleFieldReference(
-        gao.GenericStaticModuleField("moo", "bar", int)
-    )
+    ref_2 = vr.StaticModuleFieldReference(gao.GenericStaticModuleField("moo", "bar", int))
     assert ref != ref_2
 
 
