@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
@@ -15,12 +15,12 @@ import pynguin.testcase.variablereference as vr
 import pynguin.utils.namingscope as ns
 
 
-@pytest.fixture()
+@pytest.fixture
 def naming_scope():
     return ns.NamingScope()
 
 
-@pytest.fixture()
+@pytest.fixture
 def variable_type_naming_scope():
     return ns.VariableTypeNamingScope()
 
@@ -63,7 +63,7 @@ def test_naming_scope_known_indices_has_name(naming_scope):
         (dict[int, str], "dict_0"),
         (list[str], "list_0"),
         (list, "list_0"),
-        (Union[int, str, bool], "var_0"),  # For a union we get var0
+        (Union[int, str, bool], "var_0"),  # For a union we get var0  # noqa: UP007
         (Iterable[int], "iterable_0"),
         (MagicMock, "magic_mock_0"),
     ],
@@ -82,9 +82,7 @@ def test_variable_type_conversion(variable_type_naming_scope, tp, name, type_sys
         (dict[int, str], "dict_0", "dict_1"),
     ],
 )
-def test_variable_type_counter(
-    variable_type_naming_scope, tp, name0, name1, type_system
-):
+def test_variable_type_counter(variable_type_naming_scope, tp, name0, name1, type_system):
     var = vr.VariableReference(MagicMock(), type_system.convert_type_hint(tp))
     assert variable_type_naming_scope.get_name(var) == name0
     var = vr.VariableReference(MagicMock(), type_system.convert_type_hint(tp))
@@ -92,9 +90,7 @@ def test_variable_type_counter(
 
 
 def test_variable_type_runtime(type_system):
-    scope = ns.VariableTypeNamingScope(
-        return_type_trace={0: type_system.convert_type_hint(int)}
-    )
+    scope = ns.VariableTypeNamingScope(return_type_trace={0: type_system.convert_type_hint(int)})
     var = MagicMock(type=type_system.convert_type_hint(None))
     var.get_statement_position.return_value = 1
     assert scope.get_name(var) == "var_0"
