@@ -1,10 +1,11 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2023 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2025 Pynguin Contributors
 #
 #  SPDX-License-Identifier: MIT
 #
 import inspect
+import math
 import operator
 
 from builtins import isinstance as real_isinstance
@@ -19,14 +20,14 @@ from pynguin.utils.orderedset import OrderedSet
 
 def test_type_tracing_max_depth():
     proxy = tt.ObjectProxy(MagicMock())
-    for i in range(tt._MAX_PROXY_NESTING):
+    for _i in range(tt._MAX_PROXY_NESTING):
         proxy = proxy["foo"]
     assert isinstance(proxy, tt.ObjectProxy)
 
 
 def test_type_tracing_max_depth_after():
     proxy = tt.ObjectProxy(MagicMock())
-    for i in range(tt._MAX_PROXY_NESTING + 1):
+    for _i in range(tt._MAX_PROXY_NESTING + 1):
         proxy = proxy["foo"]
     assert not isinstance(proxy, tt.ObjectProxy)
 
@@ -35,7 +36,7 @@ def test_type_tracing_max_depth_get_attr():
     mock = MagicMock()
     mock.foo = mock
     proxy = tt.ObjectProxy(mock)
-    for i in range(tt._MAX_PROXY_NESTING):
+    for _i in range(tt._MAX_PROXY_NESTING):
         proxy = proxy.foo
     assert isinstance(proxy, tt.ObjectProxy)
 
@@ -44,7 +45,7 @@ def test_type_tracing_max_depth_after_get_attr():
     mock = MagicMock()
     mock.foo = mock
     proxy = tt.ObjectProxy(mock)
-    for i in range(tt._MAX_PROXY_NESTING + 1):
+    for _i in range(tt._MAX_PROXY_NESTING + 1):
         proxy = proxy.foo
     assert not isinstance(proxy, tt.ObjectProxy)
 
@@ -52,7 +53,7 @@ def test_type_tracing_max_depth_after_get_attr():
 def test_type_tracing_max_depth_iter():
     mock = [[[[[[[MagicMock()]]]]]]]
     proxy = tt.ObjectProxy(mock)
-    for i in range(tt._MAX_PROXY_NESTING):
+    for _i in range(tt._MAX_PROXY_NESTING):
         proxy = next(iter(proxy))
     assert isinstance(proxy, tt.ObjectProxy)
 
@@ -60,7 +61,7 @@ def test_type_tracing_max_depth_iter():
 def test_type_tracing_max_depth_after_iter():
     mock = [[[[[[[MagicMock()]]]]]]]
     proxy = tt.ObjectProxy(mock)
-    for i in range(tt._MAX_PROXY_NESTING + 1):
+    for _i in range(tt._MAX_PROXY_NESTING + 1):
         proxy = next(iter(proxy))
     assert not isinstance(proxy, tt.ObjectProxy)
 
@@ -84,16 +85,13 @@ def test_method_called():
     proxy = tt.ObjectProxy("foo")
     assert proxy.startswith("fo")
     assert "startswith" in tt.UsageTraceNode.from_proxy(proxy).children
-    assert (
-        "__call__"
-        in tt.UsageTraceNode.from_proxy(proxy).children["startswith"].children
-    )
+    assert "__call__" in tt.UsageTraceNode.from_proxy(proxy).children["startswith"].children
 
 
 def test_loop_over_list():
     proxy = tt.ObjectProxy(["foo", "bar"])
     with tt.shim_isinstance():
-        for i, element in enumerate(proxy):
+        for element in proxy:
             assert isinstance(element, str)
     assert str in tt.UsageTraceNode.from_proxy(proxy).children["__iter__"].type_checks
 
@@ -120,18 +118,16 @@ def test_dont_record_objectproxy_instance_check_2():
 
 def test_dont_record_objectproxy_instance_check_3():
     proxy = tt.ObjectProxy(42)
-    with tt.shim_isinstance():
-        with pytest.raises(TypeError):
-            assert isinstance(proxy, tt.ObjectProxy(int))
+    with tt.shim_isinstance(), pytest.raises(TypeError):
+        assert isinstance(proxy, tt.ObjectProxy(int))
     assert inspect.isbuiltin(isinstance)
     assert len(tt.UsageTraceNode.from_proxy(proxy).type_checks) == 0
 
 
 def test_dont_record_objectproxy_instance_check_4():
     proxy = tt.ObjectProxy(42)
-    with tt.shim_isinstance():
-        with pytest.raises(TypeError):
-            assert isinstance(proxy, (tt.ObjectProxy(int), float))
+    with tt.shim_isinstance(), pytest.raises(TypeError):
+        assert isinstance(proxy, (tt.ObjectProxy(int), float))
     assert inspect.isbuiltin(isinstance)
     assert len(tt.UsageTraceNode.from_proxy(proxy).type_checks) == 0
 
@@ -211,9 +207,7 @@ def test_compares_op(op, name):
 def test_contains():
     proxy = tt.ObjectProxy([42])
     assert 42 in proxy
-    assert (
-        int in tt.UsageTraceNode.from_proxy(proxy).children["__contains__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__contains__"].arg_types[0]
 
 
 def test_len():
@@ -245,10 +239,7 @@ def test_contains_proxy():
     proxy = tt.ObjectProxy([42])
     proxy2 = tt.ObjectProxy(42)
     assert proxy2 in proxy
-    assert (
-        len(tt.UsageTraceNode.from_proxy(proxy).children["__contains__"].arg_types[0])
-        == 0
-    )
+    assert len(tt.UsageTraceNode.from_proxy(proxy).children["__contains__"].arg_types[0]) == 0
 
 
 def test_bytes():
@@ -286,18 +277,14 @@ def test_truediv():
     value = 7
     proxy = tt.ObjectProxy(value)
     assert value / 2 == proxy / 2
-    assert (
-        int in tt.UsageTraceNode.from_proxy(proxy).children["__truediv__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__truediv__"].arg_types[0]
 
 
 def test_floordiv():
     value = 7
     proxy = tt.ObjectProxy(value)
     assert value // 2 == proxy // 2
-    assert (
-        int in tt.UsageTraceNode.from_proxy(proxy).children["__floordiv__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__floordiv__"].arg_types[0]
 
 
 def test_mod():
@@ -311,9 +298,7 @@ def test_divmod():
     value = 7
     proxy = tt.ObjectProxy(value)
     assert divmod(value, 3) == divmod(proxy, 3)
-    assert (
-        int in tt.UsageTraceNode.from_proxy(proxy).children["__divmod__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__divmod__"].arg_types[0]
 
 
 def test_pow():
@@ -327,18 +312,14 @@ def test_lshift():
     value = 2
     proxy = tt.ObjectProxy(value)
     assert value << 3 == proxy << 3
-    assert (
-        int in tt.UsageTraceNode.from_proxy(proxy).children["__lshift__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__lshift__"].arg_types[0]
 
 
 def test_rshift():
     value = 2
     proxy = tt.ObjectProxy(value)
     assert value >> 3 == proxy >> 3
-    assert (
-        int in tt.UsageTraceNode.from_proxy(proxy).children["__rshift__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__rshift__"].arg_types[0]
 
 
 def test_and():
@@ -390,19 +371,14 @@ def test_rtruediv():
     value = 2
     proxy = tt.ObjectProxy(value)
     assert 7 / value == 7 / proxy
-    assert (
-        int in tt.UsageTraceNode.from_proxy(proxy).children["__rtruediv__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__rtruediv__"].arg_types[0]
 
 
 def test_rfloordiv():
     value = 2
     proxy = tt.ObjectProxy(value)
     assert 7 // value == 7 // proxy
-    assert (
-        int
-        in tt.UsageTraceNode.from_proxy(proxy).children["__rfloordiv__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__rfloordiv__"].arg_types[0]
 
 
 def test_rmod():
@@ -416,9 +392,7 @@ def test_rdivmod():
     value = 3
     proxy = tt.ObjectProxy(value)
     assert divmod(7, value) == divmod(7, proxy)
-    assert (
-        int in tt.UsageTraceNode.from_proxy(proxy).children["__rdivmod__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__rdivmod__"].arg_types[0]
 
 
 def test_rpow():
@@ -432,18 +406,14 @@ def test_rlshift():
     value = 2
     proxy = tt.ObjectProxy(value)
     assert 3 << value == 3 << proxy
-    assert (
-        int in tt.UsageTraceNode.from_proxy(proxy).children["__rlshift__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__rlshift__"].arg_types[0]
 
 
 def test_rrshift():
     value = 2
     proxy = tt.ObjectProxy(value)
     assert 3 >> value == 3 >> proxy
-    assert (
-        int in tt.UsageTraceNode.from_proxy(proxy).children["__rrshift__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__rrshift__"].arg_types[0]
 
 
 def test_rand():
@@ -503,9 +473,7 @@ def test_itruediv():
     value /= 2
     proxy /= 2
     assert value == proxy
-    assert (
-        int in tt.UsageTraceNode.from_proxy(copy).children["__itruediv__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(copy).children["__itruediv__"].arg_types[0]
 
 
 def test_ifloordiv():
@@ -514,9 +482,7 @@ def test_ifloordiv():
     value //= 3
     proxy //= 3
     assert value == proxy
-    assert (
-        int in tt.UsageTraceNode.from_proxy(copy).children["__ifloordiv__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(copy).children["__ifloordiv__"].arg_types[0]
 
 
 def test_imod():
@@ -543,9 +509,7 @@ def test_ilshift():
     value <<= 1
     proxy <<= 1
     assert value == proxy
-    assert (
-        int in tt.UsageTraceNode.from_proxy(copy).children["__ilshift__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(copy).children["__ilshift__"].arg_types[0]
 
 
 def test_irshift():
@@ -554,9 +518,7 @@ def test_irshift():
     value >>= 1
     proxy >>= 1
     assert value == proxy
-    assert (
-        int in tt.UsageTraceNode.from_proxy(copy).children["__irshift__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(copy).children["__irshift__"].arg_types[0]
 
 
 def test_iand():
@@ -587,21 +549,21 @@ def test_ior():
 
 
 def test_neg():
-    value = 3.1415
+    value = math.pi
     proxy = tt.ObjectProxy(value)
     assert -value == -proxy
     assert tt.UsageTraceNode.from_proxy(proxy).children["__neg__"]
 
 
 def test_pos():
-    value = -3.1415
+    value = -math.pi
     proxy = tt.ObjectProxy(value)
     assert +value == +proxy
     assert tt.UsageTraceNode.from_proxy(proxy).children["__pos__"]
 
 
 def test_abs():
-    value = -3.1415
+    value = -math.pi
     proxy = tt.ObjectProxy(value)
     assert abs(value) == abs(proxy)
     assert tt.UsageTraceNode.from_proxy(proxy).children["__abs__"]
@@ -616,7 +578,7 @@ def test_invert():
 
 @pytest.mark.parametrize("func", [round, int, float, complex])
 def test_various_num_methods(func):
-    value = 3.1415
+    value = math.pi
     proxy = tt.ObjectProxy(value)
     assert func(value) == func(proxy)
 
@@ -626,9 +588,7 @@ def test_getitem_list():
     element = proxy[0]
     assert element == "entry"
     assert isinstance(element, tt.ObjectProxy)
-    assert (
-        int in tt.UsageTraceNode.from_proxy(proxy).children["__getitem__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__getitem__"].arg_types[0]
 
 
 def test_getitem_list_slice():
@@ -636,10 +596,7 @@ def test_getitem_list_slice():
     element = proxy[1:]
     assert element == ["b"]
     assert isinstance(element, tt.ObjectProxy)
-    assert (
-        slice
-        in tt.UsageTraceNode.from_proxy(proxy).children["__getitem__"].arg_types[0]
-    )
+    assert slice in tt.UsageTraceNode.from_proxy(proxy).children["__getitem__"].arg_types[0]
 
 
 def test_getitem_dict():
@@ -647,9 +604,7 @@ def test_getitem_dict():
     element = proxy["foo"]
     assert element == "entry"
     assert isinstance(element, tt.ObjectProxy)
-    assert (
-        str in tt.UsageTraceNode.from_proxy(proxy).children["__getitem__"].arg_types[0]
-    )
+    assert str in tt.UsageTraceNode.from_proxy(proxy).children["__getitem__"].arg_types[0]
 
 
 def test_setitem_list():
@@ -658,9 +613,7 @@ def test_setitem_list():
     element = proxy[0]
     assert element == 42
     assert isinstance(element, tt.ObjectProxy)
-    assert (
-        int in tt.UsageTraceNode.from_proxy(proxy).children["__setitem__"].arg_types[0]
-    )
+    assert int in tt.UsageTraceNode.from_proxy(proxy).children["__setitem__"].arg_types[0]
 
 
 def test_unwrap():
@@ -677,10 +630,7 @@ def test_pretty():
     knowledge = tt.UsageTraceNode("ROOT")
     knowledge.children["foo"].arg_types[0].add(int)
     knowledge.type_checks.add(str)
-    assert (
-        knowledge.pretty()
-        == "['ROOT', type_checks: {str}]\n└──['foo', arg_types: {0: {int}}]"
-    )
+    assert knowledge.pretty() == "['ROOT', type_checks: {str}]\n└──['foo', arg_types: {0: {int}}]"
 
 
 def test_getattr():
@@ -692,9 +642,10 @@ def test_getattr():
 def test_has_path():
     proxy = tt.ObjectProxy([1])
     proxy.count(1)
-    count_call_knowledge = tt.UsageTraceNode.from_proxy(proxy).find_path(
-        ("count", "__call__")
-    )
+    count_call_knowledge = tt.UsageTraceNode.from_proxy(proxy).find_path((
+        "count",
+        "__call__",
+    ))
     assert count_call_knowledge.arg_types[0] == OrderedSet([int])
 
 
